---------------------------------------------------------------------------------------------------
-- UOC - ETIG - TFC Bases de dades relacionals	2013-2014 1r semestre
---------------------------------------------------------------------------------------------------
-- Base de Dades d'un Sistema Centralitzat de Control de Despesa Pblica dels Parlaments Europeus
---------------------------------------------------------------------------------------------------
-- Arxiu : 07_Procediments.sql      -- Autor    : Santi Alcayde Pagan   -- Aula: 1
-- Data  : 11/11/2013               -- Consultor: Manel Rella Ruiz      
---------------------------------------------------------------------------------------------------
-- Funci: Scripts de Creaci de procediments generals
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** 00. ALTA D'EXECUCI DE PROCEDIMENT 
---------------------------------------------------------------------------------------------------
-- Insereix el resultat de l'execuci d'un procediment en la taula LOG_PROCEDIMENT
-- Si no es passen tots els parmetres es registre aquest error en la mateixa taula
---------------------------------------------------------------------------------------------------
CREATE OR REPLACE 
PROCEDURE DCE.PRO_LOG_PROCEDIMENT (

    i_proces   IN LOG_PROCEDIMENT.PROCEDIMENT%TYPE, -- Nom del procediment
    i_entrada  IN LOG_PROCEDIMENT.ENTRADA%TYPE,     -- Parmetres d'entrada del procediment
    i_resultat IN LOG_PROCEDIMENT.RESULTAT%TYPE,    -- Resultat de l'execuci del procediment
    i_abans    IN LOG_PROCEDIMENT.ABANS%TYPE,       -- Valors abans d'executar el procediment
    i_despres  IN LOG_PROCEDIMENT.DESPRES%TYPE,     -- Valors desprs d'executar el procediment
    i_error    IN LOG_PROCEDIMENT.ES_ERROR%TYPE,    -- Indicador d'error en l'execuci  
    i_usuari   IN LOG_PROCEDIMENT.USUARI%TYPE,      -- Usuari que ha executat el procediment
    i_data     IN LOG_PROCEDIMENT.DATA_HORA%TYPE    -- Data i hora d'execuci del procediment
) IS
    -- variables pel tractament d'excepcions
  	exc_noProces      EXCEPTION; -- No s'ha indicat el parmetre i_proces
  	exc_noEntrada     EXCEPTION; -- No s'ha indicat el parmetre i_entrada
    exc_noResultat    EXCEPTION; -- No s'ha indicat el parmetre i_resultat
    exc_noError       EXCEPTION; -- No s'ha indicat el parmetre i_error
    exc_noUsuari      EXCEPTION; -- No s'ha indicat el parmetre i_usuari
    exc_noData        EXCEPTION; -- No s'ha indicat el parmetre i_data
    log_proces   VARCHAR2(60);
    log_entrada  VARCHAR2(1024);
    log_resultat VARCHAR2(256);
    log_abans    VARCHAR2(1024);
    log_despres  VARCHAR2(1024);
    log_error    CHAR(1);
    log_usuari   VARCHAR2(12);
    log_data     DATE;

BEGIN
  -- Inicialitzaci de les variables
  log_proces  := NVL(i_proces,'PROCEDIMENT NO PASSAT') || ' - PRO_LOG_PROCEDIMENT';
  log_entrada := NVL(i_entrada,'PARMETRES D''ENTRADA NO PASSATS');
  log_abans   := '';
  log_despres := '';
  log_error   := 'S';
  log_usuari  := NVL(i_usuari,USER);
  log_data    := NVL(i_data,SYSDATE);

  -- Comprovaci dels parmetres d'entrada
  IF (i_proces IS NULL OR i_proces = '') THEN RAISE exc_noProces; END IF;
  IF (i_entrada IS NULL OR i_entrada = '') THEN RAISE exc_noEntrada; END IF;
  IF (i_resultat IS NULL OR i_resultat = '') THEN RAISE exc_noResultat; END IF;
  IF (i_error IS NULL OR i_error = '') THEN RAISE exc_noError; END IF;
  IF (i_usuari IS NULL OR i_usuari = '') THEN RAISE exc_noUsuari; END IF;
  IF (i_data IS NULL OR i_data = '') THEN RAISE exc_noData; END IF;

  -- Inserci correcte del registre
  INSERT INTO DCE.LOG_PROCEDIMENT 
      (PROCEDIMENT, ENTRADA, RESULTAT, ABANS, DESPRES, ES_ERROR, USUARI, DATA_HORA)
      VALUES (i_proces, i_entrada, i_resultat, i_abans, i_despres, i_error, i_usuari, i_data);
    
  COMMIT;
    
  -- Control d'excepcions
  EXCEPTION
    
  WHEN exc_noProces THEN
  log_resultat := 'ERROR: NO S''HA INDICAT EL NOM DEL PROCEDIMENT';
  INSERT INTO DCE.LOG_PROCEDIMENT
  (PROCEDIMENT, ENTRADA, RESULTAT, ABANS, DESPRES, ES_ERROR, USUARI, DATA_HORA) VALUES 
  (log_proces, log_entrada, log_resultat, log_abans, log_despres, log_error, log_usuari, log_data);
  DBMS_OUTPUT.PUT_LINE(log_proces || ': ' || log_resultat);

  WHEN exc_noEntrada THEN
  log_resultat := 'ERROR: NO S''HA INDICAT ELS PARMETRES D''ENTRADA';
  INSERT INTO DCE.LOG_PROCEDIMENT 
  (PROCEDIMENT, ENTRADA, RESULTAT, ABANS, DESPRES, ES_ERROR, USUARI, DATA_HORA) VALUES 
  (log_proces, log_entrada, log_resultat, log_abans, log_despres, log_error, log_usuari, log_data);
  DBMS_OUTPUT.PUT_LINE(log_proces || ': ' || log_resultat);

  WHEN exc_noResultat THEN
  log_resultat := 'ERROR: NO S''HA INDICAT EL RESULTAT DE L''EXECUCI';
  INSERT INTO DCE.LOG_PROCEDIMENT 
  (PROCEDIMENT, ENTRADA, RESULTAT, ABANS, DESPRES, ES_ERROR, USUARI, DATA_HORA) VALUES 
  (log_proces, log_entrada, log_resultat, log_abans, log_despres, log_error, log_usuari, log_data);
  DBMS_OUTPUT.PUT_LINE(log_proces || ': ' || log_resultat);

  WHEN exc_noError THEN
  log_error := 'ERROR: NO S''HA INDICAT EL INDICADOR D''ERROR DE L''EXECUCI';
  INSERT INTO DCE.LOG_PROCEDIMENT 
  (PROCEDIMENT, ENTRADA, RESULTAT, ABANS, DESPRES, ES_ERROR, USUARI, DATA_HORA) VALUES 
  (log_proces, log_entrada, log_resultat, log_abans, log_despres, log_error, log_usuari, log_data);
  DBMS_OUTPUT.PUT_LINE(log_proces || ': ' || log_resultat);

  WHEN exc_noUsuari THEN
  log_resultat := 'ERROR: NO S''HA INDICAT EL USUARI DE L''EXECUCI';
  INSERT INTO DCE.LOG_PROCEDIMENT 
  (PROCEDIMENT, ENTRADA, RESULTAT, ABANS, DESPRES, ES_ERROR, USUARI, DATA_HORA) VALUES 
  (log_proces, log_entrada, log_resultat, log_abans, log_despres, log_error, log_usuari, log_data);
  DBMS_OUTPUT.PUT_LINE(log_proces || ': ' || log_resultat);

  WHEN exc_noData THEN
  log_resultat := 'ERROR: NO S''HA INDICAT LA DATA DE L''EXECUCI';
  INSERT INTO DCE.LOG_PROCEDIMENT 
  (PROCEDIMENT, ENTRADA, RESULTAT, ABANS, DESPRES, ES_ERROR, USUARI, DATA_HORA) VALUES 
  (log_proces, log_entrada, log_resultat, log_abans, log_despres, log_error, log_usuari, log_data);
  DBMS_OUTPUT.PUT_LINE(log_proces || ': ' || log_resultat);

  WHEN OTHERS THEN
  log_resultat := 'ERROR: ' || SQLERRM;
  INSERT INTO DCE.LOG_PROCEDIMENT 
  (PROCEDIMENT, ENTRADA, RESULTAT, ABANS, DESPRES, ES_ERROR, USUARI, DATA_HORA) VALUES 
  (log_proces, log_entrada, log_resultat, log_abans, log_despres, log_error, log_usuari, log_data);
  DBMS_OUTPUT.PUT_LINE(log_proces || ': ' || log_resultat);

END PRO_LOG_PROCEDIMENT;
/
COMMIT;  
---------------------------------------------------------------------------------------------------