---------------------------------------------------------------------------------------------------
-- UOC - ETIG - TFC Bases de dades relacionals	2013-2014 1r semestre
---------------------------------------------------------------------------------------------------
-- Base de Dades d'un Sistema Centralitzat de Control de Despesa Pblica dels Parlaments Europeus
---------------------------------------------------------------------------------------------------
-- Arxiu : 08_Paquet_EST.sql        -- Autor    : Santi Alcayde Pagan   -- Aula: 1
-- Data  : 27/11/2013               -- Consultor: Manel Rella Ruiz      
---------------------------------------------------------------------------------------------------
-- Funci: Scripts de Creaci del PAQUET D'ESTADSTIQUES
-- El resultat de l'exeuci dels procediments es guarda en la taula d'auditoria LOG_PROCEDIMENT
---------------------------------------------------------------------------------------------------

-- *** OBSERVACI SOBRE NOMENCLATURES *************************************************************
-- Els noms dels parmetes d'entrada (input) comencen per i_
-- Els noms dels parmetres de sortida (output) comencen per o_
-- Els noms de les variables pel registre (log) de l'execuci dels procediments comencen per log_
-- Els noms de les variables pel tractament de les excepcions comencen per exc_
-- Els noms de la resta de variables comencen per v_
-- Els noms de dels cursors comencen per C_
-- Els noms de les vistes comencen per VIS_
-- Les funcions comencen per F_
-- Els procedimens de carcter general comencen per PRO_
-- No s'han escrit els accents en les paraules
--*************************************************************************************************

---------------------------------------------------------------------------------------------------
-- ***********************   C A P  A L E R A    D E L    P A Q U E T   *********************** --
---------------------------------------------------------------------------------------------------

CREATE OR REPLACE PACKAGE DCE.EST IS

    -- Variables globals pel registre en la taula LOG_PROCEDIMENT
    log_proces   VARCHAR2(60);
    log_entrada  VARCHAR2(1024);
    log_resultat VARCHAR2(256);
    log_abans    VARCHAR2(1024);
    log_despres  VARCHAR2(1024);
    log_error    CHAR(1);
    log_usuari   VARCHAR2(12);
    log_data     DATE;
    
    -- Altres variables globals
    -- Data i hora i usuari
    v_quiquan    VARCHAR2(80)  := 'Efectuada el ' || 
        to_char(sysdate,'dd/mm/yyyy hh24:mi:ss') || ' per l''usuari ' || user;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_01_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un parlament: la suma de totes les despeses dels darrers 4 anys (incloent les despeses 
   generals dels parlaments i les despeses associades als parlamentaris).
   - Insereix la informaci actual en la taula E1_CAMBRA_4ANYS quan s'executa un 
     procediment ABM de despeses generals i personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria  
   - L'existncia de despeses de la cambra indicada   
   - Es computa la suma de les despeses a partir de l'any actual fins els tres anys anteriors.
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_01_INSERIR (
    i_cambra   IN  NUMBER,    -- Identificador bbdd de la cambra parlamentria
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_01_LLEGIR.
---------------------------------------------------------------------------------------------------
/* Donat un parlament: la suma de totes les despeses dels darrers 4 anys (incloent les despeses 
   generals dels parlaments i les despeses associades als parlamentaris).
   - Mostra la informaci actual de la taula E1_CAMBRA_4ANYS
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria  
   - L'existncia de dades estadstiques de la cambra indicada     
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_01_LLEGIR (
    i_cambra   IN  NUMBER,    -- Identificador bbdd de la cambra parlamentria
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_02_INSERIR. 
/* Donat un parlament i any: la diferencia en euros entre el parlamentari que ms despeses 
   associades t i el que menys. 
   - Insereix la informaci actual en la taula E2_DIFERENCIA_MEMBRES quan s'executa un 
     procediment ABM de despeses personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria  
   - L'existncia de despeses personals de la cambra i any indicats
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_02_INSERIR (
    i_cambra   IN  NUMBER,    -- Identificador bbdd de la cambra parlamentria
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_02_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un parlament i any: la diferencia en euros entre el parlamentari que ms despeses 
   associades t i el que menys. 
   - Mostra la informaci actual de la taula E2_DIFERENCIA_MEMBRES
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria 
   - L'existncia de dades estadstiques de la cambra i any indicats    
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_02_LLEGIR (
    i_cambra   IN  NUMBER,    -- Identificador bbdd de la cambra parlamentria
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_03_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el parlament que ha tingut un total de despeses ms alt 
   (incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).
   - Insereix la informaci actual de la taula E3_MAX_CAMBRA quan s'executa un procediment ABM 
     de despeses.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_03_INSERIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_03_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el parlament que ha tingut un total de despeses ms alt 
   (incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).
   - Mostra la informaci actual de la taula E3_MAX_CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_03_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_04_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un parlamentari: la despesa total que ha tingut els darrers 4 anys.
   - Insereix la informaci actual en la taula E4_MEMBRE_4ANYS quan s'executa un 
     procediment ABM de despeses personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia del/de la membre de cambra parlamentria
   - L'existncia de despeses del/de la membre indicat
   - Es computa la suma de les despeses a partir de l'any actual fins els tres anys anteriors.
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_04_INSERIR (
    i_membre   IN  NUMBER,    -- Identificador bbdd de membre de cambra parlamentria
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_04_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un parlamentari: la despesa total que ha tingut els darrers 4 anys.
   - Mostra la informaci actual de la taula E4_MEMBRE_4ANYS
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia del/de la membre de cambra parlamentria
   - L'existncia de dades estadstiques del membre de cambra indicat 
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_04_LLEGIR (
    i_membre   IN  NUMBER,    -- Identificador bbdd de membre de cambra parlamentria
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_05_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: la mitjana de despesa dun parlamentari durant aquell any 
   (tenint en compte tots els parlamentaris de tots els parlaments).
   - Insereix la informaci actual de la taula E5_MITJANA_MEMBRE quan s'executa un procediment ABM 
     de despeses.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses de l'any indicat  
   - La mitjana es calcula fent la divisi entre la suma de totes les despeses  i el nmero 
     de membres que les han fet.    
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_05_INSERIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_05_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: la mitjana de despesa dun parlamentari durant aquell any 
   (tenint en compte tots els parlamentaris de tots els parlaments).
   - Mostra la informaci actual de la taula E5_MITJANA_MEMBRE
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_05_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_06_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el parlamentari que ha tingut ms despesa associada i el que menys 
   (tenint en compte tots els parlaments).
   - Insereix la informaci actual en la taula E6_MES_MENYS_MEMBRE quan s'executa un procediment
     ABM de despeses personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses personals de l'any indicat   
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_06_INSERIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_06_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el parlamentari que ha tingut ms despesa associada i el que menys 
   (tenint en compte tots els parlaments).
   - Mostra la informaci actual de la taula E6_MES_MENYS_MEMBRE
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_06_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_07_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el nom del parlament amb una mitjana de despesa associada als seus 
   parlamentaris ms alta, i el valor daquesta mitjana de despesa.
   - Insereix la informaci actual en la taula E7_MITJANA_MAX_CAMBRA quan s'executa un procediment
     ABM de despeses personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses personals de l'any indicat   
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_07_INSERIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_07_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el nom del parlament amb una mitjana de despesa associada als seus 
   parlamentaris ms alta, i el valor daquesta mitjana de despesa.
   - Mostra la informaci actual de la taula E7_MITJANA_MAX_CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_07_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_08_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: la suma de totes les despeses de tots els parlaments (incloent les 
   despeses generals dels parlaments i les despeses associades als parlamentaris).
   - Insereix la informaci actual en la taula E8_TOTAL_CAMBRES quan s'executa un procediment
     ABM de despeses personals i generals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses generals i personals de l'any indicat   
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_08_INSERIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_08_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: la suma de totes les despeses de tots els parlaments (incloent les 
   despeses generals dels parlaments i les despeses associades als parlamentaris).
   - Mostra la informaci actual de la taula E8_TOTAL_CAMBRES
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_08_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
);

END;
/

---------------------------------------------------------------------------------------------------
-- ***************************     C O S    D E L    P A Q U E T      ************************** --
---------------------------------------------------------------------------------------------------

CREATE OR REPLACE PACKAGE BODY DCE.EST IS

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_01_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un parlament: la suma de totes les despeses dels darrers 4 anys (incloent les despeses 
   generals dels parlaments i les despeses associades als parlamentaris).
   - Insereix la informaci actual en la taula E1_CAMBRA_4ANYS quan s'executa un 
     procediment ABM de despeses generals i personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria  
   - L'existncia de despeses de la cambra indicada   
   - Es computa la suma de les despeses a partir de l'any actual fins els tres anys anteriors.
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_01_INSERIR (
    i_cambra   IN  NUMBER,    -- Identificador bbdd de la cambra parlamentria
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra   EXCEPTION; -- No s'ha indicat el parmetre i_cambra

    -- Variables per a la consulta
    v_idCambra     NUMBER(3);     -- Identificador bbdd de la cambra parlamentria
    v_cambra       VARCHAR2(50);  -- Nom de la cambra parlamentria
    v_despesa      NUMBER(14,2);  -- SUma total de les despeses dels darrers 4 anys
    v_sentencia    NUMBER(1);     -- Nmero de declaraci SQL
    v_count        NUMBER(3);     -- Comptador
    v_anyActual    NUMBER(4);     -- Any actual
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_01_INSERIR';
    log_entrada    := '<' || i_cambra || '>';
    log_resultat   := 'OK. ACTUALITZACI DE L''ESTADSTICA 01 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_idCambra     := 0;
    v_cambra       := '';
    v_despesa      := 0;
    v_count        := 0;
    v_anyActual    := to_number(to_char(sysdate,'yyyy'));
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '') THEN RAISE exc_noCambra; END IF;
    
    -- Comprovaci de l'existencia de la cambra parlamentria
    v_sentencia := 1;
    SELECT ibd INTO v_idCambra FROM dce.cambra WHERE ibd = i_cambra;

    -- Obtenim la suma del total de despeses de la cambra dels darrers 4 anys
    v_sentencia := 2;
    SELECT "Id.Cambra", max("Cambra parlamentria"), sum("Import net") 
       INTO v_idCambra, v_cambra, v_despesa
    FROM dce.vis_despeses
    WHERE "Id.Cambra" = i_cambra AND (v_anyActual - "Any" < 4)
    GROUP BY "Id.Cambra";
    
    -- Comprovem si existeix l'estadstica per la cambra indicada
    SELECT COUNT(*) INTO v_count FROM dce.e1_cambra_4anys WHERE cambra = i_cambra;
    
    -- Si no existeix, es crea
    IF v_count = 0 THEN
        INSERT INTO DCE.E1_CAMBRA_4ANYS(CAMBRA, DESPESA)
           VALUES(i_cambra, v_despesa);    
    ELSE  -- Si existeix, actualitzem les dades
        UPDATE DCE.E1_CAMBRA_4ANYS SET
            DESPESA = v_despesa
        WHERE CAMBRA = i_cambra;
    END IF;       

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
    
    -- Excepcions
    EXCEPTION 
    WHEN exc_noCambra THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL CODI DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA ' || i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
      WHEN 2 THEN
        log_resultat := 'ERROR: NO S''HAN TROBAT DESPESES DELS DARRERS 4 ANYS DE LA CAMBRA ' || 
            i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);      
    END CASE;
    
    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END ESTADISTICA_01_INSERIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_01_LLEGIR.
---------------------------------------------------------------------------------------------------
/* Donat un parlament: la suma de totes les despeses dels darrers 4 anys (incloent les despeses 
   generals dels parlaments i les despeses associades als parlamentaris).
   - Mostra la informaci actual de la taula E1_CAMBRA_4ANYS
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria  
   - L'existncia de dades estadstiques de la cambra indicada     
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_01_LLEGIR (
    i_cambra   IN  NUMBER,    -- Identificador bbdd de la cambra parlamentria
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra   EXCEPTION; -- No s'ha indicat el parmetre i_cambra

    -- Variables per a la consulta
    v_cambra       VARCHAR2(50);  -- Nom de la cambra parlamentria
    v_despesa      NUMBER(14,2);  -- SUma total de les despeses dels darrers 4 anys
    v_sentencia    NUMBER(1);     -- Nmero de declaraci SQL
    v_seleccio     VARCHAR2(80);  -- Criteris de seleci que acompanyen al ttol

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_01_LLEGIR';
    log_entrada    := '<' || i_cambra || '>';
    log_resultat   := 'OK. CONSULTA DE L''ESTADSTICA 01 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_cambra       := '';
    v_despesa      := 0;
    v_seleccio     := '';
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '') THEN RAISE exc_noCambra; END IF;
    
    -- Comprovaci de l'existencia de la cambra parlamentria
    v_sentencia := 1;
    SELECT CAMBRA INTO v_cambra FROM CAMBRA WHERE IBD = i_cambra;

    -- Obtenci de les dades de l'estadstica
    v_sentencia := 2;
    SELECT despesa INTO v_despesa FROM dce.e1_cambra_4anys WHERE cambra = i_cambra;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Cambra ' || i_cambra || ' ' || v_cambra;
            
        log_abans := v_seleccio;
		
        -- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ESTADSTICA_01. SUMA DE TOTES LES DESPESES '|| 
            ' D''UNA CAMBRA');
        DBMS_OUTPUT.PUT_LINE('                    DELS DARRERS 4 ANYS');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE('Suma total de despeses: ' || 
          TRIM(TO_CHAR(v_despesa,'9G999G999G990D99')));
        DBMS_OUTPUT.PUT_LINE('');
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);     

    -- Excepcions
    EXCEPTION 
    WHEN exc_noCambra THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL CODI DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA ' || i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
      WHEN 2 THEN
       log_resultat := 'ERROR: NO S''HAN TROBAT DADES ESTADSTIQUES DE LA CAMBRA '||
           'PARLAMENTRIA ' || i_cambra;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    END CASE;

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    
END ESTADISTICA_01_LLEGIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_02_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un parlament i any: la diferencia en euros entre el parlamentari que ms despeses 
   associades t i el que menys. 
   - Insereix la informaci actual en la taula E2_DIFERENCIA_MEMBRES quan s'executa un 
     procediment ABM de despeses personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria  
   - L'existncia de despeses personals de la cambra i any indicats
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_02_INSERIR(
    i_cambra   IN  NUMBER,    -- Identificador bbdd de la cambra parlamentria
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra   EXCEPTION; -- No s'ha indicat el parmetre i_cambra
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any 

    -- Variables per a la consulta
    v_idCambra     NUMBER(3);     -- Identificador bbdd de la cambra parlamentria
    v_membreMes    NUMBER(5);     -- Identificador bbdd del membre amb ms despesa
    v_membreMenys  NUMBER(5);     -- Identificador bbdd del membre amb menys despesa
    v_despesaMes   NUMBER(14,2);  -- Suma de les despeses superiors
    v_despesaMenys NUMBER(14,2);  -- Suma de les despeses inferiors
    v_diferencia   NUMBER(14,2);  -- Diferncia de les despeses
    v_sentencia    NUMBER(1);     -- Nmero de declaraci SQL
    v_count        NUMBER(3);     -- Comptador
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_02_INSERIR';
    log_entrada    := '<' || i_cambra || '> <' || i_any || '>';
    log_resultat   := 'OK. ACTUALITZACI DE L''ESTADSTICA 02 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_idCambra     := 0;
    v_membreMes    := 0;
    v_membreMenys  := 0;
    v_despesaMes   := 0;
    v_despesaMenys := 0;
    v_diferencia   := 0;
    v_count        := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '') THEN RAISE exc_noCambra; END IF;
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Comprovaci de l'existencia de la cambra parlamentria
    v_sentencia := 1;
    SELECT ibd INTO v_idCambra FROM dce.cambra WHERE ibd = i_cambra;

    -- Obtenim les dades del membre amb ms despeses de la cambra i l'any indicats
    v_sentencia := 2;
    SELECT "Id.Membre", "Despeses" INTO v_membreMes, v_despesaMes
    FROM (SELECT "Id.Membre", sum("Import net") AS "Despeses"
          FROM dce.vis_despeses_membres 
          WHERE "Id.Cambra" = i_cambra AND "Any" = i_any
          GROUP BY "Id.Membre"
          ORDER BY 2 DESC)
    WHERE rownum = 1;

    -- Obtenim les dades del membre amb menys despeses de la cambra i l'any indicats
    v_sentencia := 3;
    SELECT "Id.Membre", "Despeses" INTO v_membreMenys, v_despesaMenys
    FROM (SELECT "Id.Membre", sum("Import net") AS "Despeses"
             FROM dce.vis_despeses_membres 
          WHERE "Id.Cambra" = i_cambra AND "Any" = i_any
          GROUP BY "Id.Membre"
          ORDER BY 2 ASC)
    WHERE rownum = 1;

    -- Calculem la diferncia de despesa
    v_diferencia := v_despesaMes - v_despesaMenys;
    
    -- Comprovem si existeix l'estadstica pels cambra i any indicats
    SELECT COUNT(*) INTO v_count FROM dce.e2_diferencia_membres 
       WHERE cambra = i_cambra AND exercici = i_any;
    
    -- Si no existeix, es crea
    IF v_count = 0 THEN
        INSERT INTO DCE.E2_DIFERENCIA_MEMBRES(CAMBRA, EXERCICI, MEMBRE_MES, MEMBRE_MENYS, 
                                          DESPESA_MES, DESPESA_MENYS, DIFERENCIA)
        VALUES(i_cambra, i_any, v_membreMes, v_membreMenys, v_despesaMes, v_despesaMenys, 
               v_diferencia);    
    ELSE  -- Si existeix, actualitzem les dades
        UPDATE DCE.E2_DIFERENCIA_MEMBRES SET
            MEMBRE_MES    = v_membreMes,
            MEMBRE_MENYS  = v_membreMenys,
            DESPESA_MES   = v_despesaMes,
            DESPESA_MENYS = v_despesaMenys,
            DIFERENCIA    = v_diferencia
        WHERE CAMBRA = i_cambra AND EXERCICI = i_any;
    END IF;       

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
    
    -- Excepcions
    EXCEPTION 
    WHEN exc_noCambra THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL CODI DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA ' || i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
      WHEN 2 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT EL/LA MEMBRE AMB MS DESPESES PER L''ANY ' || 
            i_any;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);      
      WHEN 3 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT EL/LA MEMBRE AMB MENYS DESPESES PER L''ANY ' || 
            i_any;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);      
    END CASE;
    
    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END ESTADISTICA_02_INSERIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_02_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un parlament i any: la diferencia en euros entre el parlamentari que ms despeses 
   associades t i el que menys. 
   - Mostra la informaci actual de la taula E2_DIFERENCIA_MEMBRES
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria 
   - L'existncia de dades estadstiques de la cambra i any indicats    
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_02_LLEGIR (
    i_cambra   IN  NUMBER,    -- Identificador bbdd de la cambra parlamentria
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra   EXCEPTION; -- No s'ha indicat el parmetre i_cambra
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any

    -- Variables per a la consulta
    v_cambra       VARCHAR2(50);  -- Nom de la cambra parlamentria
    v_membreMes    VARCHAR2(50);  -- Cognoms i nom del membre amb ms despesa
    v_membreMenys  VARCHAR2(50);  -- Cognoms i nom del membre amb menys despesa
    v_despesaMes   NUMBER(14,2);  -- Suma de les despeses superiors
    v_despesaMenys NUMBER(14,2);  -- Suma de les despeses inferiors
    v_diferencia   NUMBER(14,2);  -- Diferncia de les despeses
    v_seleccio     VARCHAR2(80);  -- Criteris de seleci que acompanyen al ttol
    v_sentencia    NUMBER(1);     -- Nmero de declaraci SQL

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_02_LLEGIR';
    log_entrada    := '<' || i_cambra || '> <' || i_any || '>';
    log_resultat   := 'OK. CONSULTA DE L''ESTADSTICA 02 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_cambra       := '';
    v_membreMes    := '';
    v_membreMenys  := '';
    v_despesaMes   := 0;
    v_despesaMenys := 0;
    v_diferencia   := 0;
    v_seleccio     := '';
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '') THEN RAISE exc_noCambra; END IF;
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Comprovaci de l'existencia de la cambra parlamentria
    v_sentencia := 1;
    SELECT cambra INTO v_cambra FROM dce.cambra WHERE ibd = i_cambra;

    -- Obtenci de les dades de l'estadstica
    v_sentencia := 2;
    SELECT substr(m.cognoms || ', ' || m.nom,1,60), e.despesa_mes,
           substr(n.cognoms || ', ' || n.nom,1,60), e.despesa_menys, e.diferencia
    INTO v_membreMes, v_despesaMes, v_membreMenys, v_despesaMenys, v_diferencia
    FROM dce.membre_cambra m, dce.membre_cambra n, dce.e2_diferencia_membres e
    WHERE m.ibd = e.membre_mes AND n.ibd = e.membre_menys AND 
          e.cambra = i_cambra AND e.exercici = i_any;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Cambra ' || i_cambra || ' ' || v_cambra || ' / Any ' || i_any;
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ESTADSTICA_02. DIFERNCIA EN EUROS ENTRE EL '|| 
            'MEMBRE QUE MS DESPESES');
        DBMS_OUTPUT.PUT_LINE('                    ASSOCIADES T I EL QUE MENYS ' ||
             'EN T, D''UNA CAMBRA');
        DBMS_OUTPUT.PUT_LINE('                    PARLAMENTRIA EN UN ANY');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Membre de la cambra amb ms despeses ' || 
                                  '/ menys despeses',60) ||
                             RPAD('Despeses',19));
        DBMS_OUTPUT.PUT_LINE(RPAD('-',59,'-') || ' ' ||
                             '------------------ ');
 
        DBMS_OUTPUT.PUT_LINE(RPAD(v_membreMes,60) ||
            LPAD(TO_CHAR(v_despesaMes,'9G999G999G990D99'),18));
        DBMS_OUTPUT.PUT_LINE(RPAD(v_membreMenys,60) ||
            LPAD(TO_CHAR(v_despesaMenys,'9G999G999G990D99'),18));
        DBMS_OUTPUT.PUT_LINE(RPAD(' ',60) || '------------------ ');            
        DBMS_OUTPUT.PUT_LINE(RPAD(' ',60) ||
            LPAD(TO_CHAR(v_diferencia,'9G999G999G990D99'),18));
        DBMS_OUTPUT.PUT_LINE('');            
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);     

    -- Excepcions
    EXCEPTION 
    WHEN exc_noCambra THEN
       log_resultat := 'ERROR: NO S''HA INDICAT EL CODI DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noAny THEN
       log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA ' || i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
      WHEN 2 THEN
       log_resultat := 'ERROR: NO S''HAN TROBAT DADES ESTADSTIQUES DE LA CAMBRA '||
           'PARLAMENTRIA ' || i_cambra || ' PER L''ANY ' || i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    END CASE;

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    
END ESTADISTICA_02_LLEGIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_03_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el parlament que ha tingut un total de despeses ms alt 
   (incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).
   - Insereix la informaci actual de la taula E3_MAX_CAMBRA quan s'executa un procediment ABM 
     de despeses.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_03_INSERIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any 

    -- Variables per a la consulta
    v_idCambra     NUMBER(3);     -- Identificador bbdd de la cambra parlamentria
    v_despeses     NUMBER(14,2);  -- Import de les despeses
    v_count        NUMBER(3);     -- Comptador
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_03_INSERIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. ACTUALITZACI DE L''ESTADSTICA 03 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_idCambra     := 0;
    v_despeses     := 0;
    v_count        := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenim les dades de la cambra amb ms despeses per l'any indicat
    SELECT "Id.Cambra", "Import" INTO v_idCambra, v_despeses
    FROM (SELECT "Id.Cambra", Sum("Import net") AS "Import"
          FROM dce.vis_despeses 
          WHERE "Any" = i_any
          GROUP BY "Id.Cambra" 
          ORDER BY 2 DESC)
    WHERE rownum = 1;
    
    -- Comprovem si existeix l'estadstica per l'any indicat
    SELECT COUNT(*) INTO v_count FROM dce.e3_max_cambra WHERE exercici = i_any;
    
    -- Si no existeix, es crea
    IF v_count = 0 THEN
        INSERT INTO DCE.E3_MAX_CAMBRA(CAMBRA, EXERCICI, DESPESA)
        VALUES(v_idCambra, i_any, v_despeses);    
    ELSE  -- Si existeix, actualitzem les dades
        UPDATE DCE.E3_MAX_CAMBRA SET
            CAMBRA    = v_idCambra,
            EXERCICI  = i_any,
            DESPESA   = v_despeses
        WHERE EXERCICI = i_any;
    END IF;       

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
    
    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
       log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA AMB MS DESPESESS PER L''ANY '|| i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END ESTADISTICA_03_INSERIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_03_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el parlament que ha tingut un total de despeses ms alt 
   (incloent les despeses generals dels parlaments i les despeses associades als parlamentaris).
   - Mostra la informaci actual de la taula E3_MAX_CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_03_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any

    -- Variables per a la consulta
    v_idCambra     NUMBER(3);     -- Identificador bbdd de la cambra parlamentria
    v_despeses     NUMBER(14,2);  -- Import de les despeses
    v_cambra       VARCHAR2(50);  -- Nom de la cambra parlamentria
    v_seleccio     VARCHAR2(80);  -- Criteris de seleci que acompanyen al ttol

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_03_LLEGIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. CONSULTA DE L''ESTADSTICA 03 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_idCambra     := 0;
    v_despeses     := 0;
    v_cambra       := '';
    v_seleccio     := '';
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenci de les dades de l'estadstica
    SELECT c.ibd, c.cambra, e.despesa
    INTO v_idCambra, v_cambra, v_despeses
    FROM dce.cambra c, dce.e3_max_cambra e
    WHERE c.ibd = e.cambra AND e.exercici = i_any;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Any ' || i_any;
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ESTADSTICA_03. CAMBRA PARLAMENTRIA QUE HA TINGUT UN');
        DBMS_OUTPUT.PUT_LINE('                    TOTAL  DE DESPESES  MS ALT EN UN ANY');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Codi Cambra amb ms despeses',55) ||
                             RPAD('Despeses',19));
        DBMS_OUTPUT.PUT_LINE('---- ' || RPAD('-',49,'-') || ' ' ||
                             '------------------ ');
 
        DBMS_OUTPUT.PUT_LINE(LPAD(v_idCambra,4) || ' ' ||
            RPAD(v_cambra,50) ||
            LPAD(TO_CHAR(v_despeses,'9G999G999G990D99'),18));
        DBMS_OUTPUT.PUT_LINE('');
   END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);     

    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
       log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
       log_resultat := 'ERROR: NO S''HAN TROBAT DADES ESTADSTIQUES PER L''ANY '|| i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    
END ESTADISTICA_03_LLEGIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_04_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un parlamentari: la despesa total que ha tingut els darrers 4 anys.
   - Insereix la informaci actual en la taula E4_MEMBRE_4ANYS quan s'executa un 
     procediment ABM de despeses personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia del/de la membre de cambra parlamentria
   - L'existncia de despeses del/de la membre indicat
   - Es computa la suma de les despeses a partir de l'any actual fins els tres anys anteriors.
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_04_INSERIR (
    i_membre   IN  NUMBER,    -- Identificador bbdd de membre de cambra parlamentria
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noMembre   EXCEPTION; -- No s'ha indicat el parmetre i_membre

    -- Variables per a la consulta
    v_idMembre     NUMBER(3);     -- Identificador bbdd de la cambra parlamentria
    v_membre       VARCHAR2(60);  -- Cognoms i nom del/de la membre de la cambra
    v_despesa      NUMBER(14,2);  -- Suma total de les despeses dels darrers 4 anys
    v_sentencia    NUMBER(1);     -- Nmero de declaraci SQL
    v_count        NUMBER(3);     -- Comptador
    v_anyActual    NUMBER(4);     -- Any actual
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_04_INSERIR';
    log_entrada    := '<' || i_membre || '>';
    log_resultat   := 'OK. ACTUALITZACI DE L''ESTADSTICA 04 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_idMembre     := 0;
    v_membre       := '';
    v_despesa      := 0;
    v_count        := 0;
    v_anyActual    := to_number(to_char(sysdate,'yyyy'));
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_membre IS NULL OR i_membre = '') THEN RAISE exc_noMembre; END IF;
    
    -- Comprovaci de l'existencia del/de la membre de la cambra
    v_sentencia := 1;
    SELECT ibd INTO v_idMembre FROM dce.membre_cambra WHERE ibd = i_membre;

    -- Obtenim la suma del total de despeses del/de la membre dels darrers 4 anys
    v_sentencia := 2;
    SELECT "Id.Membre", SUBSTR(MAX("Cognoms" || ', ' || "Nom"),1,60), SUM("Import net") 
    INTO v_idMembre, v_membre, v_despesa
    FROM dce.vis_despeses_membres
    WHERE "Id.Membre" = i_membre AND (v_anyActual - "Any" < 4)
    GROUP BY "Id.Membre";
    
    -- Comprovem si existeix l'estadstica del/de la membre indicat/da
    SELECT COUNT(*) INTO v_count FROM dce.e4_membre_4anys WHERE membre = i_membre;
    
    -- Si no existeix, es crea
    IF v_count = 0 THEN
        INSERT INTO DCE.E4_MEMBRE_4ANYS(MEMBRE, DESPESA)
           VALUES(i_membre, v_despesa);    
    ELSE  -- Si existeix, actualitzem les dades
        UPDATE DCE.E4_MEMBRE_4ANYS SET
            DESPESA = v_despesa
        WHERE MEMBRE = i_membre;
    END IF;       

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
    
    -- Excepcions
    EXCEPTION 
    WHEN exc_noMembre THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL IBD DE MEMBRE DE CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT EL/LA MEMBRE DE CAMBRA PARLAMENTRIA ' || i_membre;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
      WHEN 2 THEN
        log_resultat := 'ERROR: NO S''HAN TROBAT DESPESES DELS DARRERS 4 ANYS DEL/DE LA MEMBRE ' || 
            i_membre;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);      
    END CASE;
    
    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END ESTADISTICA_04_INSERIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_04_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un parlamentari: la despesa total que ha tingut els darrers 4 anys.
   - Mostra la informaci actual de la taula E4_MEMBRE_4ANYS
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia del/de la membre de cambra parlamentria
   - L'existncia de dades estadstiques del membre de cambra indicat 
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_04_LLEGIR (
    i_membre   IN  NUMBER,    -- Identificador bbdd de membre de cambra parlamentria
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noMembre   EXCEPTION; -- No s'ha indicat el parmetre i_membre

    -- Variables per a la consulta
    v_membre       VARCHAR2(60);  -- Cognoms i nom del/de la membre de la cambra
    v_idCambra     NUMBER(3);     -- Codi de la cambra parlamentria
    v_cambra       VARCHAR2(50);  -- Nom de la cambra parlamentria a la que pertany
    v_numero       NUMBER(3);     -- Nmero de membre en la cambra
    v_despesa      NUMBER(14,2);  -- SUma total de les despeses dels darrers 4 anys
    v_sentencia    NUMBER(1);     -- Nmero de declaraci SQL
    v_seleccio1    VARCHAR2(80);  -- Criteris de seleci que acompanyen al ttol L1
    v_seleccio2    VARCHAR2(80);  -- Criteris de seleci que acompanyen al ttol L2
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_04_LLEGIR';
    log_entrada    := '<' || i_membre || '>';
    log_resultat   := 'OK. CONSULTA DE L''ESTADSTICA 04 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
	v_idCambra     := 0;
	v_cambra       := '';
	v_numero       := 0;
    v_membre       := '';
    v_despesa      := 0;
    v_seleccio1    := '';
    v_seleccio2    := '';
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_membre IS NULL OR i_membre = '') THEN RAISE exc_noMembre; END IF;
    
    -- Comprovaci de l'existencia del/de la membre de la cambra
    v_sentencia := 1;
    SELECT SUBSTR("Nom" || ' ' || "Cognoms",1,60), "Nm.", "Cambra parlamentria", "Id.Cambra"
        INTO v_membre, v_numero, v_cambra, v_idCambra
    FROM dce.vis_membres WHERE "Id.Membre" = i_membre;

    -- Obtenci de les dades de l'estadstica
    v_sentencia := 2;
    SELECT despesa INTO v_despesa
        FROM dce.e4_membre_4anys WHERE membre = i_membre;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio1 := 'Membre ' || i_membre || ' ' || v_membre;
        v_seleccio2 := 'Cambra ' || v_idCambra || ' ' || v_cambra || 
                       ' Nm.membre ' || v_numero;
            
        log_abans := v_seleccio1 || ' /' || v_seleccio2;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ESTADSTICA_04. SUMA DE TOTES  LES DESPESES '|| 
                             'D''UN/A  MEMBRE');
        DBMS_OUTPUT.PUT_LINE('                    DE CAMBRA PARLAMENTRIA DELS ' || 
                             'DARRERS 4 ANYS');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio1);
        DBMS_OUTPUT.PUT_LINE(v_seleccio2);        
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE('Suma total de despeses: ' || 
            TRIM(TO_CHAR(v_despesa,'9G999G999G990D99')));
        DBMS_OUTPUT.PUT_LINE('');
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);     

    -- Excepcions
    EXCEPTION 
    WHEN exc_noMembre THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL IBD DE MEMBRE DE CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT EL/LA MEMBRE DE CAMBRA PARLAMENTRIA ' || i_membre;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
      WHEN 2 THEN
       log_resultat := 'ERROR: NO S''HAN TROBAT DADES ESTADSTIQUES DEL/DE LA MEMBRE '|| i_membre;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    END CASE;

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    
END ESTADISTICA_04_LLEGIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_05_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: la mitjana de despesa dun parlamentari durant aquell any 
   (tenint en compte tots els parlamentaris de tots els parlaments).
   - Insereix la informaci actual en la taula E5_MITJANA_MEMBRE quan s'executa un procediment ABM 
     de despeses.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses de l'any indicat   
   - La mitjana es calcula fent la divisi entre la suma de totes les despeses  i el nmero 
     de membres que les han fet.   
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_05_INSERIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any 

    -- Variables per a la consulta
    v_mitjana      NUMBER(14,2);  -- Import de les despeses
    v_count        NUMBER(3);     -- Comptador
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_05_INSERIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. ACTUALITZACI DE L''ESTADSTICA 05 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_mitjana      := 0;
    v_count        := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenim les dades de la mitja de despeses d'un parlamentari per l'any indicat
    SELECT SUM("Import net")/COUNT(DISTINCT("Id.Membre")) INTO v_mitjana
       FROM dce.vis_despeses_membres
    WHERE "Any" = i_any;
    
    -- Comprovem si existeix l'estadstica per l'any indicat
    SELECT COUNT(*) INTO v_count FROM dce.e5_mitjana_membre WHERE exercici = i_any;
    
    -- Si no existeix, es crea
    IF v_count = 0 THEN
        INSERT INTO DCE.E5_MITJANA_MEMBRE (EXERCICI, MITJANA)
        VALUES(i_any, v_mitjana);    
    ELSE  -- Si existeix, actualitzem les dades
        UPDATE DCE.E5_MITJANA_MEMBRE  SET
            MITJANA   = v_mitjana
        WHERE EXERCICI = i_any;
    END IF;       

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
    
    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
       log_resultat := 'ERROR: NO S''HA TROBAT LA MITJA DE DESPESES DE MEMBRE PER L''ANY '|| i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END ESTADISTICA_05_INSERIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_05_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: la mitjana de despesa dun parlamentari durant aquell any 
   (tenint en compte tots els parlamentaris de tots els parlaments).
   - Mostra la informaci actual de la taula E5_MITJANA_MEMBRE
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_05_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any

    -- Variables per a la consulta
    v_idCambra     NUMBER(3);     -- Identificador bbdd de la cambra parlamentria
    v_mitjana      NUMBER(14,2);  -- Import de les despeses
    v_cambra       VARCHAR2(50);  -- Nom de la cambra parlamentria
    v_seleccio     VARCHAR2(80);  -- Criteris de seleci que acompanyen al ttol

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_05_LLEGIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. CONSULTA DE L''ESTADSTICA 05 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_idCambra     := 0;
    v_mitjana      := 0;
    v_cambra       := '';
    v_seleccio     := '';
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenci de les dades de l'estadstica
    SELECT e.mitjana INTO v_mitjana
        FROM dce.e5_mitjana_membre e WHERE e.exercici = i_any;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Any ' || i_any;
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));        
        DBMS_OUTPUT.PUT_LINE('*** ESTADSTICA_05. MITJANA DE DESPESA D''UN/A MEMBRE');
        DBMS_OUTPUT.PUT_LINE('                    DE CAMBRA PARLAMENTRIA EN UN ANY');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE('Mitjana de despesa ' || i_any || ': ' || 
          TRIM(TO_CHAR(v_mitjana,'9G999G999G990D99')));
        DBMS_OUTPUT.PUT_LINE('');
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);     

    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
       log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
       log_resultat := 'ERROR: NO S''HAN TROBAT DADES ESTADSTIQUES PER L''ANY '|| i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    
END ESTADISTICA_05_LLEGIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_06_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el parlamentari que ha tingut ms despesa associada i el que menys 
   (tenint en compte tots els parlaments).
   - Insereix la informaci actual en la taula E6_MES_MENYS_MEMBRE quan s'executa un procediment
     ABM de despeses personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses personals de l'any indicat   
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_06_INSERIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any 

    -- Variables per a la consulta
    v_membreMes    NUMBER(5);     -- Identificador bbdd del membre amb ms despesa
    v_membreMenys  NUMBER(5);     -- Identificador bbdd del membre amb menys despesa
    v_despesaMes   NUMBER(14,2);  -- Suma de les despeses superiors
    v_despesaMenys NUMBER(14,2);  -- Suma de les despeses inferiors
    v_count        NUMBER(3);     -- Comptador
    v_sentencia    NUMBER(1);     -- Nmero de declaraci SQL
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_06_INSERIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. ACTUALITZACI DE L''ESTADSTICA 06 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_membreMes    := 0;
    v_membreMenys  := 0;
    v_despesaMes   := 0;
    v_despesaMenys := 0;
    v_count        := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenim les dades del membre amb ms despeses per l'any indicat
    v_sentencia := 1;
    SELECT "Id.Membre", "Despeses" INTO v_membreMes, v_despesaMes
    FROM (SELECT "Id.Membre", sum("Import net") AS "Despeses"
          FROM dce.vis_despeses_membres 
          WHERE "Any" = i_any
          GROUP BY "Id.Membre"
          ORDER BY 2 DESC)
    WHERE rownum = 1;

    -- Obtenim les dades del membre amb menys despeses per l'any indicat
    v_sentencia := 2;
    SELECT "Id.Membre", "Despeses" INTO v_membreMenys, v_despesaMenys
    FROM (SELECT "Id.Membre", sum("Import net") AS "Despeses"
          FROM dce.vis_despeses_membres 
          WHERE "Any" = i_any
          GROUP BY "Id.Membre"
          ORDER BY 2 ASC)
    WHERE rownum = 1;
    
    -- Comprovem si existeix l'estadstica per l'any indicat
    SELECT COUNT(*) INTO v_count FROM dce.e6_mes_menys_membre WHERE exercici = i_any;
    
    -- Si no existeix, es crea
    IF v_count = 0 THEN
        INSERT INTO DCE.E6_MES_MENYS_MEMBRE 
            (EXERCICI, MEMBRE_MES, MEMBRE_MENYS, DESPESA_MES, DESPESA_MENYS)
        VALUES(i_any, v_membreMes, v_membreMEnys, v_despesaMes,  v_despesaMenys);    
    ELSE  -- Si existeix, actualitzem les dades
        UPDATE DCE.E6_MES_MENYS_MEMBRE  SET
            MEMBRE_MES   = v_membreMes,
            MEMBRE_MENYS = v_membreMenys,
            DESPESA_MES   = v_despesaMes,
            DESPESA_MENYS = v_despesaMenys
        WHERE EXERCICI = i_any;
    END IF;       

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
    
    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT EL/LA MEMBRE AMB MS DESPESES PER L''ANY ' || i_any;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);      
      WHEN 2 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT EL/LA MEMBRE AMB MENYS DESPESES PER L''ANY ' || i_any;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);      
    END CASE;

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END ESTADISTICA_06_INSERIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_06_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el parlamentari que ha tingut ms despesa associada i el que menys 
   (tenint en compte tots els parlaments).
   - Mostra la informaci actual de la taula E6_MES_MENYS_MEMBRE
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_06_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any

    -- Variables per a la consulta
    v_membreMes    VARCHAR2(50);  -- Cognoms i nom del membre amb ms despesa
    v_membreMenys  VARCHAR2(50);  -- Cognoms i nom del membre amb menys despesa
    v_despesaMes   NUMBER(14,2);  -- Suma de les despeses superiors
    v_despesaMenys NUMBER(14,2);  -- Suma de les despeses inferiors
    v_diferencia   NUMBER(14,2);  -- Diferncia de les despeses
    v_seleccio     VARCHAR2(80);  -- Criteris de seleci que acompanyen al ttol

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_06_LLEGIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. CONSULTA DE L''ESTADSTICA 06 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_membreMes    := '';
    v_membreMenys  := '';
    v_despesaMes   := 0;
    v_despesaMenys := 0;
    v_diferencia   := 0;
    v_seleccio     := '';
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenci de les dades de l'estadstica
    SELECT substr(m.cognoms || ', ' || m.nom,1,60), e.despesa_mes,
           substr(n.cognoms || ', ' || n.nom,1,60), e.despesa_menys
    INTO v_membreMes, v_despesaMes, v_membreMenys, v_despesaMenys
    FROM dce.membre_cambra m, dce.membre_cambra n, dce.e6_mes_menys_membre e
    WHERE m.ibd = e.membre_mes and n.ibd = e.membre_menys AND e.exercici = i_any;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio   := 'Any ' || i_any;
        v_diferencia := v_despesaMes - v_despesaMenys;    
        log_abans    := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ESTADSTICA_06. MEMBRES AMB MS I MENYS DESPESES ' ||
            'ASSOCIADES DE');
        DBMS_OUTPUT.PUT_LINE('                    TOTES LES CAMBRES PARLAMENTRIES ' ||
            'EN UN ANY');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Membre amb ms despeses / menys despeses',60) ||
                             RPAD('Despeses',19));
        DBMS_OUTPUT.PUT_LINE(RPAD('-',59,'-') || ' ' ||
                             '------------------ ');
 
        DBMS_OUTPUT.PUT_LINE(RPAD(v_membreMes,60) ||
            LPAD(TO_CHAR(v_despesaMes,'9G999G999G990D99'),18));
        DBMS_OUTPUT.PUT_LINE(RPAD(v_membreMenys,60) ||
            LPAD(TO_CHAR(v_despesaMenys,'9G999G999G990D99'),18));
        DBMS_OUTPUT.PUT_LINE(RPAD(' ',60) || '------------------ ');            
        DBMS_OUTPUT.PUT_LINE(RPAD(' ',60) ||
            LPAD(TO_CHAR(v_diferencia,'9G999G999G990D99'),18));
        DBMS_OUTPUT.PUT_LINE('');            
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);     

    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
       log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
       log_resultat := 'ERROR: NO S''HAN TROBAT DADES ESTADSTIQUES PER L''ANY ' || i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    
END ESTADISTICA_06_LLEGIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_07_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el nom del parlament amb una mitjana de despesa associada als seus 
   parlamentaris ms alta, i el valor daquesta mitjana de despesa.
   - Insereix la informaci actual en la taula E7_MITJANA_MAX_CAMBRA quan s'executa un procediment
     ABM de despeses personals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses personals de l'any indicat   
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_07_INSERIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any 

    -- Variables per a la consulta
    v_idCambra     NUMBER(3);     -- Identificador bbdd de la cambra parlamentria
    v_mitjana      NUMBER(14,2);  -- Import de la mitjana
    v_count        NUMBER(3);     -- Comptador
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_07_INSERIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. ACTUALITZACI DE L''ESTADSTICA 07 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_idCambra     := 0;
    v_mitjana      := 0;
    v_count        := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenim les dades de la cambra amb la mitjana ms alta per l'any indicat
    select "idCambra", "mitjana" into v_idCambra, v_mitjana
    from (select  "Id.Cambra" as "idCambra", 
              sum("Import net") / count(distinct("Id.Membre")) as "mitjana"
          from dce.vis_despeses_membres where "Any" = i_any
          group by "Id.Cambra"
          order by 2 desc)
    where rownum = 1;
    
    -- Comprovem si existeix l'estadstica per l'any indicat
    SELECT COUNT(*) INTO v_count FROM dce.e7_mitjana_max_cambra WHERE exercici = i_any;
    
    -- Si no existeix, es crea
    IF v_count = 0 THEN
        INSERT INTO DCE.E7_MITJANA_MAX_CAMBRA(EXERCICI, CAMBRA, MITJANA)
        VALUES(i_any, v_idCambra, v_mitjana);    
    ELSE  -- Si existeix, actualitzem les dades
        UPDATE DCE.E7_MITJANA_MAX_CAMBRA SET
            CAMBRA    = v_idCambra,
            MITJANA   = v_mitjana
        WHERE EXERCICI = i_any;
    END IF;       

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
    
    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
       log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA AMB LA MITJANA MS ALTA PER L''ANY ' || i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END ESTADISTICA_07_INSERIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_07_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: el nom del parlament amb una mitjana de despesa associada als seus 
   parlamentaris ms alta, i el valor daquesta mitjana de despesa.
   - Mostra la informaci actual de la taula E7_MITJANA_MAX_CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_07_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any

    -- Variables per a la consulta
    v_idCambra     NUMBER(3);     -- Identificador bbdd de la cambra parlamentria
    v_mitjana      NUMBER(14,2);  -- Import de la mitjana
    v_cambra       VARCHAR2(50);  -- Nom de la cambra parlamentria
    v_seleccio     VARCHAR2(80);  -- Criteris de seleci que acompanyen al ttol

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_03_LLEGIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. CONSULTA DE L''ESTADSTICA 03 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_idCambra     := 0;
    v_mitjana      := 0;
    v_cambra       := '';
    v_seleccio     := '';
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenci de les dades de l'estadstica
    SELECT c.ibd, c.cambra, e.mitjana
       INTO v_idCambra, v_cambra, v_mitjana
       FROM dce.cambra c, dce.e7_mitjana_max_cambra e
       WHERE c.ibd = e.cambra AND e.exercici = i_any;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Any ' || i_any;
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',70,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ESTADSTICA_07. CAMBRA PARLAMENTRIA AMB LA MITJANA ' 
            || ' DE DESPESA ASSOCIADA');
        DBMS_OUTPUT.PUT_LINE('                    ALS/A LES  SEUS/SEVES MEMBRES MS ' ||
            'ALTA EN UN ANY');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Codi Cambra amb mitjana ms alta',55) ||
                             RPAD('Mitjana',19));
        DBMS_OUTPUT.PUT_LINE('---- ' || RPAD('-',49,'-') || ' ' ||
                             '------------------ ');
 
        DBMS_OUTPUT.PUT_LINE(LPAD(v_idCambra,4) || ' ' ||
            RPAD(v_cambra,50) ||
            LPAD(TO_CHAR(v_mitjana,'9G999G999G990D99'),18));
        DBMS_OUTPUT.PUT_LINE('');
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);     

    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
       log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
       log_resultat := 'ERROR: NO S''HAN TROBAT DADES ESTADSTIQUES PER L''ANY '|| i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    
END ESTADISTICA_07_LLEGIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_08_INSERIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: la suma de totes les despeses de tots els parlaments (incloent les 
   despeses generals dels parlaments i les despeses associades als parlamentaris).
   - Insereix la informaci actual en la taula E8_TOTAL_CAMBRES quan s'executa un procediment
     ABM de despeses personals i generals.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de despeses generals i personals de l'any indicat   
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_08_INSERIR (
    i_any      IN  NUMBER,   -- Any de les despeses
    o_rsp     OUT  VARCHAR2  -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny    EXCEPTION;  -- No s'ha indicat el parmetre i_any 

    -- Variables per a la consulta
    v_despesa      NUMBER(14,2);  -- Suma del import net de totes les despeses
    v_count        NUMBER(3);     -- Comptador
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_08_INSERIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. ACTUALITZACI DE L''ESTADSTICA 08 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_despesa      := 0;
    v_count        := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenim la suma total de despeses de totes les cambres per l'any indicat
    SELECT SUM(Import_net) INTO v_despesa
       FROM dce.despesa
    WHERE TO_NUMBER(TO_CHAR(data_despesa,'YYYY')) = i_any;
    
    -- Comprovem si existeix l'estadstica per l'any indicat
    SELECT COUNT(*) INTO v_count FROM dce.e8_total_cambres WHERE exercici = i_any;
    
    -- Si no existeix, es crea
    IF v_count = 0 THEN
        INSERT INTO DCE.E8_TOTAL_CAMBRES (EXERCICI, DESPESA)
        VALUES(i_any, v_despesa);    
    ELSE  -- Si existeix, actualitzem les dades
        UPDATE DCE.E8_TOTAL_CAMBRES  SET
            DESPESA = v_despesa
        WHERE EXERCICI = i_any;
    END IF;       

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
    
    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
       log_resultat := 'ERROR: NO S''HA TROBAT LA SUMA DE DESPESES DE LES CAMBRES PER L''ANY '|| i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END ESTADISTICA_08_INSERIR;

---------------------------------------------------------------------------------------------------
-- *** ESTADISTICA_08_LLEGIR. 
---------------------------------------------------------------------------------------------------
/* Donat un any concret: la suma de totes les despeses de tots els parlaments (incloent les 
   despeses generals dels parlaments i les despeses associades als parlamentaris).
   - Mostra la informaci actual de la taula E8_TOTAL_CAMBRES
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats  
   - L'existncia de dades estadstiques de l'any indicat
-------------------------------------------------------------------------------------------------*/
PROCEDURE ESTADISTICA_08_LLEGIR (
    i_any      IN  NUMBER,    -- Any de les despeses
    o_rsp     OUT  VARCHAR2   -- Resultat de l'execuci del procediment estadstic
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny      EXCEPTION; -- No s'ha indicat el parmetre i_any

    -- Variables per a la consulta
    v_despesa      NUMBER(14,2);  -- Suma del import net de totes les despeses
    v_seleccio     VARCHAR2(80);  -- Criteris de seleci que acompanyen al ttol

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces     := 'EST.ESTADISTICA_08_LLEGIR';
    log_entrada    := '<' || i_any || '>';
    log_resultat   := 'OK. CONSULTA DE L''ESTADSTICA 08 EFECTUADA';
    log_abans      := '';
    log_despres    := '';
    log_error      := 'S';
    log_usuari     := USER;
    log_data       := SYSDATE;
    o_rsp          := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_despesa      := 0;
    v_seleccio     := '';
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Obtenci de les dades de l'estadstica
    SELECT despesa INTO v_despesa FROM dce.e8_total_cambres WHERE exercici = i_any;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Any ' || i_any;
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ESTADSTICA_08. SUMA DE TOTES LES DESPESES  DE TOTES');
        DBMS_OUTPUT.PUT_LINE('                    LES CAMBRES PARLAMENTRIES EN UN ANY');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE('*** ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',80,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE('Suma total de despeses ' || i_any || ': ' || 
          TRIM(TO_CHAR(v_despesa,'9G999G999G990D99')));
        DBMS_OUTPUT.PUT_LINE('');
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);     

    -- Excepcions
    EXCEPTION 
    WHEN exc_noAny THEN
       log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
       log_resultat := 'ERROR: NO S''HAN TROBAT DADES ESTADSTIQUES PER L''ANY '|| i_any;
       o_rsp := log_resultat;
       PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    
END ESTADISTICA_08_LLEGIR;

-------------------------------------------------------------------------------------------------*/
END;
/ 
