---------------------------------------------------------------------------------------------------
-- UOC - ETIG - TFC Bases de dades relacionals	2013-2014 1r semestre
---------------------------------------------------------------------------------------------------
-- Base de Dades d'un Sistema Centralitzat de Control de Despesa Pblica dels Parlaments Europeus
---------------------------------------------------------------------------------------------------
-- Arxiu : 10_Paquet_CON.sql        -- Autor    : Santi Alcayde Pagan   -- Aula: 1
-- Data  : 11/11/2013               -- Consultor: Manel Rella Ruiz      
---------------------------------------------------------------------------------------------------
-- Funci: Scripts de Creaci del PAQUET DE CONSULTES
-- El resultat de l'exeuci dels procediments es guarda en la taula d'auditoria LOG_PROCEDIMENT
---------------------------------------------------------------------------------------------------

-- *** OBSERVACI SOBRE NOMENCLATURES *************************************************************
-- Els noms dels parmetes d'entrada (input) comencen per i_
-- Els noms dels parmetres de sortida (output) comencen per o_
-- Els noms de les variables pel registre (log) de l'execuci dels procediments comencen per log_
-- Els noms de les variables pel tractament de les excepcions comencen per exc_
-- Els noms de la resta de variables comencen per v_
-- Els noms de dels cursors comencen per C_
-- Els noms de les vistes comencen per VIS_
-- Les funcions comencen per F_
-- Els procedimens de carcter general comencen per PRO_
-- No s'han escrit els accents en les paraules
--*************************************************************************************************

---------------------------------------------------------------------------------------------------
-- ***********************   C A P  A L E R A    D E L    P A Q U E T   *********************** --
---------------------------------------------------------------------------------------------------

CREATE OR REPLACE PACKAGE  DCE.CON IS

    -- Variables globals pel registre en la taula LOG_PROCEDIMENT
    log_proces    VARCHAR2(60);     -- Nom del procediment que s'ha executat
    log_entrada   VARCHAR2(1024);   -- Parmetres d'entrada del procediment
    log_resultat  VARCHAR2(256);    -- Resultat de l'execucu del procediment
    log_abans     VARCHAR2(1024);   -- Valors abans d'executar el procediment, si en modifica
    log_despres   VARCHAR2(1024);   -- Valors desprs d'executar el procediment, si en modifica
    log_error     CHAR(1);          -- Indicador de si s'ha produt o no un error
    log_usuari    VARCHAR2(12);     -- Usuari de la bbdd que ha executat el procediment
    log_data      DATE;             -- Data i hora en que s'ha executat el procediment
    
     -- Variables globals de les consultes
     -- Data i hora i usuari
     v_quiquan    VARCHAR2(40)  := to_char(sysdate,'dd/mm/yyyy hh24:mi:ss') || ' - ' || user;

---------------------------------------------------------------------------------------------------
-- *** DESPESES_01.
---------------------------------------------------------------------------------------------------
/* Donada una cambra parlamentria, un any i un membre de la cambra: el llistat de totes les 
   despeses associades al membre de la cambra en aquell any.
   Tot aix ordenat de forma descendent pel valor de cada despesa.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria
   - L'existncia del nmero de membre en la cambra indicada
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_01 (
    i_cambra       IN  NUMBER,    -- Codi identificador bbdd de la cambra parlamentria
    i_membre       IN  NUMBER,    -- Nmero de membre en la cambra
    i_any          IN  NUMBER,    -- Any de les despeses
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** DESPESES_02.
---------------------------------------------------------------------------------------------------
/* Donada una cambra parlamentria, i un any: el llistat dels seus membres amb la despesa directa 
   que ha fet cadascun, incloent:
      - El seu nom i cognoms
      - Partit poltic actual
      - Suma de totes les seves despeses directes associades
   Tot aix ordenat de forma descendent pel valor de la suma de les despeses.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_02 (
    i_cambra       IN  NUMBER,    -- Codi identificador bbdd de la cambra parlamentria
    i_any          IN  NUMBER,    -- Any de les despeses
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** DESPESES_03.
---------------------------------------------------------------------------------------------------
/* Donat un any: el llistat de la despesa total del totes les cambres parlamentries, incloent:
    - Nom de la cambra parlamentria
    - Nombre de membres de la cambra
    - Despesa mitja dels seus membres (tenint en compte noms les despeses directes realitzades
      pels membres de la cambra).
    - Suma de les despeses totals associades als membres de la cambra.
    - Suma de les despeses totals associades a la cambra parlamentria (s a dir les que no estan
      associades a un membre concret).
    - Suma de totes les despeses.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_03 (
    i_any          IN  NUMBER,    -- Any de les despeses
    i_iva          IN  CHAR,      -- Imports amb o sense IVA: S=Amb IVA, N=Sense IVA    
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** DESPESES_04.
---------------------------------------------------------------------------------------------------
/* Donat un any i una cambra parlamentria: el llistat de la despesa segmentada per categories
   de despesa, s a dir, un llistat que retorni les columnes segents:
     - Codi de la categoria de despesa.
     - Descripci de la categoria de despesa.
     - Suma de les despeses totals associades a la categoria de despesa, 
       durant lany indicat i en la cambra parlamentria indicada.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_04 (
    i_cambra       IN  NUMBER,    -- Codi identificador bbdd de la cambra parlamentria
    i_any          IN  NUMBER,    -- Any de les despeses
    i_tipus        IN  CHAR,      -- Tipus de despeses: G=Generals, P=Personals, T=Totes
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** DESPESES_05.
---------------------------------------------------------------------------------------------------
/* Donada una cambra parlamentria i un any: el llistat de membres que superen la despesa mitja
   dels tots els membres daquella cambra durant lany indicat, incloent: 
     - El seu nom i cognom 
     - Partit poltic actual 
     - Suma de totes les despeses directes associades al membre de la cambra durant aquell any
     - Percentatge en que es supera la despesa mitjana  
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria     
   - La mitjana es calcula fent la divisi entre la suma de totes les despeses  i el nmero de 
     membres que les han fet.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_05 (
    i_cambra       IN  NUMBER,    -- Codi identificador bbdd de la cambra parlamentria
    i_any          IN  NUMBER,    -- Any de les despeses
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** DESPESES_06.
---------------------------------------------------------------------------------------------------
/* Donat un any:el llistat de totes les cambres parlamentries amb el seu estat comptable,incloent:
     - Nom de la cambra parlamentria
     - Nombre de membres de la cambra
     - Suma de totes les despeses pendents daprovaci.
     - Suma de totes les despeses aprovades.
     - Suma de totes les despeses pendents dabonament.
     - Suma de totes les despeses abonades.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_06 (
    i_any          IN  NUMBER,    -- Any de les despeses
    i_tipus        IN  CHAR,      -- Tipus de despeses: G=Generals, P=Personals, T=Totes
    i_iva          IN  CHAR,      -- Imports amb IVA o sense: S=Amb IVA, N=Sense IVA
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** DESPESES_06b.
---------------------------------------------------------------------------------------------------
/* Donat un any: el llistat de totes les cambres parlamentries amb el seu estat comptable 
   categoritzat,incloent:
     - Nom de la cambra parlamentria
     - Situaci comptable
     - Suma de les despeses totals associades de cada situaci comptable de cada cambra durant 
       lany indicat
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_06b (
    i_any          IN  NUMBER,    -- Any de les despeses
    i_tipus        IN  CHAR,      -- Tipus de despeses: G=Generals, P=Personals, T=Totes
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** 01. CONSULTA DE CAMBRES PARLAMENTRIES.
---------------------------------------------------------------------------------------------------
-- Fa una relaci de cambres parlamentries en funci dels parmetres demanats, on en els noms pot 
-- fer-se una cerca per paraula. Exemple i_cambra = catalunya
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRES (
    i_cambra        IN  VARCHAR2,  -- Nom de la cambra parlamentria o cap per a totes
    i_estat         IN  VARCHAR2,  -- Nom del estat o cap per a tots
    i_pais          IN  VARCHAR2,  -- Nom del pais o regi o cap per a tots
    i_baixa         IN  CHAR,      -- Indicador de situaci de baixa 'S' o 'N'o cap per a totes
    o_rsp          OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** 02. CONSULTA DE MEMBRES DE CAMBRES PARLAMENTRIES.
---------------------------------------------------------------------------------------------------
-- Fa una relaci de membres de cambres parlamentries en funci dels parmetres demanats, on en 
-- els noms pot fer-se una cerca per paraula. Exemple i_cognoms = junqueras
---------------------------------------------------------------------------------------------------
PROCEDURE MEMBRES (
    i_cognoms       IN  VARCHAR2,  -- Cognoms del membre de la cambra o cap per a tots
    i_nom           IN  VARCHAR2,  -- Nom del membre de la cambra o cap per a tots
    i_cambra        IN  VARCHAR2,  -- Nom de la cambra parlamentria o cap per a totes
    i_partit        IN  VARCHAR2,  -- Nom del partit poltic o cap per a tots
    i_baixa         IN  CHAR,      -- Indicador de situaci de baixa 'S' o 'N'o cap per a totes
    o_rsp          OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** 03. CONSULTA DE DESPESES DE CAMBRES PARLAMENTRIES.
---------------------------------------------------------------------------------------------------
-- Fa una relaci de despeses de cambres parlamentries en funci dels parmetres demanats, on en 
-- els noms pot fer-se una cerca per paraula. Exemple i_cambra = catalunya
---------------------------------------------------------------------------------------------------
PROCEDURE DESPESES_CAMBRES (
    i_dataIni       IN  VARCHAR2,  -- Data inicial de despesa amb format 'dd/mm/yyyy'
    i_dataFin       IN  VARCHAR2,  -- Data final de despesa amb format 'dd/mm/yyyy'
    i_importIni     IN  NUMBER,    -- Import brut inicial de la despesa o 0 per a tots
    i_importFin     IN  NUMBER,    -- Import brut final de la despesa o 0 per a tots
    i_cambra        IN  VARCHAR2,  -- Nom de la cambra parlamentria o cap per a totes
    i_categoria     IN  VARCHAR2,  -- Nom de la categoria general de la despesa
    i_pagament      IN  VARCHAR2,  -- Nom de la forma de pagament de la despesa
    i_situacio      IN  VARCHAR2,  -- Nom de la situaci de la despesa
    i_descripcio    IN  VARCHAR2,  -- Descripci de la despesa
    i_justificacio  IN  VARCHAR2,  -- Descripci de la despesa
    o_rsp          OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);

---------------------------------------------------------------------------------------------------
-- *** 04. CONSULTA DE DESPESES DE MEMBRES DE CAMBRES PARLAMENTRIES.
---------------------------------------------------------------------------------------------------
-- Fa una relaci de despeses de membres de cambres parlamentries en funci dels parmetres 
-- demanats, on en els noms pot fer-se una cerca per paraula. Exemple i_cambra = catalunya
---------------------------------------------------------------------------------------------------
PROCEDURE DESPESES_MEMBRES (
    i_dataIni       IN  VARCHAR2,  -- Data inicial de despesa amb format 'dd/mm/yyyy'
    i_dataFin       IN  VARCHAR2,  -- Data final de despesa amb format 'dd/mm/yyyy'
    i_importIni     IN  NUMBER,    -- Import brut inicial de la despesa o 0 per a tots
    i_importFin     IN  NUMBER,    -- Import brut final de la despesa o 0 per a tots
    i_cognoms       IN  VARCHAR2,  -- Cognoms del membre de cambra parlamentria o cap per a tots
    i_nom           IN  VARCHAR2,  -- Nom del membre de cambra parlamentria o cap per a tots
    i_partit        IN  VARCHAR2,  -- Nom del partit poltic o cap per a tots
    i_categoria     IN  VARCHAR2,  -- Nom de la categoria general de la despesa
    i_pagament      IN  VARCHAR2,  -- Nom de la forma de pagament de la despesa
    i_situacio      IN  VARCHAR2,  -- Nom de la situaci de la despesa
    i_descripcio    IN  VARCHAR2,  -- Descripci de la despesa
    i_justificacio  IN  VARCHAR2,  -- Descripci de la despesa
    o_rsp          OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
);


END;
/

---------------------------------------------------------------------------------------------------
-- ***************************     C O S    D E L    P A Q U E T      ************************** --
---------------------------------------------------------------------------------------------------

CREATE OR REPLACE PACKAGE BODY DCE.CON IS

-- *** OBSERVACI SOBRE CURSORS ******************************************************************
-- En les segents consultes s'ha recorregut els cursos mitjanant FOR . IN per mostrar que amb
-- aquest tipus d'iteraci s'executa automticament el OPEN i el CLOSE del cursor i que es pot  
-- accedir directament a la informaci de les tuples sense utilitzar variables ni INTO... 
-- ************************************************************************************************

---------------------------------------------------------------------------------------------------
-- *** DESPESES_01.
---------------------------------------------------------------------------------------------------
/* Donada una cambra parlamentria, un any i un membre de la cambra: el llistat de totes les 
   despeses associades al membre de la cambra en aquell any.
   Tot aix ordenat de forma descendent pel valor de cada despesa.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria
   - L'existncia del nmero de membre en la cambra indicada
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_01 (
    i_cambra       IN  NUMBER,    -- Codi identificador bbdd de la cambra parlamentria
    i_membre       IN  NUMBER,    -- Nmero de membre en la cambra
    i_any          IN  NUMBER,    -- Any de les despeses en format yyyy
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra       EXCEPTION; -- No s'ha indicat el parmetre i_cambra
    exc_noMembre       EXCEPTION; -- No s'ha indicat el parmetre i_membre
    exc_noAny          EXCEPTION; -- No s'ha indicat el parmetre i_any

    -- Variables per a la consulta
    v_cambra     VARCHAR2(50);    -- Nom de la cambra parlamentria
    v_membre     VARCHAR2(70);    -- Nom i cognoms del membre de la cambra
    v_sumImport  NUMBER(14,2);    -- Suma total del import
    v_sumIva     NUMBER(14,2);    -- Suma total del IVA
    v_sumTotal   NUMBER(14,2);    -- Suma total del total
    v_compte     NUMBER(6);       -- Compte total de files de despeses
    v_seleccio   VARCHAR2(200);   -- Criteris de seleci que acompanyen al ttol
    v_sentencia  NUMBER(1);       -- Nmero de sentncia SQL   
    
    -- Cursor per a despeses amb el resultat de la consulta
    CURSOR C_despeses(p_cambra NUMBER, p_membre NUMBER, p_any NUMBER)
    IS
       SELECT "Id.Despesa", "Import brut", "Tipus IVA", "Import IVA", "Import net", 
           "TxData", "Descripci"
       FROM dce.vis_despeses_membres 
       WHERE "Id.Cambra" = p_cambra AND "Codi" = p_membre AND "Any" = p_any
       ORDER BY "Import brut" DESC;

BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces   := 'CON.DESPESES_01';
    log_entrada  := '<' || i_cambra || '> <' || i_membre || '> <' || i_any || '>';
    log_resultat := 'OK. RELACI DE DESPESES DEL ANY DEL MEMBRE DE LA CAMBRA EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_cambra     := '';
    v_membre     := '';
    v_sumImport  := 0;
    v_sumIva     := 0;
    v_sumTotal   := 0;
    v_compte     := 0;
    v_seleccio   := '';
    v_sentencia  := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '') THEN RAISE exc_noCambra; END IF;
    IF (i_membre IS NULL OR i_membre = '') THEN RAISE exc_noMembre; END IF;
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Comprovaci de l'existencia de la cambra parlamentria
    v_sentencia := 1;
    SELECT CAMBRA INTO v_cambra FROM CAMBRA WHERE IBD = i_cambra;
    
    -- Comprovaci de l'existencia del membre de la cambra
    v_sentencia := 2;
    SELECT NOM || ' ' || COGNOMS INTO v_membre FROM MEMBRE_CAMBRA 
        WHERE NUMERO = i_membre AND CAMBRA = i_cambra;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Cambra ' || i_cambra || ' ' || v_cambra || ' / Membre ' ||
            i_membre || ' ' || v_membre || ' / Any ' || i_any ||
            '   - Ordenaci: import de la despesa (desc)';
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',138,'-'));
        DBMS_OUTPUT.PUT_LINE('*** DESPESES_01. DESPESES ASSOCIADES A UN/A MEMBRE D''UNA' ||
            ' CAMBRA PARLAMENTRIA EN UN ANY ***   ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',138,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',138,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Id.Despesa',11) || 
                             RPAD('Data',11) ||
                             RPAD('Descripci',51) ||
                             RPAD('Import',19) ||
                             RPAD('% IVA',9) ||
                             RPAD('Import IVA',19) ||
                             RPAD('Total',20));
        DBMS_OUTPUT.PUT_LINE('---------- ' || 
                             '---------- ' ||
                             '-------------------------------------------------- ' ||
                             '------------------ ' ||
                             '-------- ' ||
                             '------------------ ' ||
                             '------------------  ');
 
        -- Bucle sobre el cursor
        FOR despesa IN C_despeses(i_cambra, i_membre, i_any)
        LOOP
            DBMS_OUTPUT.PUT_LINE(LPAD(despesa."Id.Despesa",10) || ' ' ||
                RPAD(despesa."TxData",11) ||
                RPAD(UPPER(despesa."Descripci"),51) ||
                LPAD(TO_CHAR(despesa."Import brut",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."Tipus IVA",'999D99'),8) || ' ' ||
                LPAD(TO_CHAR(despesa."Import IVA",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."Import net",'9G999G999G990D99'),18));
                v_sumImport := v_sumImport + despesa."Import brut";
                v_sumIva    := v_sumIva + despesa."Import IVA";
                v_sumTotal  := v_sumTotal + despesa."Import net";
                v_compte    := v_compte + 1;
        END LOOP; 
        
        -- Peu de la relaci
        DBMS_OUTPUT.PUT_LINE('           ' || 
                             RPAD(' ',11) ||
                             RPAD(' ',51) ||
                             '------------------ ' ||
                             '         ' ||
                             '------------------ ' ||
                             '------------------ ');
        DBMS_OUTPUT.PUT_LINE(RPAD(TRIM(TO_CHAR(v_compte)) || ' despeses.',72) ||
                             TO_CHAR(v_sumImport,'999G999G999G990D99') ||
                             '         ' || 
                             TO_CHAR(v_sumIva,'999G999G999G990D99') || 
                             TO_CHAR(v_sumTotal,'999G999G999G990D99'));
        DBMS_OUTPUT.PUT_LINE('');
                                     
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 

    WHEN exc_noCambra THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL CODI DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noMembre THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL CODI DEL MEMBRE DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA ' || i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
         
      WHEN 2 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT EL MEMBRE NMERO ' || i_membre || ' A LA CAMBRA '
           || i_cambra || ' ' || v_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);         
    END CASE;  

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESES_01;


---------------------------------------------------------------------------------------------------
-- *** DESPESES_02.
---------------------------------------------------------------------------------------------------
/* Donada una cambra parlamentria, i un any: el llistat dels seus membres amb la despesa directa 
   que ha fet cadascun, incloent:
      - El seu nom i cognoms
      - Partit poltic actual
      - Suma de totes les seves despeses directes associades
   Tot aix ordenat de forma descendent pel valor de la suma de les despeses.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_02 (
    i_cambra       IN  NUMBER,    -- Codi identificador bbdd de la cambra parlamentria
    i_any          IN  NUMBER,    -- Any de les despeses
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra       EXCEPTION; -- No s'ha indicat el parmetre i_cambra
    exc_noAny          EXCEPTION; -- No s'ha indicat el parmetre i_any

    -- Variables per a la consulta
    v_cambra     VARCHAR2(50);    -- Nom de la cambra parlamentria
    v_sumImport  NUMBER(14,2);    -- Suma total del import
    v_sumIva     NUMBER(14,2);    -- Suma total del IVA
    v_sumTotal   NUMBER(14,2);    -- Suma total del total
    v_compte     NUMBER(6);       -- Compte total de files de despeses
    v_seleccio   VARCHAR2(200);   -- Criteris de seleci que acompanyen al ttol
    v_sentencia  NUMBER(1);       -- Nmero de sentncia SQL   
    
    -- Cursor per a despeses amb el resultat de la consulta
    CURSOR C_despeses(p_cambra NUMBER, p_any NUMBER)
    IS
       SELECT SUM("Import brut") AS "Import", 
           SUM("Import IVA") AS "Import IVA", 
           SUM("Import net") AS "Total",
           "Cognoms" || ', ' || "Nom" AS "Membre de la cambra", 
           "Partit poltic"
       FROM dce.vis_despeses_membres
       WHERE "Id.Cambra" = p_cambra AND "Any" = p_any
       GROUP BY "Cognoms" || ', ' || "Nom", "Partit poltic"
       ORDER BY 3 DESC;
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces   := 'CON.DESPESES_02';
    log_entrada  := '<' || i_cambra || '> <' || i_any || '>';
    log_resultat := 'OK. RELACI DE DESPESES DEL ANY DELS MEMBRES DE LA CAMBRA EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_cambra     := '';
    v_sumImport  := 0;
    v_sumIva     := 0;
    v_sumTotal   := 0;
    v_compte     := 0;
    v_seleccio   := '';
    v_sentencia  := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '') THEN RAISE exc_noCambra; END IF;
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Comprovaci de l'existencia de la cambra parlamentria
    v_sentencia := 1;
    SELECT CAMBRA INTO v_cambra FROM CAMBRA WHERE IBD = i_cambra;
        
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Cambra ' || i_cambra || ' ' || v_cambra || ' / Any ' || i_any ||
            '   - Ordenaci: import de la despesa (desc)';
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',143,'-'));
        DBMS_OUTPUT.PUT_LINE('*** DESPESES_02. SUMA DE LES DESPESES DE MEMBRES D''UNA' ||
            ' CAMBRA PARLAMENTRIA EN UN ANY ***   ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',143,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',143,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Membre de la cambra',37) ||
                             RPAD('Partit poltic',50) ||
                             RPAD('Import',19) ||
                             RPAD('Import IVA',19) ||
                             RPAD('Total',19));
        DBMS_OUTPUT.PUT_LINE(RPAD('-',36,'-') || ' ' ||
                             RPAD('-',49,'-') || ' ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ');
 
        -- Bucle sobre el cursor
        FOR despesa IN C_despeses(i_cambra, i_any)
        LOOP
            DBMS_OUTPUT.PUT_LINE(RPAD(SUBSTR(despesa."Membre de la cambra",1,37),37) ||
                RPAD(SUBSTR(despesa."Partit poltic",1,50),50) ||
                LPAD(TO_CHAR(despesa."Import",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."Import IVA",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."Total",'9G999G999G990D99'),18));
                v_sumImport := v_sumImport + despesa."Import";
                v_sumIva    := v_sumIva + despesa."Import IVA";
                v_sumTotal  := v_sumTotal + despesa."Total";
                v_compte    := v_compte + 1;
        END LOOP; 
        
        -- Peu de la relaci
        DBMS_OUTPUT.PUT_LINE(RPAD(' ',87) ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ');
        DBMS_OUTPUT.PUT_LINE(RPAD(TRIM(TO_CHAR(v_compte)) || ' membres.',86) ||        
                             TO_CHAR(v_sumImport,'999G999G999G990D99') ||
                             TO_CHAR(v_sumIva,'999G999G999G990D99') || 
                             TO_CHAR(v_sumTotal,'999G999G999G990D99'));
        DBMS_OUTPUT.PUT_LINE('');                             
    
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 

    WHEN exc_noCambra THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL CODI DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA ' || i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    END CASE;  

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESES_02;

---------------------------------------------------------------------------------------------------
-- *** DESPESES_03.
---------------------------------------------------------------------------------------------------
/* Donat un any: el llistat de la despesa total del totes les cambres parlamentries, incloent:
    - Nom de la cambra parlamentria
    - Nombre de membres de la cambra
    - Despesa mitja dels seus membres (tenint en compte noms les despeses directes realitzades
      pels membres de la cambra).
    - Suma de les despeses totals associades als membres de la cambra.
    - Suma de les despeses totals associades a la cambra parlamentria (s a dir les que no estan
      associades a un membre concret).
    - Suma de totes les despeses.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_03 (
    i_any          IN  NUMBER,    -- Any de les despeses
    i_iva          IN  CHAR,      -- Imports amb o sense IVA: S=Amb IVA, N=Sense IVA
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables pel tractament d'excepcions
   	exc_noAny          EXCEPTION; -- No s'ha indicat el parmetre i_any
   	exc_noIva          EXCEPTION; -- No s'ha indicat el parmetre i_iva
    
    -- Variables per a la consulta
    v_cambra              VARCHAR2(50);    -- Nom de la cambra parlamentria
    v_membres             NUMBER(3);       -- Nmero de membres de la cambra parlamentria
    v_sumNMembresAlta     NUMBER(5);       -- Suma del nmero de membres d'alta
    v_sumNMembres         NUMBER(5);       -- Suma del nmero dels diferents membres
    v_sumNDespesesMembres NUMBER(5);       -- Suma del nmero de despeses dels membres
    v_sumNDespesesCambra  NUMBER(5);       -- Suma del nmero de despeses de la cambra
    v_sumDespesesTotes    NUMBER(6);       -- Suma del nmero total de despeses dels membres
    v_MitjaMembres        NUMBER(14,2);    -- Mitja de la suma de totes les despeses dels membres
    v_sumImportMembres    NUMBER(14,2);    -- Suma total del import despeses personals dels membres
    v_sumIvaMembres       NUMBER(14,2);    -- Suma total del IVA despeses personals dels membres
    v_sumTotalMembres     NUMBER(14,2);    -- Suma total del total despeses personals dels membres
    v_sumImportCambra     NUMBER(14,2);    -- Suma total del import despeses generals de la cambra
    v_sumIvaCambra        NUMBER(14,2);    -- Suma total del IVA despeses generals de la cambra
    v_sumTotalCambra      NUMBER(14,2);    -- Suma total del total despeses generals de la cambra
    v_sumImportTotes      NUMBER(14,2);    -- Suma total del import de totes les despeses
    v_sumIvaTotes         NUMBER(14,2);    -- Suma total del IVA de totes les despeses
    v_sumTotalTotes       NUMBER(14,2);    -- Suma total del total de tote les despeses
    v_compte              NUMBER(6);       -- Compte total de files de despeses
    v_seleccio            VARCHAR2(200);   -- Criteris de seleci que acompanyen al ttol
    v_titol               VARCHAR2(200);   -- Ttol del llistat    
    
    -- Cursor per a despeses
    CURSOR C_despeses(p_any NUMBER)
    IS
       SELECT  "Id.Cambra" AS "idCambra", max("Cambra parlamentria") AS "cambraParlamentaria", 
           MAX("Membres") AS "nMembresAlta",
           COUNT("Id.Despesa") AS "nDespesesTotes",
           SUM(DECODE("Nmero",0,1,0)) AS "nDespesesCambra",
           SUM(DECODE("Nmero",0,0,1)) AS "nDespesesMembres",
           COUNT(DISTINCT(DECODE("Nmero",0,0,"Id.Membre")))-1 as "nMembres",
           SUM(DECODE("Nmero",0,0,"Import brut")) / 
           (DECODE((COUNT(DISTINCT(DECODE("Nmero",0,0,"Id.Membre")))-1),0,1,
                   (COUNT(DISTINCT("Id.Membre"))-1)))  AS "mitjaMembres1",
           SUM(DECODE("Nmero",0,0,"Import net")) / 
           (DECODE((COUNT(DISTINCT(DECODE("Nmero",0,0,"Id.Membre")))-1),0,1,
                   (COUNT(DISTINCT("Id.Membre"))-1)))  AS "mitjaMembres2",
           SUM(DECODE("Nmero",0,0,"Import brut")) AS "importMembres", 
           SUM(DECODE("Nmero",0,0,"Import IVA")) AS "ivaMembres",
           SUM(DECODE("Nmero",0,0,"Import net")) AS "totalMembres",
           SUM(DECODE("Nmero",0,"Import brut",0)) AS "importCambra", 
           SUM(DECODE("Nmero",0,"Import IVA",0)) AS "ivaCambra",
           SUM(DECODE("Nmero",0,"Import net",0)) AS "totalCambra",
           SUM("Import brut") AS "importTotes", 
           SUM("Import IVA") AS "ivaTotes",
           SUM("Import net") AS "totalTotes"
       FROM dce.vis_despeses WHERE "Any" = p_any
       GROUP BY "Id.Cambra"
       ORDER BY 17 DESC;
        
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces   := 'CON.DESPESES_03';
    log_entrada  := '<' || i_any || '> <' || i_iva || '>';
    log_resultat := 'OK. RELACI DE DESPESA TOTAL DE CAMBRES PARLAMENTRIES EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_cambra              := '';
    v_membres             := 0;
    v_sumNMembresAlta     := 0;
    v_sumNMembres         := 0;
    v_sumNDespesesMembres := 0;
    v_sumNDespesesCambra  := 0;
    v_MitjaMembres        := 0;
    v_sumDespesesTotes    := 0;
    v_sumImportMembres    := 0;
    v_sumIvaMembres       := 0;
    v_sumTotalMembres     := 0;
    v_sumImportCambra     := 0;
    v_sumIvaCambra        := 0;
    v_sumTotalCambra      := 0;
    v_sumImportTotes      := 0;
    v_sumIvaTotes         := 0;
    v_sumTotalTotes       := 0;
    v_compte              := 0;
    v_seleccio            := '';
    v_titol               := '*** DESPESES_03. SUMA DE LA DESPESA TOTAL DE LES CAMBRES ' ||
        'PARLAMENTRIES EN UN ANY ***   ' || v_quiquan;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    IF (i_iva IS NULL OR i_iva = '') THEN RAISE exc_noIva; END IF;
        
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Any ' || i_any;
        IF (upper(i_iva) = 'N') THEN
            v_seleccio := v_seleccio || ' / Imports sense IVA';
        ELSE
            v_seleccio := v_seleccio || ' / Imports amb IVA';        
        END IF;
        v_seleccio := v_seleccio || '   - Ordenaci: import total de la despesa (desc)';
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',163,'-'));
        DBMS_OUTPUT.PUT_LINE(v_titol);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',163,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',163,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Codi',5) ||
                             RPAD('Cambra parlamentria',50) ||
                             'Mem alta ' ||
                             'NDes ' ||
                             RPAD('Import Membres',19) ||
                             'NMem ' ||                             
                             RPAD('Mitja Membres',19) ||
                             'NDes ' ||
                             RPAD('Import Cambres',19) ||
                             'NDes ' ||                          
                             RPAD('Import totes',19));
        DBMS_OUTPUT.PUT_LINE('---- ' ||
                             RPAD('-',49,'-') || ' ' ||
                             '-------- ' ||
                             '----- ' ||
                             '------------------ ' ||
                             '----- ' ||                             
                             '------------------ ' ||
                             '----- ' ||
                             '------------------ ' ||
                             '----- ' ||
                             '------------------ ');
 
        -- Bucle sobre el cursor de despeses
        FOR despesa IN C_despeses(i_any)
        LOOP
          IF (upper(i_iva) = 'N') THEN
            DBMS_OUTPUT.PUT_LINE(
                RPAD(despesa."idCambra",5) ||
                RPAD(despesa."cambraParlamentaria",51) ||
                LPAD(TO_CHAR(despesa."nMembresAlta",'990'),7) ||' ' ||
                LPAD(TO_CHAR(despesa."nDespesesMembres",'990'),5) || ' ' ||
                LPAD(TO_CHAR(despesa."importMembres",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."nMembres",'990'),5) || ' ' ||
                LPAD(TO_CHAR(despesa."mitjaMembres1",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."nDespesesCambra",'990'),5) || ' ' ||
                LPAD(TO_CHAR(despesa."importCambra",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."nDespesesTotes",'990'),5) || ' ' ||
                LPAD(TO_CHAR(despesa."importTotes",'9G999G999G990D99'),18)); 
            v_sumNMembresAlta     := v_sumNMembresAlta + despesa."nMembresAlta";
            v_sumNMembres         := v_sumNMembres + despesa."nMembres";
            v_sumNDespesesMembres := v_sumNDespesesMembres + despesa."nDespesesMembres";
            v_sumNDespesesCambra  := v_sumNDespesesCambra + despesa."nDespesesCambra";
            v_sumDespesesTotes    := v_sumDespesesTotes + despesa."nDespesesTotes";
            v_sumImportMembres    := v_sumImportMembres + despesa."importMembres";
            v_sumImportCambra     := v_sumImportCambra + despesa."importCambra";
            v_sumImportTotes      := v_sumImportTotes + despesa."importTotes";
            v_compte              := v_compte + 1;
            IF(v_sumNMembres > 0) THEN
                v_mitjaMembres := v_sumImportMembres / v_sumNMembres; END IF;
          ELSE
            DBMS_OUTPUT.PUT_LINE(
                RPAD(despesa."idCambra",5) ||
                RPAD(despesa."cambraParlamentaria",51) ||
                LPAD(TO_CHAR(despesa."nMembresAlta",'990'),7) ||' ' ||
                LPAD(TO_CHAR(despesa."nDespesesMembres",'990'),5) || ' ' ||
                 LPAD(TO_CHAR(despesa."totalMembres",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."nMembres",'990'),5) || ' ' ||
                LPAD(TO_CHAR(despesa."mitjaMembres2",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."nDespesesCambra",'990'),5) || ' ' ||
                LPAD(TO_CHAR(despesa."totalCambra",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."nDespesesTotes",'990'),5) || ' ' ||
                LPAD(TO_CHAR(despesa."totalTotes",'9G999G999G990D99'),18)); 
            v_sumNMembresAlta     := v_sumNMembresAlta + despesa."nMembresAlta";
            v_sumNMembres         := v_sumNMembres + despesa."nMembres";
            v_sumNDespesesMembres := v_sumNDespesesMembres + despesa."nDespesesMembres";
            v_sumNDespesesCambra  := v_sumNDespesesCambra + despesa."nDespesesCambra";
            v_sumDespesesTotes    := v_sumDespesesTotes + despesa."nDespesesTotes";
            v_sumTotalMembres     := v_sumTotalMembres + despesa."totalMembres";
            v_sumTotalCambra      := v_sumTotalCambra + despesa."totalCambra";
            v_sumTotalTotes       := v_sumTotalTotes + despesa."totalTotes";
            v_compte              := v_compte + 1;          
            IF(v_sumNMembres > 0) THEN
                v_mitjaMembres := v_sumTotalMembres / v_sumNMembres; END IF;
          END IF;
        END LOOP;         
        
        -- Peu de la relaci
        DBMS_OUTPUT.PUT_LINE('     ' ||
                             RPAD(' ',50,' ') ||
                             '-------- ' ||
                             '----- ' ||
                             '------------------ ' ||
                             '----- ' ||
                             '------------------ ' ||
                             '----- ' ||
                             '------------------ ' ||
                             '----- ' ||
                             '------------------ ');

        IF (upper(i_iva) = 'N') THEN
            DBMS_OUTPUT.PUT_LINE(RPAD(TRIM(TO_CHAR(v_compte)) || ' cambres.',54) ||
                             TO_CHAR(v_sumNMembresAlta,'9999G990') ||
                             TO_CHAR(v_sumNDespesesMembres,'9G990') ||
                             TO_CHAR(v_sumImportMembres,'999G999G999G990D99') ||
                             TO_CHAR(v_sumNMembres,'9G990') ||
                             TO_CHAR(v_MitjaMembres,'999G999G999G990D99') ||
                             TO_CHAR(v_sumNDespesesCambra,'9G990') ||
                             TO_CHAR(v_sumImportCambra,'999G999G999G999D99') ||
                             TO_CHAR(v_sumDespesesTotes,'9G990') ||
                             TO_CHAR(v_sumImportTotes,'999G999G999G990D99'));
        ELSE
            DBMS_OUTPUT.PUT_LINE(RPAD(TRIM(TO_CHAR(v_compte)) || ' cambres.',54) ||
                             TO_CHAR(v_sumNMembresAlta,'9999G990') ||
                             TO_CHAR(v_sumNDespesesMembres,'9G990') ||
                             TO_CHAR(v_sumTotalMembres,'999G999G999G990D99') ||
                             TO_CHAR(v_sumNMembres,'9G990') ||
                             TO_CHAR(v_MitjaMembres,'999G999G999G990D99') ||
                             TO_CHAR(v_sumNDespesesCambra,'9G990') ||
                             TO_CHAR(v_sumTotalCambra,'999G999G999G990D99') ||
                             TO_CHAR(v_sumDespesesTotes,'9G990') ||
                             TO_CHAR(v_sumTotalTotes,'999G999G999G990D99'));        
        END IF;
        DBMS_OUTPUT.PUT_LINE('');                             

    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 

    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noIva THEN
        log_resultat := 
           'ERROR: NO S''HA INDICAT IMPORTS AMB O SENSE IVA S=AMB IVA, N=SENSE IVA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESES_03;

---------------------------------------------------------------------------------------------------
-- *** DESPESES_04.
---------------------------------------------------------------------------------------------------
/* Donat un any i una cambra parlamentria: el llistat de la despesa segmentada per categories
   de despesa, s a dir, un llistat que retorni les columnes segents:
     - Codi de la categoria de despesa.
     - Descripci de la categoria de despesa.
     - Suma de les despeses totals associades a la categoria de despesa, 
       durant lany indicat i en la cambra parlamentria indicada.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_04 (
    i_cambra       IN  NUMBER,    -- Codi identificador bbdd de la cambra parlamentria
    i_any          IN  NUMBER,    -- Any de les despeses
    i_tipus        IN  CHAR,      -- Tipus de despeses: G=Generals, P=Personals, T=Totes
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra       EXCEPTION; -- No s'ha indicat el parmetre i_cambra
    exc_noAny          EXCEPTION; -- No s'ha indicat el parmetre i_any
    exc_noTipus        EXCEPTION; -- No s'ha indicat el parmetre i_tipus
    
    -- Variables per a la consulta
    v_cambra     VARCHAR2(50);    -- Nom de la cambra parlamentria
    v_sumImport  NUMBER(14,2);    -- Suma total del import
    v_sumIva     NUMBER(14,2);    -- Suma total del IVA
    v_sumTotal   NUMBER(14,2);    -- Suma total del total
    v_compte     NUMBER(6);       -- Compte total de files de despeses
    v_seleccio   VARCHAR2(200);   -- Criteris de seleci que acompanyen al ttol
    v_sentencia  NUMBER(1);       -- Nmero de sentncia SQL   
    v_titol      VARCHAR2(200);    -- Ttol del llistat
    
    -- Cursor per a despeses generals
    CURSOR C_despesesG(p_cambra NUMBER, p_any NUMBER)
    IS
       SELECT "Codi Categoria" AS "codi", MAX("Categoria") AS "categoria",
           SUM("Import brut") AS "import", 
           SUM("Import IVA") AS "importIva", 
           SUM("Import net") AS "total"
       FROM dce.vis_despeses_cambres
       WHERE "Id.Cambra" = p_cambra AND "Any" = p_any
       GROUP BY "Codi Categoria"
       ORDER BY 3 DESC;
    
    -- Cursor per a despeses personals
    CURSOR C_despesesP(p_cambra NUMBER, p_any NUMBER)
    IS
       SELECT "Codi Categoria" AS "codi", MAX("Categoria") AS "categoria",
           SUM("Import brut") AS "import", 
           SUM("Import IVA") AS "importIva", 
           SUM("Import net") AS "total"
       FROM dce.vis_despeses_membres
       WHERE "Id.Cambra" = p_cambra AND "Any" = p_any
       GROUP BY "Codi Categoria"
       ORDER BY 3 DESC;
        
    -- Cursor per a totes les despeses
    CURSOR C_despesesT(p_cambra NUMBER, p_any NUMBER)
    IS
       SELECT "Codi Categoria" AS "codi", MAX("Categoria") AS "categoria",
           SUM("Import brut") AS "import", 
           SUM("Import IVA") AS "importIva", 
           SUM("Import net") AS "total"
       FROM dce.vis_despeses
       WHERE "Id.Cambra" = p_cambra AND "Any" = p_any
       GROUP BY "Codi Categoria"
       ORDER BY 3 DESC; 
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces   := 'CON.DESPESES_04';
    log_entrada  := '<' || i_cambra || '> <' || i_any || '>';
    log_resultat := 'OK. RELACI DE DESPESES DE CAMBRA PARLAMENTRIA PER CATEGORIA EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_cambra     := '';
    v_sumImport  := 0;
    v_sumIva     := 0;
    v_sumTotal   := 0;
    v_compte     := 0;
    v_seleccio   := '';
    v_sentencia  := 0;
    v_titol      := '*** DESPESES_04. TOTES LES DESPESES D''UNA CAMBRA' ||
       ' EN UN ANY PER CATEGORIES ***   ' || v_quiquan;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '') THEN RAISE exc_noCambra; END IF;
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    IF (i_tipus IS NULL OR i_tipus = '' OR 
       (upper(i_tipus) <> 'G' AND upper(i_tipus) <> 'P' AND upper(i_tipus) <> 'T'))
       THEN RAISE exc_noTipus; END IF;
    
    IF( upper(i_tipus) = 'G') THEN
        v_titol := '*** DESPESES_04. DESPESES GENERALS D''UNA CAMBRA' ||
        ' EN UN ANY PER CATEGORIES ***   ' || v_quiquan;
    END IF;

    IF( upper(i_tipus) = 'P') THEN
        v_titol := '*** DESPESES_04. DESPESES PERSONALS D''UNA CAMBRA' ||
        ' EN UN ANY PER CATEGORIES ***   ' || v_quiquan;
    END IF;
    
    -- Comprovaci de l'existencia de la cambra parlamentria
    v_sentencia := 1;
    SELECT cambra INTO v_cambra FROM dce.cambra WHERE ibd = i_cambra;
    
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Cambra ' || i_cambra || ' ' || v_cambra || ' / Any ' || i_any ||
            '   - Ordenaci: import de la despesa (desc)';
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',115,'-'));
        DBMS_OUTPUT.PUT_LINE(v_titol);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',115,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',115,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
                
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Codi',9) ||
                             RPAD('Categoria',50) ||
                             RPAD('Import',19) ||
                             RPAD('Import IVA',19) ||
                             RPAD('Total',20));                             
        DBMS_OUTPUT.PUT_LINE('-------- ' ||
                             RPAD('-',49,'-') || ' ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ');
 
        -- Bucle sobre el cursor corresponent
        CASE upper(i_tipus)
        WHEN 'G' THEN
            FOR despesa IN C_despesesG(i_cambra, i_any)
            LOOP
                DBMS_OUTPUT.PUT_LINE(
                    RPAD(despesa."codi",9) ||
                    RPAD(despesa."categoria",50) ||
                    LPAD(TO_CHAR(despesa."import",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."importIva",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."total",'9G999G999G990D99'),18));
                    v_sumImport := v_sumImport + despesa."import";
                    v_sumIva    := v_sumIva + despesa."importIva";
                    v_sumTotal  := v_sumTotal + despesa."total";
                    v_compte    := v_compte + 1;
            END LOOP;         
        
        WHEN 'P' THEN
            FOR despesa IN C_despesesP(i_cambra, i_any)
            LOOP
                DBMS_OUTPUT.PUT_LINE(
                    RPAD(despesa."codi",9) ||
                    RPAD(despesa."categoria",50) ||
                    LPAD(TO_CHAR(despesa."import",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."importIva",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."total",'9G999G999G990D99'),18));
                    v_sumImport := v_sumImport + despesa."import";
                    v_sumIva    := v_sumIva + despesa."importIva";
                    v_sumTotal  := v_sumTotal + despesa."total";
                    v_compte    := v_compte + 1;
            END LOOP;         
        
        ELSE
            FOR despesa IN C_despesesT(i_cambra, i_any)
            LOOP
                DBMS_OUTPUT.PUT_LINE(
                    RPAD(despesa."codi",9) ||
                    RPAD(despesa."categoria",50) ||
                    LPAD(TO_CHAR(despesa."import",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."importIva",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."total",'9G999G999G990D99'),18));
                    v_sumImport := v_sumImport + despesa."import";
                    v_sumIva    := v_sumIva + despesa."importIva";
                    v_sumTotal  := v_sumTotal + despesa."total";
                    v_compte    := v_compte + 1;
            END LOOP;         
       
        END CASE;

        -- Peu de la relaci
        DBMS_OUTPUT.PUT_LINE('         ' ||
                             RPAD(' ',50,' ') ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ');
        DBMS_OUTPUT.PUT_LINE(RPAD(TRIM(TO_CHAR(v_compte)) || ' categories.',58) ||
                             TO_CHAR(v_sumImport,'999G999G999G990D99') ||
                             TO_CHAR(v_sumIva,'999G999G999G990D99') || 
                             TO_CHAR(v_sumTotal,'999G999G999G990D99'));
        DBMS_OUTPUT.PUT_LINE('');
                                     
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 

    WHEN exc_noCambra THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL CODI DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTipus THEN
        log_resultat := 
           'ERROR: NO S''HA INDICAT EL TIPUS DE DESPESES G=GENERALS, P=PERSONALS O T=TOTES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA ' || i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    END CASE;  

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
       (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESES_04;

---------------------------------------------------------------------------------------------------
-- *** DESPESES_05.
---------------------------------------------------------------------------------------------------
/* Donada una cambra parlamentria i un any: el llistat de membres que superen la despesa mitja
   dels tots els membres daquella cambra durant lany indicat, incloent: 
     - El seu nom i cognom 
     - Partit poltic actual 
     - Suma de totes les despeses directes associades al membre de la cambra durant aquell any
     - Percentatge en que es supera la despesa mitja
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria  
   - La mitjana es calcula fent la divisi entre la suma de totes les despeses  i el nmero de 
     membres que les han fet.      
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_05 (
    i_cambra       IN  NUMBER,    -- Codi identificador bbdd de la cambra parlamentria
    i_any          IN  NUMBER,    -- Any de les despeses
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra       EXCEPTION; -- No s'ha indicat el parmetre i_cambra
    exc_noAny          EXCEPTION; -- No s'ha indicat el parmetre i_any

    -- Variables per a la consulta
    v_cambra     VARCHAR2(50);    -- Nom de la cambra parlamentria
    v_sumImport  NUMBER(14,2);    -- Suma total del import
    v_sumIva     NUMBER(14,2);    -- Suma total del IVA
    v_sumTotal   NUMBER(14,2);    -- Suma total del total
    v_mitja      NUMBER(14,2);    -- Despesa mitja de despeses
    v_compte     NUMBER(6);       -- Compte total de files de despeses
    v_seleccio   VARCHAR2(200);   -- Criteris de seleci que acompanyen al ttol
    v_sentencia  NUMBER(1);       -- Nmero de sentncia SQL   
    
    -- Cursor per a despeses amb el resultat de la consulta
    CURSOR C_despeses(p_cambra NUMBER, p_any NUMBER, p_mitja NUMBER)
    IS
    SELECT SUM("Import brut") AS "import", 
        (SUM("Import net")*100/p_mitja) - 100 AS "tpcSupera",
         SUM("Import IVA") AS "importIva", 
         SUM("Import net") AS "total",
         "Cognoms" || ', ' || "Nom" AS "membreCambra", 
         "Partit poltic" AS "partitPolitic"
    FROM dce.vis_despeses_membres
    WHERE "Id.Cambra" = p_cambra AND "Any" = p_any
    GROUP BY "Cognoms" || ', ' || "Nom", "Partit poltic"
    HAVING SUM("Import net")/COUNT(DISTINCT("Id.Membre")) > p_mitja
    ORDER BY 3 DESC;
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces   := 'CON.DESPESES_05';
    log_entrada  := '<' || i_cambra || '> <' || i_any || '>';
    log_resultat := 'OK. RELACI DE MEMBRES QUE SUPEREN LA MITJA EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_cambra     := '';
    v_sumImport  := 0;
    v_sumIva     := 0;
    v_mitja      := 0;
    v_sumTotal   := 0;
    v_compte     := 0;
    v_seleccio   := '';
    v_sentencia  := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '') THEN RAISE exc_noCambra; END IF;
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    
    -- Comprovaci de l'existencia de la cambra parlamentria
    v_sentencia := 1;
    SELECT cambra INTO v_cambra FROM dce.cambra WHERE ibd = i_cambra;

    -- Obtenim la despesa mitja dels membres de la cambra i any indicats
    v_sentencia := 2;
    BEGIN
       SELECT SUM("Import net")/COUNT(DISTINCT("Id.Membre")) 
          INTO v_mitja FROM dce.vis_despeses_membres
       WHERE "Id.Cambra" = i_cambra AND "Any" = i_any
       GROUP BY "Id.Cambra";
       
       -- No s'ha pogut obtenir la mitja perqu no hi ha despeses
       EXCEPTION
       WHEN NO_DATA_FOUND THEN
           v_mitja := 0;
    END;
               
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Cambra ' || i_cambra || ' ' || v_cambra || ' / Any ' || i_any ||
            '    / despesa mitja ' || trim(to_char(v_mitja,'9G999G999G990D99')) || 
            '   - Ordenaci: import de la despesa (desc)';
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',151,'-'));
        DBMS_OUTPUT.PUT_LINE('*** DESPESES_05. MEMBRES D''UNA CAMBRA PARLAMENTRIA QUE' ||
            ' SUPEREN LA MITJA DE DESPESA EN UN ANY INDICAT ***   ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',151,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',151,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Membre de la cambra',37) ||
                             RPAD('Partit poltic',50) ||
                             RPAD('Import',19) ||
                             RPAD('Import IVA',19) ||
                             RPAD('Total',19) ||
                             RPAD('% ms',8));
        DBMS_OUTPUT.PUT_LINE(RPAD('-',36,'-') || ' ' ||
                             RPAD('-',49,'-') || ' ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------- ');
         
        -- Bucle sobre el cursor
        FOR despesa IN C_despeses(i_cambra, i_any, v_mitja)
        LOOP
            DBMS_OUTPUT.PUT_LINE(RPAD(SUBSTR(despesa."membreCambra",1,37),37) ||
                RPAD(SUBSTR(despesa."partitPolitic",1,50),50) ||
                LPAD(TO_CHAR(despesa."import",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."importIva",'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(despesa."total",'9G999G999G990D99'),18) ||
                LPAD(TO_CHAR(despesa."tpcSupera",'990D00'),7));
                v_sumImport := v_sumImport + despesa."import";
                v_sumIva    := v_sumIva + despesa."importIva";
                v_sumTotal  := v_sumTotal + despesa."total";
                v_compte    := v_compte + 1;
        END LOOP; 
        
        -- Peu de la relaci
        DBMS_OUTPUT.PUT_LINE(RPAD(' ',87) ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '        ');                             
        DBMS_OUTPUT.PUT_LINE(RPAD(TRIM(TO_CHAR(v_compte)) || ' membres.',86) ||        
                             TO_CHAR(v_sumImport,'999G999G999G990D99') ||
                             TO_CHAR(v_sumIva,'999G999G999G990D99') || 
                             TO_CHAR(v_sumTotal,'999G999G999G990D99') ||
                             '        ');
        DBMS_OUTPUT.PUT_LINE('');                             
    
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 

    WHEN exc_noCambra THEN
        log_resultat := 'ERROR: NO S''HA INDICAT EL CODI DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_sentencia
      WHEN 1 THEN
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA ' || i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
      WHEN 2 THEN
        log_resultat := 'ERROR: NO S''HAN TROBAT DESPESES PER A LA CAMBRA ' ||
                        'PARLAMENTRIA ' || i_cambra;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);      
    END CASE;  

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESES_05;

---------------------------------------------------------------------------------------------------
-- *** DESPESES_06.
---------------------------------------------------------------------------------------------------
/* Donat un any:el llistat de totes les cambres parlamentries amb el seu estat comptable,incloent:
     - Nom de la cambra parlamentria
     - Nombre de membres de la cambra
     - Suma de totes les despeses pendents daprovaci.
     - Suma de totes les despeses aprovades.
     - Suma de totes les despeses pendents dabonament.
     - Suma de totes les despeses abonades.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_06 (
    i_any           IN  NUMBER,    -- Any de les despeses
    i_tipus         IN  CHAR,      -- Tipus de despeses: G=Generals, P=Personals, T=Totes
    i_iva           IN  CHAR,      -- Imports amb IVA o sense: S=Amb IVA, N=Sense IVA
    o_rsp          OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny           EXCEPTION; -- No s'ha indicat el parmetre i_any
    exc_noTipus         EXCEPTION; -- No s'ha indicat el parmetre i_tipus
    exc_noIva           EXCEPTION; -- No s'ha indicat el parmetre i_iva
    
    -- Variables per a la consulta
    v_sumImportPAprovar NUMBER(14,2);    -- Suma total del import pendent d'aprovar
    v_sumIvaPAprovar    NUMBER(14,2);    -- Suma total del IVA pendent d'aprovar
    v_sumTotalPAprovar  NUMBER(14,2);    -- Suma total del total pendent d'aprovar
    v_sumImportAprovat  NUMBER(14,2);    -- Suma total del import aprovat
    v_sumIvaAprovat     NUMBER(14,2);    -- Suma total del IVA aprovat
    v_sumTotalAprovat   NUMBER(14,2);    -- Suma total del total aprovat
    v_sumImportPAbonar  NUMBER(14,2);    -- Suma total del import pendent abonar
    v_sumIvaPAbonar     NUMBER(14,2);    -- Suma total del IVA pendent abonar
    v_sumTotalPAbonar   NUMBER(14,2);    -- Suma total del total pendent abonar
    v_sumImportAbonat   NUMBER(14,2);    -- Suma total del import abonat
    v_sumIvaAbonat      NUMBER(14,2);    -- Suma total del IVA abonat
    v_sumTotalAbonat    NUMBER(14,2);    -- Suma total del total abonat
    v_membres           NUMBER(4);       -- Suma total dels membres d'alta
    v_compte            NUMBER(6);       -- Compte total de files de despeses
    v_seleccio          VARCHAR2(200);   -- Criteris de seleci que acompanyen al ttol
    v_titol             VARCHAR2(200);   -- Ttol del llistat
      
    -- Cursor per a despeses generals
    CURSOR C_despesesG(p_any NUMBER)
    IS
       SELECT "Id.Cambra" AS "codi", MAX("Cambra parlamentria") as "cambraParlamentaria", 
           MAX("Membres") AS "membres",
           SUM(DECODE("Id.Situaci",1,"Import brut",0)) AS "brutPendentAprovar", 
           SUM(DECODE("Id.Situaci",1,"Import IVA",0)) AS "ivaPendentAprovar", 
           SUM(DECODE("Id.Situaci",1,"Import net",0)) AS "netPendentAprovar", 
           SUM(DECODE("Id.Situaci",2,"Import brut",0)) AS "brutAprovat", 
           SUM(DECODE("Id.Situaci",2,"Import IVA",0)) AS "ivaAprovat", 
           SUM(DECODE("Id.Situaci",2,"Import net",0)) AS "netAprovat", 
           SUM(DECODE("Id.Situaci",3,"Import brut",0)) AS "brutPendentAbonar", 
           SUM(DECODE("Id.Situaci",3,"Import IVA",0)) AS "ivaPendentAbonar", 
           SUM(DECODE("Id.Situaci",3,"Import net",0)) AS "netPendentAbonar", 
           SUM(DECODE("Id.Situaci",4,"Import brut",0)) AS "brutAbonat", 
           SUM(DECODE("Id.Situaci",4,"Import IVA",0)) AS "ivaAbonat", 
           SUM(DECODE("Id.Situaci",4,"Import net",0)) AS "netAbonat"
       FROM dce.vis_despeses_cambres
       WHERE "Any" = p_any
       GROUP BY "Id.Cambra"
       ORDER BY 2;

    -- Cursor per a despeses personals
    CURSOR C_despesesP(p_any NUMBER)
    IS
       SELECT "Id.Cambra" AS "codi", MAX("Cambra parlamentria") as "cambraParlamentaria", 
           MAX("Membres") AS "membres",
           SUM(DECODE("Id.Situaci",1,"Import brut",0)) AS "brutPendentAprovar", 
           SUM(DECODE("Id.Situaci",1,"Import IVA",0)) AS "ivaPendentAprovar", 
           SUM(DECODE("Id.Situaci",1,"Import net",0)) AS "netPendentAprovar", 
           SUM(DECODE("Id.Situaci",2,"Import brut",0)) AS "brutAprovat", 
           SUM(DECODE("Id.Situaci",2,"Import IVA",0)) AS "ivaAprovat", 
           SUM(DECODE("Id.Situaci",2,"Import net",0)) AS "netAprovat", 
           SUM(DECODE("Id.Situaci",3,"Import brut",0)) AS "brutPendentAbonar", 
           SUM(DECODE("Id.Situaci",3,"Import IVA",0)) AS "ivaPendentAbonar", 
           SUM(DECODE("Id.Situaci",3,"Import net",0)) AS "netPendentAbonar", 
           SUM(DECODE("Id.Situaci",4,"Import brut",0)) AS "brutAbonat", 
           SUM(DECODE("Id.Situaci",4,"Import IVA",0)) AS "ivaAbonat", 
           SUM(DECODE("Id.Situaci",4,"Import net",0)) AS "netAbonat"
       FROM dce.vis_despeses_membres
       WHERE "Any" = p_any
       GROUP BY "Id.Cambra"
       ORDER BY 2;

    -- Cursor per a totes les despeses
    CURSOR C_despesesT(p_any NUMBER)
    IS
       SELECT "Id.Cambra" AS "codi", MAX("Cambra parlamentria") as "cambraParlamentaria", 
           MAX("Membres") AS "membres",
           SUM(DECODE("Id.Situaci",1,"Import brut",0)) AS "brutPendentAprovar", 
           SUM(DECODE("Id.Situaci",1,"Import IVA",0)) AS "ivaPendentAprovar", 
           SUM(DECODE("Id.Situaci",1,"Import net",0)) AS "netPendentAprovar", 
           SUM(DECODE("Id.Situaci",2,"Import brut",0)) AS "brutAprovat", 
           SUM(DECODE("Id.Situaci",2,"Import IVA",0)) AS "ivaAprovat", 
           SUM(DECODE("Id.Situaci",2,"Import net",0)) AS "netAprovat", 
           SUM(DECODE("Id.Situaci",3,"Import brut",0)) AS "brutPendentAbonar", 
           SUM(DECODE("Id.Situaci",3,"Import IVA",0)) AS "ivaPendentAbonar", 
           SUM(DECODE("Id.Situaci",3,"Import net",0)) AS "netPendentAbonar", 
           SUM(DECODE("Id.Situaci",4,"Import brut",0)) AS "brutAbonat", 
           SUM(DECODE("Id.Situaci",4,"Import IVA",0)) AS "ivaAbonat", 
           SUM(DECODE("Id.Situaci",4,"Import net",0)) AS "netAbonat"
       FROM dce.vis_despeses
       WHERE "Any" = p_any
       GROUP BY "Id.Cambra"
       ORDER BY 2;
             
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces          := 'CON.DESPESES_06';
    log_entrada         := '<' || i_any || '> <' || i_tipus || '> <' || i_iva || '>' ;
    log_resultat        := 'OK. RELACI DE DESPESES DE CAMBRES PARLAMENTRIES PER ' ||
                           'SITUACI EFECTUADA';
    log_abans           := '';
    log_despres         := '';
    log_error           := 'S';
    log_usuari          := USER;
    log_data            := SYSDATE;
    o_rsp               := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_sumImportPAprovar := 0;
    v_sumIvaPAprovar    := 0;
    v_sumTotalPAprovar  := 0;
    v_sumImportAprovat  := 0;
    v_sumIvaAprovat     := 0;
    v_sumTotalAprovat   := 0;
    v_sumImportPAbonar  := 0;
    v_sumIvaPAbonar     := 0;
    v_sumTotalPAbonar   := 0;
    v_sumImportAbonat   := 0;
    v_sumIvaAbonat      := 0;
    v_sumTotalAbonat    := 0;
    v_membres           := 0;
    v_compte            := 0;
    v_seleccio          := '';
    v_titol             := '*** DESPESES_06. SUMA DE TOTES LES DESPESES DE LES CAMBRES ' ||
        'EN UN ANY PER SITUACI COMPTABLE ***   ' || v_quiquan;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    IF (i_tipus IS NULL OR i_tipus = '' OR 
       (upper(i_tipus) <> 'G' AND upper(i_tipus) <> 'P' AND upper(i_tipus) <> 'T'))
       THEN RAISE exc_noTipus; END IF;
    IF (i_iva IS NULL OR i_iva = '') THEN RAISE exc_noIva; END IF;

    IF( upper(i_tipus) = 'G') THEN
        v_titol := '*** DESPESES_06. SUMA DE LES DESPESES GENERALS DE LES CAMBRES ' ||
            'EN UN ANY PER SITUACI COMPTABLE ***   ' || v_quiquan;
    END IF;

    IF( upper(i_tipus) = 'P') THEN
        v_titol := '*** DESPESES_06. SUMA DE LES DESPESES PERSONALS DE LES CAMBRES ' ||
            'EN UN ANY PER SITUACI COMPTABLE ***   ' || v_quiquan;
    END IF;
      
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Any ' || i_any;
        IF (UPPER(i_iva) = 'N') THEN
            v_seleccio := v_seleccio || ' / Imports sense IVA';
        ELSE
            v_seleccio := v_seleccio || ' / Imports amb IVA';        
        END IF;

        v_seleccio := v_seleccio || 
            '   - Ordenaci: alfabtica per cambra parlamentria';
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',139,'-'));
        DBMS_OUTPUT.PUT_LINE(v_titol);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',139,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',139,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Codi',5) ||
                             RPAD('Cambra parlamentria',50) ||
                             RPAD('Mem.Alta',9) ||
                             RPAD('Pendent aprovar',19) ||
                             RPAD('Aprovat',19) ||
                             RPAD('Pendent abonar',19) ||
                             RPAD('Abonat',19));
        DBMS_OUTPUT.PUT_LINE('---- ' ||
                             RPAD('-',49,'-') || ' ' ||
                             '-------- ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ');        
 
        -- Bucle sobre el cursor corresponent
        CASE upper(i_tipus)
        WHEN 'G' THEN
            FOR despesa IN C_despesesG(i_any)
            LOOP
              IF (UPPER(i_iva) = 'N') THEN  
                DBMS_OUTPUT.PUT_LINE(
                  LPAD(despesa."codi",4) || ' ' ||
                  RPAD(despesa."cambraParlamentaria",50) ||
                  LPAD(despesa."membres",8) || ' ' || 
                  LPAD(TO_CHAR(despesa."brutPendentAprovar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."brutAprovat",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."brutPendentAbonar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."brutAbonat",'9G999G999G990D99'),18));
                  v_sumImportPAprovar := v_sumImportPAprovar + despesa."brutPendentAprovar";
                  v_sumImportAprovat  := v_sumImportAprovat + despesa."brutAprovat";
                  v_sumImportPAbonar  := v_sumImportPAbonar + despesa."brutPendentAbonar";
                  v_sumImportAbonat   := v_sumImportAbonat + despesa."brutAbonat";
                  v_membres           := v_membres + despesa."membres";
                  v_compte            := v_compte + 1;
              ELSE
                DBMS_OUTPUT.PUT_LINE(
                  LPAD(despesa."codi",4) || ' ' ||
                  RPAD(despesa."cambraParlamentaria",50) ||
                  LPAD(despesa."membres",8) || ' ' || 
                  LPAD(TO_CHAR(despesa."netPendentAprovar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."netAprovat",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."netPendentAbonar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."netAbonat",'9G999G999G990D99'),18));
                  v_sumTotalPAprovar  := v_sumTotalPAprovar + despesa."netPendentAprovar";
                  v_sumTotalAprovat   := v_sumTotalAprovat + despesa."netAprovat";
                  v_sumTotalPAbonar   := v_sumTotalPAbonar + despesa."netPendentAbonar";
                  v_sumTotalAbonat    := v_sumTotalAbonat + despesa."netAbonat";
                  v_membres           := v_membres + despesa."membres";
                  v_compte            := v_compte + 1;              
              END IF;
            END LOOP;         
        
        WHEN 'P' THEN
            FOR despesa IN C_despesesP(i_any)
            LOOP
              IF (UPPER(i_iva) = 'N') THEN  
                DBMS_OUTPUT.PUT_LINE(
                  LPAD(despesa."codi",4) || ' ' ||
                  RPAD(despesa."cambraParlamentaria",50) ||
                  LPAD(despesa."membres",8) || ' ' || 
                  LPAD(TO_CHAR(despesa."brutPendentAprovar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."brutAprovat",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."brutPendentAbonar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."brutAbonat",'9G999G999G990D99'),18));
                  v_sumImportPAprovar := v_sumImportPAprovar + despesa."brutPendentAprovar";
                  v_sumImportAprovat  := v_sumImportAprovat + despesa."brutAprovat";
                  v_sumImportPAbonar  := v_sumImportPAbonar + despesa."brutPendentAbonar";
                  v_sumImportAbonat   := v_sumImportAbonat + despesa."brutAbonat";
                  v_membres           := v_membres + despesa."membres";
                  v_compte            := v_compte + 1;
              ELSE
                DBMS_OUTPUT.PUT_LINE(
                  LPAD(despesa."codi",4) || ' ' ||
                  RPAD(despesa."cambraParlamentaria",50) ||
                  LPAD(despesa."membres",8) || ' ' || 
                  LPAD(TO_CHAR(despesa."netPendentAprovar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."netAprovat",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."netPendentAbonar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."netAbonat",'9G999G999G990D99'),18));
                  v_sumTotalPAprovar  := v_sumTotalPAprovar + despesa."netPendentAprovar";
                  v_sumTotalAprovat   := v_sumTotalAprovat + despesa."netAprovat";
                  v_sumTotalPAbonar   := v_sumTotalPAbonar + despesa."netPendentAbonar";
                  v_sumTotalAbonat    := v_sumTotalAbonat + despesa."netAbonat";
                  v_membres           := v_membres + despesa."membres";
                  v_compte            := v_compte + 1;              
              END IF;
            END LOOP;                 
        ELSE
            FOR despesa IN C_despesesT(i_any)
            LOOP
              IF (UPPER(i_iva) = 'N') THEN  
                DBMS_OUTPUT.PUT_LINE(
                  LPAD(despesa."codi",4) || ' ' ||
                  RPAD(despesa."cambraParlamentaria",50) ||
                  LPAD(despesa."membres",8) || ' ' || 
                  LPAD(TO_CHAR(despesa."brutPendentAprovar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."brutAprovat",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."brutPendentAbonar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."brutAbonat",'9G999G999G990D99'),18));
                  v_sumImportPAprovar := v_sumImportPAprovar + despesa."brutPendentAprovar";
                  v_sumImportAprovat  := v_sumImportAprovat + despesa."brutAprovat";
                  v_sumImportPAbonar  := v_sumImportPAbonar + despesa."brutPendentAbonar";
                  v_sumImportAbonat   := v_sumImportAbonat + despesa."brutAbonat";
                  v_membres           := v_membres + despesa."membres";
                  v_compte            := v_compte + 1;
              ELSE
                DBMS_OUTPUT.PUT_LINE(
                  LPAD(despesa."codi",4) || ' ' ||
                  RPAD(despesa."cambraParlamentaria",50) ||
                  LPAD(despesa."membres",8) || ' ' || 
                  LPAD(TO_CHAR(despesa."netPendentAprovar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."netAprovat",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."netPendentAbonar",'9G999G999G990D99'),18) || ' ' ||
                  LPAD(TO_CHAR(despesa."netAbonat",'9G999G999G990D99'),18));
                  v_sumTotalPAprovar  := v_sumTotalPAprovar + despesa."netPendentAprovar";
                  v_sumTotalAprovat   := v_sumTotalAprovat + despesa."netAprovat";
                  v_sumTotalPAbonar   := v_sumTotalPAbonar + despesa."netPendentAbonar";
                  v_sumTotalAbonat    := v_sumTotalAbonat + despesa."netAbonat";
                  v_membres           := v_membres + despesa."membres";
                  v_compte            := v_compte + 1;              
              END IF;
            END LOOP;         
       
        END CASE;

        -- Peu de la relaci
        DBMS_OUTPUT.PUT_LINE('     ' ||
                             RPAD(' ',50,' ') ||
                             '-------- ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ');
        IF (UPPER(i_iva) = 'N') THEN  
            DBMS_OUTPUT.PUT_LINE(RPAD(TRIM(TO_CHAR(v_compte)) || ' cambres.',57) ||
                             TO_CHAR(v_membres,'9G990') ||  
                             TO_CHAR(v_sumImportPAprovar,'999G999G999G990D99') ||
                             TO_CHAR(v_sumImportAprovat,'999G999G999G990D99') ||
                             TO_CHAR(v_sumImportPAbonar,'999G999G999G990D99') ||
                             TO_CHAR(v_sumImportAbonat,'999G999G999G990D99'));
        ELSE
            DBMS_OUTPUT.PUT_LINE(RPAD(TRIM(TO_CHAR(v_compte)) || ' cambres.',57) ||
                             TO_CHAR(v_membres,'9G990') ||  
                             TO_CHAR(v_sumTotalPAprovar,'999G999G999G990D99') ||
                             TO_CHAR(v_sumTotalAprovat,'999G999G999G990D99') ||
                             TO_CHAR(v_sumTotalPAbonar,'999G999G999G990D99') ||
                             TO_CHAR(v_sumTotalAbonat,'999G999G999G990D99'));
        END IF;
        DBMS_OUTPUT.PUT_LINE('');
                                     
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 

    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTipus THEN
        log_resultat := 
           'ERROR: NO S''HA INDICAT EL TIPUS DE DESPESES G=GENERALS, P=PERSONALS O T=TOTES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noIva THEN
        log_resultat := 
           'ERROR: NO S''HA INDICAT IMPORTS AMB O SENSE IVA S=AMB IVA, N=SENSE IVA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESES_06;

---------------------------------------------------------------------------------------------------
-- *** DESPESES_06b.
---------------------------------------------------------------------------------------------------
/* Donat un any: el llistat de totes les cambres parlamentries amb el seu estat comptable 
   categoritzat,incloent:
     - Nom de la cambra parlamentria
     - Situaci comptable
     - Suma de les despeses totals associades de cada situaci comptable de cada cambra durant 
       lany indicat
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESES_06b (
    i_any          IN  NUMBER,    -- Any de les despeses
    i_tipus        IN  CHAR,      -- Tipus de despeses: G=Generals, P=Personals, T=Totes
    o_rsp         OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables pel tractament d'excepcions
    exc_noAny          EXCEPTION; -- No s'ha indicat el parmetre i_any
    exc_noTipus        EXCEPTION; -- No s'ha indicat el parmetre i_tipus
    
    -- Variables per a la consulta
    v_sumImport  NUMBER(14,2);    -- Suma total del import
    v_sumIva     NUMBER(14,2);    -- Suma total del IVA
    v_sumTotal   NUMBER(14,2);    -- Suma total del total
    v_compte     NUMBER(6);       -- Compte total de files de despeses
    v_seleccio   VARCHAR2(200);   -- Criteris de seleci que acompanyen al ttol
    v_titol      VARCHAR2(200);   -- Ttol del llistat
      
    -- Cursor per a despeses generals
    CURSOR C_despesesG(p_any NUMBER)
    IS
       SELECT "Id.Cambra" AS "codi", MAX("Cambra parlamentria") as "cambraParlamentaria", 
           "Situaci" AS "situacio",
           SUM("Import brut") AS "import", 
           SUM("Import IVA") AS "importIva", 
           SUM("Import net") AS "total"
       FROM dce.vis_despeses_cambres
       WHERE "Any" = p_any
       GROUP BY "Id.Cambra", "Situaci"
       ORDER BY 2,3;

    -- Cursor per a despeses personals
    CURSOR C_despesesP(p_any NUMBER)
    IS
       SELECT "Id.Cambra" AS "codi", MAX("Cambra parlamentria") as "cambraParlamentaria", 
           "Situaci" AS "situacio",
           SUM("Import brut") AS "import", 
           SUM("Import IVA") AS "importIva", 
           SUM("Import net") AS "total"
       FROM dce.vis_despeses_membres
       WHERE "Any" = p_any
       GROUP BY "Id.Cambra", "Situaci"
       ORDER BY 2,3;
    
    -- Cursor per a totes les despeses
    CURSOR C_despesesT(p_any NUMBER)
    IS
       SELECT "Id.Cambra" AS "codi", MAX("Cambra parlamentria") as "cambraParlamentaria", 
           "Situaci" AS "situacio",
           SUM("Import brut") AS "import", 
           SUM("Import IVA") AS "importIva", 
           SUM("Import net") AS "total"
       FROM dce.vis_despeses
       WHERE "Any" = p_any
       GROUP BY "Id.Cambra", "Situaci"
       ORDER BY 2,3;
         
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces   := 'CON.DESPESES_06b';
    log_entrada  := '<' || i_any || '>';
    log_resultat := 'OK. RELACI DE DESPESES DE CAMBRES PARLAMENTRIES PER ' ||
        'SITUACI COMPTABLE EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Inicialitzaci de les variables per a la consulta
    v_sumImport  := 0;
    v_sumIva     := 0;
    v_sumTotal   := 0;
    v_compte     := 0;
    v_seleccio   := '';
    v_titol      := '*** DESPESES_06b. SUMA DE TOTES LES DESPESES DE LES CAMBRES ' ||
        'EN UN ANY PER SITUACI COMPTABLE ***   ' || v_quiquan;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_any IS NULL OR i_any = '' OR i_any = 0) THEN RAISE exc_noAny; END IF;
    IF (i_tipus IS NULL OR i_tipus = '' OR 
       (upper(i_tipus) <> 'G' AND upper(i_tipus) <> 'P' AND upper(i_tipus) <> 'T'))
       THEN RAISE exc_noTipus; END IF;
    
    IF( upper(i_tipus) = 'G') THEN
        v_titol := '*** DESPESES_06b. SUMA DE LES DESPESES GENERALS DE LES CAMBRES ' ||
            'EN UN ANY PER SITUACI COMPTABLE ***   ' || v_quiquan;
    END IF;

    IF( upper(i_tipus) = 'P') THEN
        v_titol := '*** DESPESES_06b. SUMA DE LES DESPESES PERSONALS DE LES CAMBRES ' ||
            'EN UN ANY PER SITUACI COMPTABLE ***   ' || v_quiquan;
    END IF;
      
    BEGIN
        -- Criteris de selecci
        v_seleccio := 'Any ' || i_any || 
            '   - Ordenaci: alfabtica per cambra parlamentria i situaci comptable';
            
        log_abans := v_seleccio;
		
    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',141,'-'));
        DBMS_OUTPUT.PUT_LINE(v_titol);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',141,'-'));
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',141,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
        
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Codi',5) ||
                             RPAD('Cambra parlamentria',50) ||
                             RPAD('Situaci comptable',30) ||
                             RPAD('Import',19) ||
                             RPAD('Import IVA',19) ||
                             RPAD('Total',20));
        DBMS_OUTPUT.PUT_LINE('---- ' ||
                             RPAD('-',49,'-') || ' ' ||
                             RPAD('-',29,'-') || ' ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ');
 
        -- Bucle sobre el cursor corresponent
        CASE upper(i_tipus)
        WHEN 'G' THEN
            FOR despesa IN C_despesesG(i_any)
            LOOP
                DBMS_OUTPUT.PUT_LINE(
                    LPAD(despesa."codi",4) || ' ' ||
                    RPAD(despesa."cambraParlamentaria",50) ||
                    RPAD(despesa."situacio",30) ||
                    LPAD(TO_CHAR(despesa."import",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."importIva",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."total",'9G999G999G990D99'),18));
                    v_sumImport := v_sumImport + despesa."import";
                    v_sumIva    := v_sumIva + despesa."importIva";
                    v_sumTotal  := v_sumTotal + despesa."total";
                    v_compte    := v_compte + 1;
            END LOOP;         
        
        WHEN 'P' THEN
            FOR despesa IN C_despesesP(i_any)
            LOOP
                DBMS_OUTPUT.PUT_LINE(
                    LPAD(despesa."codi",4) || ' ' ||
                    RPAD(despesa."cambraParlamentaria",50) ||
                    RPAD(despesa."situacio",30) ||
                    LPAD(TO_CHAR(despesa."import",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."importIva",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."total",'9G999G999G990D99'),18));
                    v_sumImport := v_sumImport + despesa."import";
                    v_sumIva    := v_sumIva + despesa."importIva";
                    v_sumTotal  := v_sumTotal + despesa."total";
                    v_compte    := v_compte + 1;
            END LOOP;         
        
        ELSE
            FOR despesa IN C_despesesT(i_any)
            LOOP
                DBMS_OUTPUT.PUT_LINE(
                    LPAD(despesa."codi",4) || ' ' ||
                    RPAD(despesa."cambraParlamentaria",50) ||
                    RPAD(despesa."situacio",30) ||
                    LPAD(TO_CHAR(despesa."import",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."importIva",'9G999G999G990D99'),18) || ' ' ||
                    LPAD(TO_CHAR(despesa."total",'9G999G999G990D99'),18));
                    v_sumImport := v_sumImport + despesa."import";
                    v_sumIva    := v_sumIva + despesa."importIva";
                    v_sumTotal  := v_sumTotal + despesa."total";
                    v_compte    := v_compte + 1;
            END LOOP;         
       
        END CASE;

        -- Peu de la relaci
        DBMS_OUTPUT.PUT_LINE('     ' ||
                             RPAD(' ',50,' ') ||
                             RPAD(' ',30,' ') ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '------------------ ');
        DBMS_OUTPUT.PUT_LINE(RPAD(TRIM(TO_CHAR(v_compte)) || ' cambres/situaci.',85) ||
                             TO_CHAR(v_sumImport,'99G999G999G990D99') ||
                             TO_CHAR(v_sumIva,'999G999G999G990D99') || 
                             TO_CHAR(v_sumTotal,'999G999G999G990D99'));
        DBMS_OUTPUT.PUT_LINE('');
                                     
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 

    WHEN exc_noAny THEN
        log_resultat := 'ERROR: NO S''HA INDICAT L''ANY DE LES DESPESES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTipus THEN
        log_resultat := 
           'ERROR: NO S''HA INDICAT EL TIPUS DE DESPESES G=GENERALS, P=PERSONALS O T=TOTES';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESES_06b;


-- *** OBSERVACI SOBRE CURSORS *******************************************************************
-- En les segents consultes s'ha recorregut els cursos d'una altra manera vlida, mitjanant OPEN,
-- LOOP, CLOSE, variables i INTO. 
-- *************************************************************************************************

---------------------------------------------------------------------------------------------------
-- *** 01. CONSULTA DE CAMBRES PARLAMENTRIES.
---------------------------------------------------------------------------------------------------
-- Fa una relaci de cambres parlamentries en funci dels parmetres demanats, on en els noms pot 
-- fer-se una cerca per paraula. Exemple i_cambra = catalunya
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRES (
    i_cambra        IN  VARCHAR2,  -- Nom de la cambra parlamentria o cap per a totes
    i_estat         IN  VARCHAR2,  -- Nom del estat o cap per a tots
    i_pais          IN  VARCHAR2,  -- Nom del pais o regi o cap per a tots
    i_baixa         IN  CHAR,      -- Indicador de situaci de baixa 'S' o 'N'o cap per a totes
    o_rsp          OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables per a la consulta
    v_idEstat      NUMBER(3);      -- Identificador bbdd del estat membre
    v_estat        VARCHAR2(20);   -- Nom del estat membre
    v_idPais       NUMBER(3);      -- Identificador bbdd del pais o regi
    v_pais         VARCHAR2(40);   -- Nom del pais o regi
    v_idCambra     NUMBER(3);      -- Identificador bbdd de la cambra parlamentria
    v_cambra       VARCHAR2(50);   -- Nom de la cabra parlamentria
    v_membres      NUMBER(3);      -- Nmero de membres de la cambra
    v_baixa        VARCHAR2(10);   -- Data de baixa de la cambra parlamentria
    v_seleccio     VARCHAR2(200);  -- Criteris de seleci que acompanyen al ttol
    
    -- Cursor per a cambres amb el resultat de la consulta
    CURSOR C_cambra(p_cambra VARCHAR2, p_estat VARCHAR2, p_pais VARCHAR2, p_baixa CHAR) 
    IS
       SELECT "Id.Estat", "Estat membre", "Id.Pais", "Pais o regi", 
           "Id.Cambra", "Cambra parlamentria", "Membres", "TxBaixa" 
       FROM dce.vis_cambres
       WHERE "Cambra parlamentria" like '%' || p_cambra || '%' AND
           "Estat membre" like '%' || p_estat || '%' AND
           "Pais o regi" like '%' || p_pais || '%' AND
           DECODE("Baixa",'','N',NULL,'N','S') =  
           DECODE(p_baixa,'',decode("Baixa",'','N',NULL,'N','S'), NULL,
           DECODE("Baixa",'','N',NULL,'N','S'),p_baixa);
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'CON.CAMBRES';
    log_entrada  := '<' || i_cambra || '> <' || i_estat || '> <' || 
                    i_pais || '> <' || i_baixa || '>';
    log_resultat := 'OK. CONSULTA DE CAMBRES EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    BEGIN
        -- Criteris de selecci
        IF (i_cambra IS NOT NULL OR i_cambra <> '') THEN 
            v_seleccio := ' / Cambra com: ' || i_cambra; 
        END IF;
        IF (i_estat IS NOT NULL OR i_estat <> '') THEN 
            v_seleccio := v_seleccio || ' / Estat membre com: ' || i_estat; 
        END IF;
        IF (i_pais IS NOT NULL OR i_pais <> '') THEN 
            v_seleccio := v_seleccio || ' / Pais o regi com: ' || i_pais; 
        END IF;
        IF (i_baixa IS NOT NULL OR i_baixa <> '') THEN 
            v_seleccio := v_seleccio || ' / En situaci de baixa: ' || i_baixa; 
        END IF;
        IF (v_seleccio = '' OR v_seleccio IS NULL) THEN 
            v_seleccio := 'Totes les cambres'; 
        END IF;
        log_abans := v_seleccio;
        
        -- Obrim el cursor
        OPEN C_cambra(upper(i_cambra), upper(i_estat), upper(i_pais), 
		                upper(i_baixa));

    	  -- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',140,'-'));         
        DBMS_OUTPUT.PUT_LINE('***  C O  N S U L T A    D E    C A M B R E S' ||
                             '    P A R L A M E N T  R I E S. ***  ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',140,'-'));         
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',140,'-'));         
        DBMS_OUTPUT.PUT_LINE('');
 
        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Id.',4) || 
                             RPAD('Estat membre',21) ||
                             RPAD('Id.',4) ||
                             RPAD('Pais o regi',41) ||
                             RPAD('Id.',4) ||
                             RPAD('Cambra parlamentria',51) ||
                             RPAD('Mem.',5) ||
                             RPAD('Baixa',10));
        DBMS_OUTPUT.PUT_LINE('--- ' || 
                             '-------------------- ' ||
                             '--- ' ||
                             '---------------------------------------- ' ||
                             '--- ' ||
                             '-------------------------------------------------- '|| 
                             '---- ' ||
                             '----------');
 
        -- Bucle sobre el cursor
        LOOP
            FETCH C_cambra INTO v_idEstat, v_estat, v_idPais,v_pais, 
                v_idCambra, v_cambra, v_membres, v_baixa;

            -- Fins arribar al final
            EXIT WHEN C_cambra%NOTFOUND;
		    -- Lnia de la consulta
            DBMS_OUTPUT.PUT_LINE(LPAD(v_idEstat,3)  || ' ' || 
       	                         RPAD(v_estat,21)   || 
      	                         LPAD(v_idPais,3)   || ' ' ||
      	                         RPAD(v_pais,41)    || 
      	                         LPAD(v_idCambra,3) || ' ' ||
      	                         RPAD(v_cambra,51)  || 
      	                         LPAD(v_membres,4)  || ' ' ||
      	                         RPAD(v_baixa,10));
        END LOOP;
        
        -- Peu de la consuta
        DBMS_OUTPUT.PUT_LINE(' ');
        DBMS_OUTPUT.PUT_LINE(C_cambra%ROWCOUNT || ' cambres selecionades.');

        CLOSE C_cambra;
    
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION
    
    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRES;

---------------------------------------------------------------------------------------------------
-- *** 02. CONSULTA DE MEMBRES DE CAMBRES PARLAMENTRIES.
---------------------------------------------------------------------------------------------------
-- Fa una relaci de membres de cambres parlamentries en funci dels parmetres demanats, on en 
-- els noms pot fer-se n a cerca per paraula. Exemple i_cognoms = junqueras
---------------------------------------------------------------------------------------------------
PROCEDURE MEMBRES (
    i_cognoms       IN  VARCHAR2,  -- Cognoms del membre de la cambra o cap per a tots
    i_nom           IN  VARCHAR2,  -- Nom del membre de la cambra o cap per a tots
    i_cambra        IN  VARCHAR2,  -- Nom de la cambra parlamentria o cap per a totes
    i_partit        IN  VARCHAR2,  -- Nom del partit poltic o cap per a tots
    i_baixa         IN  CHAR,      -- Indicador de situaci de baixa 'S' o 'N'o cap per a totes
    o_rsp          OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS

    -- Variables per a la consulta
    v_idMembre     NUMBER(3);      -- Identificador bbdd del membre de la cambra
    v_cognoms      VARCHAR2(50);   -- Cognoms del membre de la cambra
    v_nom          VARCHAR2(20);   -- Nom del membre de la cambra
    v_idCambra     NUMBER(3);      -- Identificador bbdd de la cambra parlamentria
    v_cambra       VARCHAR2(50);   -- Nom de la cabra parlamentria
    v_numero       NUMBER(3);      -- Nmero del membre dins la cambra
    v_idPartit     NUMBER(3);      -- Identificador bbdd del partit poltic
    v_partit       VARCHAR2(70);   -- Nom del partit poltic
    v_baixa        VARCHAR2(10);   -- Data de baixa del membre de la cambra parlamentria
    v_seleccio     VARCHAR2(200);  -- Criteris de seleci que acompanyen al ttol
    
    -- Cursor per a membres amb el resultat de la consulta
    CURSOR C_membre(p_cognoms VARCHAR2, p_nom VARCHAR2, p_cambra VARCHAR2,
	            p_partit VARCHAR2, p_baixa CHAR) 
    IS
       SELECT "Id.Membre", "Cognoms", "Nom", "Id.Cambra", 
           "Cambra parlamentria", "Nm.", "Id.Partit", "Partit poltic", "TxBaixa" 
       FROM dce.vis_membres 
       WHERE "Cognoms" LIKE '%' || p_cognoms || '%' AND
           "Nom" LIKE '%' || p_nom || '%' AND
           "Cambra parlamentria" LIKE '%' || p_cambra || '%' AND
           "Partit poltic" LIKE '%' || p_partit || '%' AND
           DECODE("Baixa",'','N',NULL,'N','S') =  
           DECODE(p_baixa,'',DECODE("Baixa",'','N',NULL,'N','S'), NULL,
           DECODE("Baixa",'','N',NULL,'N','S'),p_baixa);
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'CON.MEMBRES';
    log_entrada  := '<' || i_cognoms || '> <' || i_nom || '> <' || 
                    i_cambra || '> <' || i_partit || '> <' || i_baixa || '>';
    log_resultat := 'OK. CONSULTA DE MEMBRES DE CAMBRES EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    BEGIN
        -- Criteris de selecci
        IF (i_cognoms IS NOT NULL OR i_cognoms <> '') THEN 
            v_seleccio := v_seleccio || ' / Cognoms com: ' || i_cognoms; 
        END IF;
        IF (i_nom IS NOT NULL OR i_nom <> '') THEN 
            v_seleccio := v_seleccio || ' / Nom com: ' || i_nom; 
        END IF;
        IF (i_cambra IS NOT NULL OR i_cambra <> '') THEN 
            v_seleccio := v_seleccio || ' / Cambra com: ' || i_cambra; 
        END IF;
        IF (i_partit IS NOT NULL OR i_partit <> '') THEN 
            v_seleccio := v_seleccio || ' / Partit com: ' || i_partit; 
        END IF;
        IF (i_baixa IS NOT NULL OR i_baixa <> '') THEN 
            v_seleccio := v_seleccio || ' / En situaci de baixa: ' || i_baixa; 
        END IF;
        IF (v_seleccio = '' OR v_seleccio IS NULL) THEN 
            v_seleccio := 'Tots els membres'; 
        END IF;
        log_abans := v_seleccio;
        
	    -- Obrim el cursor
		OPEN C_membre(upper(i_cognoms), upper(i_nom), upper(i_cambra), 
                  upper(i_partit), upper(i_baixa));

    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',174,'-'));         
        DBMS_OUTPUT.PUT_LINE('***  C O  N S U L T A    D E    M E M B R E S   ' || 
                             'D E    C A M B R E S    P A R L A M E N T  R I E S. ***  '
                             || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',174,'-'));         
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',174,'-'));         
        DBMS_OUTPUT.PUT_LINE('');

        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Id.',4) || 
                             RPAD('Cognoms',41) ||
                             RPAD('Nom',14) ||
                             RPAD('Nm.',5) ||
                             RPAD('Id.',4) ||
                             RPAD('Cambra parlamentria',41) ||
                             RPAD('Id.',4) ||
                             RPAD('Partit poltic',51) ||
                             RPAD('Baixa',10));
        DBMS_OUTPUT.PUT_LINE('--- ' || 
                             '---------------------------------------- ' ||
                             '------------- ' ||
                             '---- ' ||
                             '--- ' || 
                             '---------------------------------------- ' ||
                             '--- ' ||
                             '-------------------------------------------------- '|| 
                             '----------');
 
        -- Bucle sobre el cursor
        LOOP
            FETCH C_membre INTO v_idMembre, v_cognoms, v_nom, v_idCambra, v_cambra, 
                  v_numero, v_idPartit, v_partit, v_baixa;

            -- Fins arribar al final
            EXIT WHEN C_membre%NOTFOUND;
		    -- Lnia de la consulta
            DBMS_OUTPUT.PUT_LINE(LPAD(v_idMembre,3) || ' ' || 
       	                         RPAD(SUBSTR(v_cognoms,1,40),41) ||
       	                         RPAD(SUBSTR(v_nom,1,13),14)     ||
       	                         LPAD(v_numero,4)   || ' ' ||       	                         
      	                         LPAD(v_idCambra,3) || ' ' ||
       	                         RPAD(SUBSTR(v_cambra,1,40),41)  ||
       	                         LPAD(v_idPartit,3) || ' ' ||
      	                         RPAD(SUBSTR(v_partit,1,50),51)  ||
      	                         RPAD(v_baixa,10));
        END LOOP;
        
        -- Peu de la consuta
        DBMS_OUTPUT.PUT_LINE(' ');
        DBMS_OUTPUT.PUT_LINE(C_membre%ROWCOUNT || ' membres selecionats.');

		CLOSE C_membre;
    
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 
     
    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END MEMBRES;

---------------------------------------------------------------------------------------------------
-- *** 03. CONSULTA DE DESPESES DE CAMBRES PARLAMENTRIES.
---------------------------------------------------------------------------------------------------
-- Fa una relaci de despeses de cambres parlamentries en funci dels parmetres demanats, on en 
-- els noms pot fer-se una cerca per paraula. Exemple i_cambra = catalunya
---------------------------------------------------------------------------------------------------
PROCEDURE DESPESES_CAMBRES (
    i_dataIni       IN  VARCHAR2,  -- Data inicial de despesa amb format 'dd/mm/yyyy'
    i_dataFin       IN  VARCHAR2,  -- Data final de despesa amb format 'dd/mm/yyyy'
    i_importIni     IN  NUMBER,    -- Import brut inicial de la despesa o 0 per a tots
    i_importFin     IN  NUMBER,    -- Import brut final de la despesa o 0 per a tots
    i_cambra        IN  VARCHAR2,  -- Nom de la cambra parlamentria o cap per a totes
    i_categoria     IN  VARCHAR2,  -- Nom de la categoria general de la despesa
    i_pagament      IN  VARCHAR2,  -- Nom de la forma de pagament de la despesa
    i_situacio      IN  VARCHAR2,  -- Nom de la situaci de la despesa
    i_descripcio    IN  VARCHAR2,  -- Descripci de la despesa
    i_justificacio  IN  VARCHAR2,  -- Descripci de la despesa
    o_rsp          OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables per a la consulta
    v_idDespesa    NUMBER(9);      -- Identificador bbdd de la despesa
    v_data         VARCHAR2(10);   -- Data de la espesa amb format dd/mm/yyyy
    v_idCambra     NUMBER(3);      -- Identificador bbdd de la cambra parlamentria
    v_cambra       VARCHAR2(50);   -- Nom de la cambra parlamentria
    v_idCategoria  NUMBER(3);      -- Identificador bbdd de la cateoria general
    v_categoria    VARCHAR2(50);   -- Nom de la categoria general
    v_import       NUMBER(12,2);   -- Import de la despesa
    v_tpcIva       NUMBER(5,2);    -- Tipus d'IVA
    v_importIva    NUMBER(12,2);   -- Import del IVA
    v_total        NUMBER(12,2);   -- Import total net
    v_idPagament   NUMBER(3);      -- Identificador bbdd de la forma de pagament
    v_pagament     VARCHAR2(40);   -- Nom de la forma de pagament
    v_idSituacio   NUMBER(3);      -- Identificador bbdd de la situaci de la despesa
    v_situacio     VARCHAR2(40);   -- Nom de la situaci de la despesa
    v_descripcio   VARCHAR2(50);   -- Descripci de la despesa
    v_justificacio VARCHAR2(50);   -- Justificaci de la despesa
    v_seleccio     VARCHAR2(200);  -- Criteris de seleci que acompanyen al ttol
    v_impIni       NUMBER(12,2);   -- Import inicial pel cursor
    v_impFin       NUMBER(12,2);   -- Import final pel cursor    
    v_sumImport    NUMBER(14,2);   -- Suma total del import
    v_sumIva       NUMBER(14,2);   -- Suma total del IVA
    v_sumTotal     NUMBER(14,2);   -- Suma total del total
    
    -- Cursor per a despeses amb el resultat de la consulta
   CURSOR C_despesa_cambra(p_dataIni VARCHAR2, p_dataFin VARCHAR2, 
       p_importIni NUMBER, p_importFin NUMBER, p_cambra VARCHAR2, p_categoria VARCHAR2, 
       p_pagament VARCHAR2, p_situacio VARCHAR2, p_descripcio VARCHAR2, p_justificacio VARCHAR2)
   IS
      SELECT "Id.Despesa", "TxData", "Id.Cambra", "Cambra parlamentria", "Id.Categoria",
          "Categoria", "Import brut", "Tipus IVA", "Import IVA", "Import net", "Id.Pagament", 
          "Forma pagament", "Id.Situaci","Situaci", "Descripci", "Justificaci"
      FROM dce.vis_despeses_cambres 
      WHERE "Data despesa" >= 
                  DECODE(p_dataIni,NULL,"Data despesa",'',"Data despesa",TO_DATE(p_dataIni)) AND 
          "Data despesa" <= 
                  DECODE(p_dataFin,NULL,"Data despesa",'',"Data despesa",TO_DATE(p_dataFin)) AND
          "Import brut" >= p_importIni AND "Import brut" <= p_importFin AND
          "Cambra parlamentria" LIKE '%' || p_cambra || '%' AND
          "Categoria" LIKE '%' || p_categoria || '%' AND
          "Forma pagament" LIKE '%' || p_pagament || '%' AND
          "Situaci" LIKE '%' || p_situacio || '%' AND
          "Descripci" LIKE '%' || p_descripcio || '%' AND
          "Justificaci" LIKE '%' || p_justificacio;
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces   := 'CON.DESPESES_CAMBRES';
    log_entrada  := '<' || i_dataIni || '> <' || i_dataFin || '> <' || i_cambra || '> <' || 
        i_categoria || '> <' || i_pagament || '> <' || i_situacio || '> <' || i_descripcio || 
        '> <' || i_justificacio || '>';
    log_resultat := 'OK. CONSULTA DE DESPESES DE CAMBRES EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    v_impIni     := 0;
    v_impFin     := 9999999999;
    v_sumImport  := 0;
    v_sumIva     := 0;
    v_sumTotal   := 0;
        
    BEGIN
        -- Criteris de selecci
        IF (i_dataIni IS NOT NULL OR i_dataIni <> '') THEN 
            v_seleccio := v_seleccio || ' / Data inici >= ' || i_dataIni; 
        END IF;
        IF (i_dataFin IS NOT NULL OR i_dataIni <> '') THEN 
            v_seleccio := v_seleccio || ' / Data final <= ' || i_dataFin; 
        END IF;
        IF (i_importIni IS NOT NULL OR i_importIni <> '') THEN
            v_impIni := i_importIni;
            v_seleccio := v_seleccio || ' / Import Inicial >= ' || i_importIni;
        END IF; 
        IF (i_importFin IS NOT NULL OR i_importFin <> '') THEN
            v_impFin := i_importFin;
            v_seleccio := v_seleccio || ' / Import Final <= ' || i_importFin;
        END IF;         
        IF (i_cambra IS NOT NULL OR i_cambra <> '') THEN 
            v_seleccio := v_seleccio || ' / Cambra com: ' || i_cambra; 
        END IF;
        IF (i_categoria IS NOT NULL OR i_categoria <> '') THEN 
            v_seleccio := v_seleccio || ' / Categoria com: ' || i_categoria; 
        END IF;
        IF (i_pagament IS NOT NULL OR i_pagament <> '') THEN 
            v_seleccio := v_seleccio || ' / Forma pagament com: ' || i_pagament; 
        END IF;

        IF (i_situacio IS NOT NULL OR i_situacio <> '') THEN 
            v_seleccio := v_seleccio || ' / Situaci com: ' || i_situacio; 
        END IF;
        IF (i_descripcio IS NOT NULL OR i_descripcio <> '') THEN 
            v_seleccio := v_seleccio || ' / Descripci com: ' || i_descripcio; 
        END IF;
        IF (i_justificacio IS NOT NULL OR i_justificacio <> '') THEN 
            v_seleccio := v_seleccio || ' / Justificaci com: ' || i_justificacio; 
        END IF;
        IF (v_seleccio = '' OR v_seleccio IS NULL) THEN 
            v_seleccio := 'Totes les despeses'; 
        END IF;
        log_abans := v_seleccio;
        
        -- Obrim el cursor
        OPEN C_despesa_cambra(i_dataIni, i_dataFin, v_impIni, v_impFin, upper(i_cambra), 
            upper(i_categoria), upper(i_pagament), upper(i_situacio), upper(i_descripcio), 
            upper(i_justificacio));

    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',198,'-'));         
        DBMS_OUTPUT.PUT_LINE('***  C O  N S U L T A    D E    D E S P E S E S    D E   ' || 
                             'C A M B R E S    P A R L A M E N T  R I E S. ***  ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',198,'-'));         
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',198,'-'));         
        DBMS_OUTPUT.PUT_LINE('');

        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Id.',10) || 
                             RPAD('Data',11) ||
                             RPAD('Id.',4) ||
                             RPAD('Cambra parlamentria',41) ||
                             RPAD('Id.',4) ||
                             RPAD('Categoria',31) ||
                             RPAD('Import',19) ||
                             RPAD('IVA',6) ||
                             RPAD('Import IVA',19) ||
                             RPAD('Import net',19) ||
                             RPAD('Id.',3) ||
                             RPAD('Situacio',31));                           
        DBMS_OUTPUT.PUT_LINE('--------- ' || 
                             '---------- ' ||
                             '--- ' ||
                             '----------------------------------------- ' ||
                             '--- ' || 
                             '------------------------------ ' ||
                             '------------------ ' ||
                             '----- ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '-- ' ||
                             '------------------------------ ');
        -- Bucle sobre el cursor
        LOOP
            FETCH C_despesa_cambra INTO v_idDespesa, v_data, v_idCambra, v_cambra, v_idCategoria,
                  v_categoria, v_import, v_tpcIva, v_importIva, v_total, v_idPagament, v_pagament, 
                  v_idSituacio, v_situacio, v_descripcio, v_justificacio;

            -- Fins arribar al final
            EXIT WHEN C_despesa_cambra%NOTFOUND;
            
            -- Lnia de la consulta
            DBMS_OUTPUT.PUT_LINE(LPAD(v_idDespesa,9) || ' ' || 
                RPAD(v_data,11) ||
                LPAD(v_idCambra,3) || ' ' ||
                RPAD(v_cambra,41)  ||
                LPAD(v_idCategoria,4) || ' ' ||
                RPAD(SUBSTR(v_categoria,1,30),31)  ||
                LPAD(TO_CHAR(v_import,   '9G999G999G990D99'),18) || ' ' ||
                LPAD(v_tpcIva,5)   || ' ' ||
                LPAD(TO_CHAR(v_importIva,'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(v_total,    '9G999G999G990D99'),18) || ' ' ||    	                         
                LPAD(v_idSituacio,3) || ' ' ||
                RPAD(v_situacio,31));
            v_sumImport := v_sumImport + v_import;
            v_sumIva    := v_sumIva + v_importIva;
            v_sumTotal  := v_sumTotal + v_total;                
        END LOOP;
        
        -- Peu de la consuta
        DBMS_OUTPUT.PUT_LINE(RPAD(' ',102,' ') ||
           '------------------ ' ||
           '      ------------------ ' ||
           '------------------ ');
        DBMS_OUTPUT.PUT_LINE(RPAD(C_despesa_cambra%ROWCOUNT || 
           ' despeses selecionades.',102) ||
            TO_CHAR(v_sumImport,'99G999G999G990D99') ||
            '      ' ||
            TO_CHAR(v_sumIva,'999G999G999G990D99') || 
            TO_CHAR(v_sumTotal,'999G999G999G990D99'));
        DBMS_OUTPUT.PUT_LINE(''); 

		CLOSE C_despesa_cambra;
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 
    
    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESES_CAMBRES;

---------------------------------------------------------------------------------------------------
-- *** 04. CONSULTA DE DESPESES DE MEMBRES DE CAMBRES PARLAMENTRIES.
---------------------------------------------------------------------------------------------------
-- Fa una relaci de despeses de membres de cambres parlamentries en funci dels parmetres 
-- demanats, on en els noms pot fer-se una cerca per paraula. Exemple i_cambra = catalunya
---------------------------------------------------------------------------------------------------
PROCEDURE DESPESES_MEMBRES (
    i_dataIni       IN  VARCHAR2,  -- Data inicial de despesa amb format 'dd/mm/yyyy'
    i_dataFin       IN  VARCHAR2,  -- Data final de despesa amb format 'dd/mm/yyyy'
    i_importIni     IN  NUMBER,    -- Import brut inicial de la despesa o 0 per a tots
    i_importFin     IN  NUMBER,    -- Import brut final de la despesa o 0 per a tots
    i_cognoms       IN  VARCHAR2,  -- Cognoms del membre de cambra parlamentria o cap per a tots
    i_nom           IN  VARCHAR2,  -- Nom del membre de cambra parlamentria o cap per a tots
    i_partit        IN  VARCHAR2,  -- Nom del partit poltic o cap per a tots
    i_categoria     IN  VARCHAR2,  -- Nom de la categoria general de la despesa
    i_pagament      IN  VARCHAR2,  -- Nom de la forma de pagament de la despesa
    i_situacio      IN  VARCHAR2,  -- Nom de la situaci de la despesa
    i_descripcio    IN  VARCHAR2,  -- Descripci de la despesa
    i_justificacio  IN  VARCHAR2,  -- Descripci de la despesa
    o_rsp          OUT  VARCHAR2   -- Resultat de l'execuci del procediment de la consulta
) IS
    -- Variables per a la consulta
    v_idDespesa    NUMBER(9);      -- Identificador bbdd de la despesa
    v_data         VARCHAR2(10);   -- Data de la espesa amb format dd/mm/yyyy
    v_idMembre     NUMBER(3);      -- Identificador bbdd del membre
    v_cognoms      VARCHAR2(50);   -- Cognoms del membre
    v_nom          VARCHAR2(20);   -- Nom del membre
    v_idPartit     NUMBER(3);      -- Identificador bbdd del partit poltic
    v_partit       VARCHAR2(70);   -- Nom del partit poltic
    v_idCategoria  NUMBER(3);      -- Identificador bbdd de la cateoria general
    v_categoria    VARCHAR2(50);   -- Nom de la categoria general
    v_import       NUMBER(12,2);   -- Import de la despesa
    v_tpcIva       NUMBER(5,2);    -- Tipus d'IVA
    v_importIva    NUMBER(12,2);   -- Import del IVA
    v_total        NUMBER(12,2);   -- Import total net
    v_idPagament   NUMBER(3);      -- Identificador bbdd de la forma de pagament
    v_pagament     VARCHAR2(40);   -- Nom de la forma de pagament
    v_idSituacio   NUMBER(3);      -- Identificador bbdd de la situaci de la despesa
    v_situacio     VARCHAR2(40);   -- Nom de la situaci de la despesa
    v_descripcio   VARCHAR2(50);   -- Descripci de la despesa
    v_justificacio VARCHAR2(50);   -- Justificaci de la despesa
    v_seleccio     VARCHAR2(200);  -- Criteris de seleci que acompanyen al ttol
    v_impIni       NUMBER(12,2);   -- Import inicial pel cursor
    v_impFin       NUMBER(12,2);   -- Import final pel cursor    
    v_sumImport    NUMBER(14,2);   -- Suma total del import
    v_sumIva       NUMBER(14,2);   -- Suma total del IVA
    v_sumTotal     NUMBER(14,2);   -- Suma total del total

    
    -- Cursor per a despeses amb el resultat de la consulta
   CURSOR C_despesa_membre(p_dataIni VARCHAR2, p_dataFin VARCHAR2, p_importIni NUMBER, 
       p_importFin NUMBER, p_cognoms VARCHAR2, p_nom VARCHAR2, p_partit VARCHAR2, 
       p_categoria VARCHAR2, p_pagament VARCHAR2, p_situacio VARCHAR2, 
       p_descripcio VARCHAR2, p_justificacio VARCHAR2)
   IS
      SELECT "Id.Despesa", "TxData", "Id.Membre", "Cognoms", "Nom", "Id.Partit", 
          "Partit poltic", "Id.Categoria", "Categoria", "Import brut", "Tipus IVA", 
          "Import IVA", "Import net", "Id.Pagament", "Forma pagament", "Id.Situaci",
          "Situaci", "Descripci", "Justificaci"
      FROM dce.vis_despeses_membres 
      WHERE "Data despesa" >= decode(p_dataIni,NULL,"Data despesa",'',
            "Data despesa",to_date(p_dataIni)) AND 
            "Data despesa" <= decode(p_dataFin,NULL,"Data despesa",'',
            "Data despesa",to_date(p_dataFin)) AND
            "Import brut" >= p_importIni AND "Import brut" <= p_importFin AND
            "Cognoms" LIKE '%' || p_cognoms || '%' AND
            "Nom" LIKE '%' || p_nom || '%' AND
            "Partit poltic" LIKE '%' || p_partit || '%' AND                    
            "Categoria" LIKE '%' || p_categoria || '%' AND
            "Forma pagament" LIKE '%' || p_pagament || '%' AND
            "Situaci" LIKE '%' || p_situacio || '%' AND
            "Descripci" LIKE '%' || p_descripcio || '%' AND
            "Justificaci" LIKE '%' || p_justificacio;
BEGIN
	
    -- Inicialitzaci de les variables del tractament d'excepcions i de la consulta
    log_proces   := 'CON.DESPESES_MEMBRES';
    log_entrada  := '<' || i_dataIni || '> <' || i_dataFin || '> <' || i_cognoms || '> <' || 
        i_nom || '> <' || i_partit || '> <' || i_categoria || '> <' || i_pagament || '> <' ||
        i_situacio || '> <' || i_descripcio || 
        '> <' || i_justificacio || '>';
    log_resultat := 'OK. CONSULTA DE DESPESES DE MEMBRES EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    v_impIni     := 0;
    v_impFin     := 9999999999;
    v_sumImport  := 0;
    v_sumIva     := 0;
    v_sumTotal   := 0;
    
    BEGIN
        -- Criteris de selecci
        IF (i_dataIni IS NOT NULL OR i_dataIni <> '') THEN 
            v_seleccio := v_seleccio || ' / Data inici >= ' || i_dataIni; 
        END IF;
        IF (i_dataFin IS NOT NULL OR i_dataIni <> '') THEN 
            v_seleccio := v_seleccio || ' / Data final <= ' || i_dataFin; 
        END IF;
        IF (i_importIni IS NOT NULL OR i_importIni <> '') THEN
            v_impIni := i_importIni;
            v_seleccio := v_seleccio || ' / Import Inicial >= ' || i_importIni;
        END IF; 
        IF (i_importFin IS NOT NULL OR i_importFin <> '') THEN
            v_impFin := i_importFin;
            v_seleccio := v_seleccio || ' / Import Final <= ' || i_importFin;
        END IF;         
        IF (i_cognoms IS NOT NULL OR i_cognoms <> '') THEN 
            v_seleccio := v_seleccio || ' / Cognoms com: ' || i_cognoms; 
        END IF;
        IF (i_nom IS NOT NULL OR i_nom <> '') THEN 
            v_seleccio := v_seleccio || ' / Nom com: ' || i_nom; 
        END IF;
        IF (i_partit IS NOT NULL OR i_partit <> '') THEN 
            v_seleccio := v_seleccio || ' / Partit com: ' || i_partit; 
        END IF;   
        IF (i_categoria IS NOT NULL OR i_categoria <> '') THEN 
            v_seleccio := v_seleccio || ' / Categoria com: ' || i_categoria; 
        END IF;
        IF (i_pagament IS NOT NULL OR i_pagament <> '') THEN 
            v_seleccio := v_seleccio || ' / Forma pagament com: ' || i_pagament; 
        END IF;

        IF (i_situacio IS NOT NULL OR i_situacio <> '') THEN 
            v_seleccio := v_seleccio || ' / Situaci com: ' || i_situacio; 
        END IF;
        IF (i_descripcio IS NOT NULL OR i_descripcio <> '') THEN 
            v_seleccio := v_seleccio || ' / Descripci com: ' || i_descripcio; 
        END IF;
        IF (i_justificacio IS NOT NULL OR i_justificacio <> '') THEN 
            v_seleccio := v_seleccio || ' / Justificaci com: ' || i_justificacio; 
        END IF;
        IF (v_seleccio = '' OR v_seleccio IS NULL) THEN 
            v_seleccio := 'Totes les despeses'; 
        END IF;
        log_abans := v_seleccio;
        
        -- Obrim el cursor
        OPEN C_despesa_membre(i_dataIni, i_dataFin, v_impIni, v_impFin, upper(i_cognoms), 
            upper(i_nom), upper(i_partit), upper(i_categoria), upper(i_pagament), 
            upper(i_situacio), upper(i_descripcio), upper(i_justificacio));

    	-- Ttol de la consulta
        DBMS_OUTPUT.PUT_LINE('');
        DBMS_OUTPUT.PUT_LINE(RPAD('-',213,'-'));         
        DBMS_OUTPUT.PUT_LINE('***  C O N S U L T A    D E    D E S P E S E S    D E   ' || 
             'M E M B R E S    D E    C A M B R A    P A R L A M E N T  R I A. ***  ' || v_quiquan);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',213,'-'));         
        DBMS_OUTPUT.PUT_LINE(v_seleccio);
        DBMS_OUTPUT.PUT_LINE(RPAD('-',213,'-'));         
        DBMS_OUTPUT.PUT_LINE('');

        -- Capalera de la consulta
        DBMS_OUTPUT.PUT_LINE(RPAD('Id.',10) || 
                             RPAD('Data',11) ||
                             RPAD('Id.',4) ||
                             RPAD('Membre de la cambra',31) ||
                             RPAD('Id.',4) ||
                             RPAD('Partit poltic',41) ||
                             RPAD('Id.',4) ||
                             RPAD('Categoria',21) ||
                             RPAD('Import',19) ||
                             RPAD('IVA',6) ||
                             RPAD('Import IVA',19) ||
                             RPAD('Import net',19) ||
                             RPAD('Id.',4) ||
                             RPAD('Situacio',21));
        DBMS_OUTPUT.PUT_LINE('--------- ' || 
                             '---------- ' ||
                             '--- ' ||
                             '------------------------------ ' ||
                             '--- ' ||
                             '---------------------------------------- ' ||
                             '--- ' || 
                             '-------------------- ' ||
                             '------------------ ' ||
                             '----- ' ||
                             '------------------ ' ||
                             '------------------ ' ||
                             '--- ' ||
                             '-------------------- ');
 
        -- Bucle sobre el cursor
        LOOP
            FETCH C_despesa_membre INTO v_idDespesa, v_data, v_idMembre, v_cognoms, v_nom, 
                v_idPartit, v_partit, v_idCategoria, v_categoria, v_import, v_tpcIva, 
                v_importIva, v_total, v_idPagament, v_pagament, v_idSituacio, v_situacio,
                v_descripcio, v_justificacio;

            -- Fins arribar al final
            EXIT WHEN C_despesa_membre%NOTFOUND;
            
            -- Lnia de la consulta
            DBMS_OUTPUT.PUT_LINE(LPAD(v_idDespesa,9) || ' ' || 
                RPAD(v_data,11) ||
                LPAD(v_idMembre,3) || ' ' ||
                RPAD(SUBSTR(v_cognoms || ' ' || v_nom,1,30),31)  ||
                LPAD(v_idPartit,3) || ' ' ||
                RPAD(SUBSTR(v_partit,1,40),41) ||
                LPAD(v_idCategoria,3) || ' ' ||
                RPAD(SUBSTR(v_categoria,1,20),21)  ||
                LPAD(TO_CHAR(v_import,   '9G999G999G990D99'),18) || ' ' ||
                LPAD(v_tpcIva,5)   || ' ' ||
                LPAD(TO_CHAR(v_importIva,'9G999G999G990D99'),18) || ' ' ||
                LPAD(TO_CHAR(v_total,    '9G999G999G990D99'),18) || ' ' ||
                LPAD(v_idSituacio,3) || ' ' ||
                RPAD(v_situacio,21));
            v_sumImport := v_sumImport + v_import;
            v_sumIva    := v_sumIva + v_importIva;
            v_sumTotal  := v_sumTotal + v_total;
        END LOOP;
        
        -- Peu de la consuta
        DBMS_OUTPUT.PUT_LINE(RPAD(' ',126,' ') ||
           '------------------ ' ||
           '      ------------------ ' ||
           '------------------ ');
        DBMS_OUTPUT.PUT_LINE(RPAD(C_despesa_membre%ROWCOUNT || 
           ' despeses selecionades.',126) ||
            TO_CHAR(v_sumImport,'99G999G999G990D99') ||
            '      ' ||
            TO_CHAR(v_sumIva,'999G999G999G990D99') || 
            TO_CHAR(v_sumTotal,'999G999G999G990D99'));
        DBMS_OUTPUT.PUT_LINE(''); 

        CLOSE C_despesa_membre;
    
    END;

    -- Execuci correcte del procediment
    log_error   := 'N';
    o_rsp       := log_resultat;
    
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    -- Excepcions
    EXCEPTION 
    
    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESES_MEMBRES;

---------------------------------------------------------------------------------------------------
END;
/ 
