---------------------------------------------------------------------------------------------------
-- UOC - ETIG - TFC Bases de dades relacionals	2013-2014 1r semestre
---------------------------------------------------------------------------------------------------
-- Base de Dades d'un Sistema Centralitzat de Control de Despesa Pblica dels Parlaments Europeus
---------------------------------------------------------------------------------------------------
-- Arxiu : 13_Del_Taules.sql        -- Autor    : Santi Alcayde Pagan   -- Aula: 1
-- Data  : 11/11/2013               -- Consultor: Manel Rella Ruiz      
---------------------------------------------------------------------------------------------------
-- Funci: Donar de baixa/alta cambres i membres de cambres parlamentries i eliminar despeses 
-- Executa els procediments CAMBRA_B, MEMBRE_B, DESPESA_CAMBRA_B i DESPESA_MEMBRE_B del package ABM
-- El resultat de l'exeuci dels procediments es guarda en la taula d'auditoria LOG_PROCEDIMENT
---------------------------------------------------------------------------------------------------

SET SERVEROUTPUT ON;

DECLARE
    rsp    VARCHAR2(256); -- Resultat de l'execuci del procediment
    proces VARCHAR2(80);  -- Nom del procs
BEGIN

---------------------------------------------------------------------------------------------------
-- *** BAIXA/ALTA DE CAMBRES PARLAMENTRIES 
---------------------------------------------------------------------------------------------------
-- Fa una baixa o alta lgica d'una cambra, informant una data en la taula CAMBRA
-- mitjanant el procediment CAMBRA_B del package ABM
/*-------------------------------------------------------------------------------------------------
 Parmetres a passar al procediment:
    i_ibd     IN  NUMBER (3)        -- Codi identificador bbdd de la cambra parlamentria
    i_data    IN  VARCHAR2 (10)     -- Data de la baixa, cadena en format 'DD/MM/YYYY' 
                                       '' o null per tornar a donar d'alta.
    o_rsp     OUT VARCHAR2          -- Resultat de l'execuci del procediment 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que existeixi la cambra parlmentria
   - Regla de negoci 5: Baixa/alta lgica d'una cambra parlamentria en la taula CAMBRA mitjanant
     una data de baixa que deixa la cambra inactiva/activa. Si es passa una data s'actualitza la 
     data de baixa i la cambra queda inactiva. Si no es passa cap data, es treu la data de baixa i
     la cambra torna a estar activa.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
*/
-- ELIMINACIONS CORRECTES

proces := 'BAIXA DE CAMBRA PARLAMENTRIA: ';
dce.abm.cambra_b(1, '31/10/2013', rsp);
dce.abm.cambra_b(3, '31/10/2013', rsp);
dce.abm.cambra_b(5, '31/10/2013', rsp);
dce.abm.cambra_b(7, '31/10/2013', rsp);

proces := 'ALTA DE CAMBRA PARLAMENTRIA: ';
dce.abm.cambra_b(2, '', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.cambra_b(4, '', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.cambra_b(6, '', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.cambra_b(8, '', rsp);
dbms_output.put_line(proces || rsp);

-- ELIMINACIONS INCORRECTES

-- No s'indiquen parmetres obligatoris
dce.abm.cambra_b('', '31/10/2013', rsp);
dbms_output.put_line(proces || rsp);

-- S'indica una cambra parlamentria inexistent
dce.abm.cambra_b(98, '31/10/2013', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.cambra_b(99, '', rsp);
dbms_output.put_line(proces || rsp);

---------------------------------------------------------------------------------------------------
-- *** BAIXA/ALTA DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
-- Fa una baixa o alta lgica d'un membre d'una cambra, informant una data, en la taula 
-- MEMBRE_CAMBRA mitjanant el procediment MEMBRE_B del package ABM
/*-------------------------------------------------------------------------------------------------
Parmetres a passar al procediment:
   i_ibd  IN  NUMBER (5),          -- Codi identificador bbdd del membre de la cambra
   i_data IN  VARCHAR2 (10),       -- Data de la baixa, cadena en format 'DD/MM/YYYY' 
                                      '' o null per tornar a donar d'alta.
   o_rsp  OUT VARCHAR2             -- Resultat de l'execuci del procediment
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que existeixi el membre de la cambra.
   - Regla de negoci 5: Baixa/alta lgica d'un membre d'una cambra parlamentria en la taula 
     MEMBRE_CAMBRA mitjanant una data de baixa que deixa el membre de la cambra inactiu/actiu.
   - Regla de negoci 5 i 7: Si es passa una data s'actualitza la data de baixa, el membre de la 
     cambra queda inactiu, i es decrementa el comptador de nmero de membres de la cambra 
     parlamentria a la que pertany. Si no es passa cap data, si el membre est de baixa, es treu 
     la data de baixa, el membre torna a estar actiu, i s'incrementa el comptador de nmero de 
     membres de la cambra parlamentria a la que pertany.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
*/
-- ELIMINACIONS CORRECTES

proces := 'BAIXA DE MEMBRE DE CAMBRA PARLAMENTRIA: ';
dce.abm.membre_b(1, '31/10/2013', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.membre_b(3, '31/10/2013', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.membre_b(5, '31/10/2013', rsp);
dbms_output.put_line(proces || rsp);

proces := 'ALTA DE MEMBRE DE CAMBRA PARLAMENTRIA: ';
dce.abm.membre_b(2, '', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.membre_b(4, '', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.membre_b(6, '', rsp);
dbms_output.put_line(proces || rsp);

-- ELIMINACIONS INCORRECTES

-- No s'indiquen parmetres obligatoris
dce.abm.membre_b('', '31/10/2013', rsp);
dbms_output.put_line(proces || rsp);

-- S'indica un/a membre de cambra parlamentria inexistent
dce.abm.membre_b(198, '31/10/2013', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.membre_b(199, '', rsp);
dbms_output.put_line(proces || rsp);

---------------------------------------------------------------------------------------------------
-- *** BAIXA DE DESPESES
---------------------------------------------------------------------------------------------------
-- Elimina una despesa en la taula DESPESA_CAMBRA o DESPESA_MEMBRE segons el tipus de despesa
-- mitjanant el procediment DESPESA_B del package ABM
/*-------------------------------------------------------------------------------------------------
Parmetres a passar al procediment:
   i_ibd   IN  NUMBER (9),         -- Codi identificador bbdd de la despesa
   i_tipus IN  CHAR (1),           -- Tipus de despesa: C=Cambra, M=Membre
   o_rsp   OUT VARCHAR2            -- Resultat de l'execuci del procediment
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
---------------------------------------------------------------------------------------------------
*/
-- ELIMINACIONS CORRECTES

proces := 'BAIXA DE DESPESA GENERAL DE CAMBRA PARLAMENTRIA: ';
dce.abm.despesa_b(1, 'C', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_b(22, 'c', rsp);
dbms_output.put_line(proces || rsp);

proces := 'BAIXA DE DESPESA PERSONAL DE MEMBRE DE CAMBRA PARLAMENTRIA: ';
dce.abm.despesa_b(92, 'M', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_b(102, 'm', rsp);
dbms_output.put_line(proces || rsp);

-- ELIMINACIONS INCORRECTES
proces := 'BAIXA DE DESPESA: ';

-- No s'indiquen parmetres obligatoris
dce.abm.despesa_b('', 'C', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_b(1, '', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_b('', 'm', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_b(22, '', rsp);
dbms_output.put_line(proces || rsp);

-- S'indica un tipus incorrecte
dce.abm.despesa_b(1, 'x', rsp);
dbms_output.put_line(proces || rsp);

-- S'indica un codi de despesa inexistent
dce.abm.despesa_b(998, 'C', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_b(999, 'M', rsp);
dbms_output.put_line(proces || rsp);

---------------------------------------------------------------------------------------------------
END;
/