---------------------------------------------------------------------------------------------------
-- UOC - ETIG - TFC Bases de dades relacionals	2013-2014 1r semestre
---------------------------------------------------------------------------------------------------
-- Base de Dades d'un Sistema Centralitzat de Control de Despesa Pblica dels Parlaments Europeus
---------------------------------------------------------------------------------------------------
-- Arxiu : 14_Mod_Taules.sql        -- Autor    : Santi Alcayde Pagan   -- Aula: 1
-- Data  : 11/11/2013               -- Consultor: Manel Rella Ruiz      
---------------------------------------------------------------------------------------------------
-- Funci: Modificar informaci en les taules CAMBRA,MEMBRE_CAMBRA,  DESPESA, DESPESA_CAMBRA i 
--         DESPESA_MEMBRE mitjanant els procediments _M de modificaci del paquet ABM
---------------------------------------------------------------------------------------------------

SET SERVEROUTPUT ON;

DECLARE
    rsp    VARCHAR2(256); -- Resultat de l'execuci del procediment
    proces VARCHAR2(80);  -- Nom del procs
BEGIN

---------------------------------------------------------------------------------------------------
-- *** MODIFICACI DE CAMBRES PARLAMENTRIES 
---------------------------------------------------------------------------------------------------
-- Modifica les dades d'una cambra palamentria en la taula CAMBRA.
-- mitjanant el procediment CAMBRA_M del package ABM
---------------------------------------------------------------------------------------------------
proces := 'MODIFICAR CAMBRA PARLAMENTRIA: ';
/*-------------------------------------------------------------------------------------------------
Parmetres a passar al procediment: -- Indicar '' o null per no modificar
    i_ibd     IN  NUMBER(3)         -- Codi identificador bbdd de la cambra
    i_nom     IN  VARCHAR2 (50)     -- Nom de la cambra
    i_adreca  IN  VARCHAR2 (50)     -- Adrea postal de la cambra
    i_ciutat  IN  VARCHAR2 (30)     -- Ciutat on es troba la cambra
    i_telefon IN  VARCHAR2 (20)     -- Telfon de contacte de la cambra
    i_llocWeb IN  VARCHAR2 (50)     -- URL del lloc web de la cambra
    i_estat   IN  NUMBER (3)        -- Identificador del estat membre (0 si s de pais o regi)
    i_pais    IN  NUMBER (3)        -- Identificador del pais o regi (0 si s d'estat membre)
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd de la cambra i_ibd i al menys un parmetre a modificar. 
   - Si tant <i_estat> com <i_pais>, sn tots dos a la vegada > 0, es tracta com un error, 
     ja que una cambra ho s, o b d'un estat membre, o b d'un pais o regi 
   - Si <i_estat> > 0  <i_pais> > 0, crida als procediments que fan alta, baixa i modificaci
     en les taules CAMBRA_ESTAT i CAMBRA_PAIS_REGIO.
   - Comprova que un estat no tingui ms de dues cambres i que un pas o regi noms en tingui una
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
*/
-- MODIFICACIONS CORRECTES

dce.abm.cambra_m(1,'AUSTRIAN NATIONAL COUNCIL', '', '', 'canvi telfon', '', 0, 0, rsp);
dbms_output.put_line(proces || rsp);
dce.abm.cambra_m(2,'', 'Canvi adrea', 'Canvi ciutat', '', '', 0, 0, rsp);
dbms_output.put_line(proces || rsp);

-- MODIFICACIONS INCORRECTES

-- No s'indiquen parmetres obligatoris
dce.abm.cambra_m('','AUSTRIAN NATIONAL COUNCIL', '', '', 'canvi telfon', '', 0, 0, rsp);
dbms_output.put_line(proces || rsp);

-- No s'ha indicat cap dada a modificar
dce.abm.cambra_m(1,'', '', '', '', '', 0, 0, rsp);
dbms_output.put_line(proces || rsp);

-- S'ha indicat a la vegada estat i pais o regi
dce.abm.cambra_m(1,'AUSTRIAN NATIONAL COUNCIL', '', '', 'canvi telfon', '', 3, 3, rsp);
dbms_output.put_line(proces || rsp);

-- S'ha indicat un canvi d'estat a un altre que ja t dues cambres
dce.abm.cambra_m(1,'', '', '', 'canvi telfon', '', 25, 0, rsp);
dbms_output.put_line(proces || rsp);

-- S'ha indicat un canvi de pais a un altre que ja t una cambra
dce.abm.cambra_m(1,'', '', '', 'canvi telfon', '', 0, 8, rsp);
dbms_output.put_line(proces || rsp);

---------------------------------------------------------------------------------------------------
-- *** MODIFICACI DE MEMBRES DE CAMBRES PARLAMENTRIES 
---------------------------------------------------------------------------------------------------
-- Modifica les dades d'un membre d'una cambra palamentria en la taula MEMBRE_CAMBRA.
-- mitjanant el procediment MEMBRE_M del package ABM
---------------------------------------------------------------------------------------------------
proces := 'MODIFICAR MEMBRE DE CAMBRA PARLAMENTRIA: ';
/*-------------------------------------------------------------------------------------------------
Parmetres a passar al procediment:  -- Indicar '' o null per no modificar
i_ibd	     IN NUMBER (5)             -- Codi identificador bbdd del membre
i_codi	   IN VARCHAR2 (12)          -- Codi identificador (NIF, etc.)
i_nom	     IN VARCHAR2 (20)          -- Nom del membre de la cambra
i_cognoms  IN VARCHAR2 (50)          -- Cognoms del membre de la cambra
i_telefon  IN VARCHAR2 (20)          -- Telfon del membre de la cambra
i_email	   IN VARCHAR2 (50)          -- Adrea elctrnica del membre de la cambra
i_partit   IN NUMBER (3)             -- Identificador bbdd del partit al que pertany
i_cambra   IN NUMBER (3)             -- Identificador bbdd de la cambra a la que pertany
i_numero   IN NUMBER (3)             -- Nmero identificador dins de la cambra
o_rsp     OUT VARCHAR2               -- Resultat de l'execuci del procediment 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd del membre de la cambra i al menys un parmetre a modificar. 
   - Regle de negoci 7: Si es canvia la cambra parlamentria actualitza els comptadors de nmero 
     de membres de l'antiga cambra i de la nova.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
*/
-- MODIFICACIONS CORRECTES

dce.abm.membre_m(1, 'CODI000123','joan','', '', '', '', '', '', rsp);
dbms_output.put_line(proces || rsp);

dce.abm.membre_m(15, 'CODI00015','pere','soley i morey', '(34) 971228292', 
    'jaime.soley@asamblemadrid.es', '2', '12', '1', rsp);
dbms_output.put_line(proces || rsp);

-- MODIFICACIONS INCORRECTES

-- No s'ha indicat cap dada a modificar
dce.abm.membre_m(1, '','','', '', '', '', '', '', rsp);
dbms_output.put_line(proces || rsp);

-- Partit poltic inexistent
dce.abm.membre_m(1, '','','', '', '', '999', '', '', rsp);
dbms_output.put_line(proces || rsp);

-- Cambra parlamentria inexistent
dce.abm.membre_m(1, '','','', '', '', '', '999', '', rsp);
dbms_output.put_line(proces || rsp);

-- Ja existeix un/a membre amb els parmetres i_cambra i i_numero indicats
dce.abm.membre_m(1, '','','', '', '', '', '10', '75', rsp);
dbms_output.put_line(proces || rsp);

---------------------------------------------------------------------------------------------------
-- *** MODIFICACI DE DESPESES GENERALS 
---------------------------------------------------------------------------------------------------
proces := 'MODIFICAR DESPESA DE CAMBRA PARLAMENTRIA: ';
---------------------------------------------------------------------------------------------------
-- Modifica les dades d'una despesa general d'una cambra parlamentria en la taula DESPESA_CAMBRA 
/*-------------------------------------------------------------------------------------------------
Parmetres a passar al procediment:     -- Indicar '' o null per no modificar
    i_ibd          IN  NUMBER (9)       -- Identificador bbdd de la despesa
    i_data         IN  VARCHAR2 (10)    -- Data de la despesa en format 'DD/MM/YYYY' 
    i_cambra       IN  NUMBER (3)       -- Identificador bbdd del membre
    i_categoria    IN  NUMBER (2)       -- Categoria de la despesa
    i_import       IN  NUMBER (12,2)    -- Import brut de la despesa
    i_tipusIva     IN  NUMBER (5,2)     -- Tipus d'IVA de la  despesa
    i_descripcio   IN  VARCHAR2 (50)    -- Descripci de la despesa
    i_justificacio IN  VARCHAR2 (50)    -- Justificaci de la despesa
    i_pagament     IN  NUMBER (3)       -- Forma de pagament de la despesa
    i_situacio     IN  NUMBER (2)       -- Situaci de la despesa
    i_proveidor    IN  VARCHAR2 (12)    -- Codi provedor 
    i_escaneig     IN  VARCHAR2 (50)    -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2         -- Resultat de l'execuci del procediment 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd de la despesa i al menys un parmetre a modificar. 
   - L'existncia de la cambra parlamentria si s'indica
   - L'existncia de la categoria general, si s'indica
   - L'existncia de la forma de pagament, si s'indica
   - L'existncia de la situaci de despesa, si s'indica
   - Si s'indiquen els parmetres corresponents:
      - Regla de negoci 2: El clcul del import del IVA i del import net es fa a nivell de valors 
        automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0   
      - Regla de negoci 8: Que el provedor estigui informat si la despesa > 30 euros.
   - Regla de negoci 9: No es pot entrar despeses en cambres donades de baixa si la data de 
     despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
---------------------------------------------------------------------------------------------------
*/
-- MODIFICACIONS CORRECTES

dce.abm.despesa_cambra_m(2,'04/11/2013','','','','','','','','','','', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_cambra_m(3,'','','',2435.86,'','','','','','','', rsp);
dbms_output.put_line(proces || rsp);

-- MODIFICACIONS INCORRECTES

-- No s'indica cap dada a modificar
dce.abm.despesa_cambra_m(1,'','','','','','','','','','','', rsp);
dbms_output.put_line(proces || rsp);

-- Despesa general inexistent
dce.abm.despesa_cambra_m(94,'','','','','','Nova descripci de despesa','','','','','', rsp);
dbms_output.put_line(proces || rsp);  

-- S'indica una cambra parlamentria inexistent
dce.abm.despesa_cambra_m(4,'12/10/2013',98,1,250,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,2,'proveidor01','',rsp);
dbms_output.put_line(proces || rsp);

-- S'inidica una categoria general inexistent
dce.abm.despesa_cambra_m(4,'12/10/2013',10,31,250,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,1,'proveidor01','',rsp);
dbms_output.put_line(proces || rsp);

-- S'indica una forma de pagament inexistent
dce.abm.despesa_cambra_m(4,'12/10/2013',10,1,250,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',15,1,'proveidor01','',rsp);
dbms_output.put_line(proces || rsp);

-- S'indica una situaci inexistent
dce.abm.despesa_cambra_m(4,'12/10/2013',10,1,250,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,91,'proveidor01','',rsp);
dbms_output.put_line(proces || rsp);

-- S'indica una despesa > 30 euros i no s'indica el provedor
dce.abm.despesa_cambra_m(4,'12/10/2013',10,1,250,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,1,'','',rsp);
dbms_output.put_line(proces || rsp);

-- S'indica un import zero o menor de zero
dce.abm.despesa_cambra_m(4,'12/10/2013',98,1,0,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,'','','',rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_cambra_m(4,'12/10/2013',98,1,-25,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,'','','',rsp);
dbms_output.put_line(proces || rsp);

-- S'indica un tipus d'IVA zero o menor de zero
dce.abm.despesa_cambra_m(4,'12/10/2013',98,1,250,0,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,'','','',rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_cambra_m(4,'12/10/2013',98,1,250,-21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,'','','',rsp);
dbms_output.put_line(proces || rsp);

-- S'indica una data de despesa superior a la data de baixa d'una cambra que ho est
dce.abm.despesa_cambra_m(4,'02/11/2013',7,1,25,21,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',1,2,'','',rsp);
dbms_output.put_line(proces || rsp);    

-- Per a una alta, s'indica una data de despesa superior a la de baixa d'una cambra que ho est
proces := 'NOVA DESPESA GENERAL DE CAMBRA PARLAMENTRIA: ';
dce.abm.despesa_cambra_a('02/11/2013',7,1,25,21,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',1,2,'','',rsp);
dbms_output.put_line(proces || rsp);    

---------------------------------------------------------------------------------------------------
-- *** MODIFICACI DE DESPESES PERSONALS 
---------------------------------------------------------------------------------------------------
proces := 'MODIFICAR DESPESA DE MEMBRE DE CAMBRA PARLAMENTRIA: ';
---------------------------------------------------------------------------------------------------
-- Modifica les dades d'una despesa personal d'un membre de cambra parlamentria en la taula 
-- DESPESA_MEMBRE 
/*-------------------------------------------------------------------------------------------------
Parmetres a passar al procediment:     -- Indicar '' o null per no modificar
    i_ibd          IN  NUMBER (9)       -- Identificador bbdd de la despesa
    i_data         IN  VARCHAR2 (10)    -- Data de la despesa en format 'DD/MM/YYYY' 
    i_membre       IN  NUMBER (5)       -- Identificador bbdd del membre
    i_categoria    IN  NUMBER (2)       -- Categoria de la despesa
    i_import       IN  NUMBER (12,2)    -- Import brut de la despesa
    i_tipusIva     IN  NUMBER (5,2)     -- Tipus d'IVA de la  despesa
    i_descripcio   IN  VARCHAR2 (50)    -- Descripci de la despesa
    i_justificacio IN  VARCHAR2 (50)    -- Justificaci de la despesa
    i_pagament     IN  NUMBER (3)       -- Forma de pagament de la despesa
    i_situacio     IN  NUMBER (2)       -- Situaci de la despesa
    i_proveidor    IN  VARCHAR2 (12)    -- Codi provedor 
    i_escaneig     IN  VARCHAR2 (50)    -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2         -- Resultat de l'execuci del procediment 
-------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd de la despesa i al menys un parmetre a modificar. 
   - L'existncia del membre de cambra parlamentria si s'indica
   - L'obtenci del partit poltic del membre en el moment d'emmagatzemar la despesa
   - Que la data de la despesa no sigui superior a la data de baixa del membre, si n'hi ha
   - L'existncia de la categoria personal, si s'indica
   - L'existncia de la forma de pagament, si s'indica
   - L'existncia de la situaci de despesa, si s'indica
   - Si s'indiquen els parmetres corresponents:
      - Regla de negoci 2: El clcul del import del IVA i del import net es fa a nivell de valors 
        automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0
      - Regla de negoci 8; Que el provedor estigui informat si la despesa > 30 euros.
      - Regla de negoci 10: No es pot entrar despeses en membres donats de baixa si la data de 
        despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
---------------------------------------------------------------------------------------------------
*/
-- MODIFICACIONS CORRECTES

dce.abm.despesa_membre_m(94,'04/11/2013','','','','','','','','','','', rsp);
dbms_output.put_line(proces || rsp);
dce.abm.despesa_membre_m(96,'','','',8435.86,'','','','','','','', rsp);
dbms_output.put_line(proces || rsp);

-- MODIFICACIONS INCORRECTES

-- No s'indica cap dada a modificar
dce.abm.despesa_membre_m(92,'','','','','','','','','','','', rsp);
dbms_output.put_line(proces || rsp);

-- Despesa personal inexistent
dce.abm.despesa_membre_m(44,'','','','','','Nova descripci de despesa','','','','','', rsp);
dbms_output.put_line(proces || rsp);  

-- S'indica un membre de cambra parlamentria inexistent
dce.abm.despesa_membre_m(95,'11/10/2013',98,2,25,21,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',1,1,'','',rsp);
dbms_output.put_line(proces || rsp);  

-- S'inidica una categoria personal inexistent
dce.abm.despesa_membre_m(95,'11/10/2013',2,96,25,21,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',1,1,'','',rsp);
dbms_output.put_line(proces || rsp);  

-- S'indica una forma de pagament inexistent
dce.abm.despesa_membre_m(95,'11/10/2013',2,2,25,21,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',97,1,'','',rsp);
dbms_output.put_line(proces || rsp);  

-- S'indica una situaci inexistent
dce.abm.despesa_membre_m(95,'11/10/2013',2,2,25,21,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',1,98,'','',rsp);
dbms_output.put_line(proces || rsp);  

-- S'indica una despesa > 30 euros i no s'indica el provedor
dce.abm.despesa_membre_m(95,'02/10/2013',3,1,250,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,2,'','',rsp);
dbms_output.put_line(proces || rsp);    

-- S'indica un import zero o menor de zero
dce.abm.despesa_membre_m(95,'11/10/2013',1,2,0,21,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',1,1,'','',rsp);
dbms_output.put_line(proces || rsp);  
dce.abm.despesa_membre_m(95,'11/10/2013',1,2,-23,21,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',1,1,'','',rsp);
dbms_output.put_line(proces || rsp);  

-- S'indica un tipus d'IVA zero o menor de zero
dce.abm.despesa_membre_m(95,'11/10/2013',1,2,125,0,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',1,1,'proveidor01','',rsp);
dbms_output.put_line(proces || rsp);  
dce.abm.despesa_membre_m(95,'11/10/2013',1,2,125,-19,'Descripci de la despesa fins a 50 caracters',
    'Justificaci de la despesa fins a 50 caracters',1,1,'proveidor01','',rsp);
dbms_output.put_line(proces || rsp);  

-- S'indica una data de despesa superior a la data de baixa d'un membre que ho est
dce.abm.despesa_membre_m(95,'02/11/2013',3,2,250,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,1,'','',rsp);
dbms_output.put_line(proces || rsp);    

-- Per a una alta, s'indica una data de despesa superior a la data de baixa d'un membre que ho est
proces := 'NOVA DESPESA PERSONAL DE MEMBRE DE CAMBRA PARLAMENTRIA: ';
dce.abm.despesa_membre_a('02/11/2013',3,1,25,21,'Descripci de la despesa fins a 50 caract',
    'Justificaci de la despesa fins a 50 caracters',1,2,'','',rsp);
dbms_output.put_line(proces || rsp);    

---------------------------------------------------------------------------------------------------
END;
/