---------------------------------------------------------------------------------------------------
-- UOC - ETIG - TFC Bases de dades relacionals	2013-2014 1r semestre
---------------------------------------------------------------------------------------------------
-- Base de Dades d'un Sistema Centralitzat de Control de Despesa Pblica dels Parlaments Europeus
---------------------------------------------------------------------------------------------------
-- Arxiu : 09_Paquet_ABM.sql        -- Autor    : Santi Alcayde Pagan   -- Aula: 1
-- Data  : 11/11/2013               -- Consultor: Manel Rella Ruiz      
---------------------------------------------------------------------------------------------------
-- Funci: Scripts de Creaci del PAQUET DE PROCEDIMENTS D'ALTA, BAIXA I MODIFICACI
-- El resultat de l'exeuci dels procediments es guarda en la taula d'auditoria LOG_PROCEDIMENT
---------------------------------------------------------------------------------------------------

-- *** OBSERVACI SOBRE NOMENCLATURES *************************************************************
-- Els noms dels parmetes d'entrada (input) comencen per i_
-- Els noms dels parmetres de sortida (output) comencen per o_
-- Els noms de les variables pel registre (log) de l'execuci dels procediments comencen per log_
-- Els noms de les variables pel tractament de les excepcions comencen per exc_
-- Els noms de la resta de variables comencen per v_
-- Els noms de dels cursors comencen per C_
-- Els noms de les vistes comencen per VIS_
-- Les funcions comencen per F_
-- Els procedimens de carcter general comencen per PRO_
-- No s'han escrit els accents en les paraules
--*************************************************************************************************

---------------------------------------------------------------------------------------------------
-- ***********************   C A P  A L E R A    D E L    P A Q U E T   *********************** --
---------------------------------------------------------------------------------------------------

CREATE OR REPLACE PACKAGE DCE.ABM IS

    -- Variables globals pel registre en la taula LOG_PROCEDIMENT
    log_proces   VARCHAR2(60);     -- Nom del procediment que s'ha executat
    log_entrada  VARCHAR2(1024);   -- Parmetres d'entrada del procediment
    log_resultat VARCHAR2(256);    -- Resultat de l'execucu del procediment
    log_abans    VARCHAR2(1024);   -- Valors abans d'executar el procediment, si en modifica
    log_despres  VARCHAR2(1024);   -- Valors desprs d'executar el procediment, si en modifica
    log_error    CHAR(1);          -- Indicador de si s'ha produt o no un error
    log_usuari   VARCHAR2(12);     -- Usuari de la bbdd que ha executat el procediment
    log_data     DATE;             -- Data i hora en que s'ha executat el procediment

---------------------------------------------------------------------------------------------------
-- *** 01. C A M B R E S    P A R L A M E N T  R I E S
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** F01.CAMBRES PARLAMENTRIES PER ESTAT
---------------------------------------------------------------------------------------------------
-- Funci que comprova si existeix un estat membre i si ja hi ha dues cambres d'alta per aquest.
-- Retorna: -1 si no existeix l'estat o, si existeix, el nmero de cambres que hi ha.
-- Procediments que la criden : CAMBRA_A, CAMBRA_B i CAMBRA_M
---------------------------------------------------------------------------------------------------
FUNCTION F_CAMBRA_ESTAT (
    i_estat IN CAMBRA_ESTAT.ESTAT%TYPE   -- Codi identificador bbdd del estat a comprovar
) RETURN NUMBER;

---------------------------------------------------------------------------------------------------
-- *** F02.CAMBRES PARLAMENTRIES PER PAIS O REGI
---------------------------------------------------------------------------------------------------
-- Funci que comprova si existeix un pas o regi i si ja hi ha una cambra d'alta per aquest.
-- Retorna: -1 si no existeix el pais o, si existeix, el nmero de cambres que hi ha. 
-- Procediments que la criden : CAMBRA_A, CAMBRA_B i CAMBRA_M
---------------------------------------------------------------------------------------------------
FUNCTION F_CAMBRA_PAIS_REGIO (
    i_pais IN CAMBRA_PAIS_REGIO.PAIS_REGIO%TYPE   -- Codi identificador bbdd del estat a comprovar
)RETURN NUMBER;

---------------------------------------------------------------------------------------------------
-- *** P01A.ALTA DE CAMBRA PARLAMENTRIA 
---------------------------------------------------------------------------------------------------
/* Insereix una nova cambra parlamentria en la taula CAMBRA.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Si tant <i_estat> com <i_pais>, sn tots dos a la vegada = 0  b > 0, es tracta com un error,
     ja que una cambra o b ho s d'un estat membre o b d'un pais o regi. 
   - L'existncia d'una cambra amb el mateix nom.
   - Si <i_estat> > 0 crida al procediment CAMBRA_ESTAT_A que fa l'alta en la taula CAMBRA_ESTAT
   - Si <i_pais> > 0 crida al procediment CAMBRA_PAIS_REGIO_A i fa l'alta en la taula 
     CAMBRA_PAIS_REGIO
   - Regles de negoci 3 i 4: L'existncia del estat membre o del pais/regi i del nmero de 
     cambres d'alta que tenen, que com a mxim s 1 per pais o regi i 2 per estat membre. 
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE CAMBRA_A (
    i_nom     IN  CAMBRA.CAMBRA%TYPE,  -- Nom de la cambra
    i_adreca  IN  CAMBRA.ADRECA%TYPE,  -- Adrea postal de la cambra
    i_ciutat  IN  CAMBRA.CIUTAT%TYPE,  -- Ciutat on es troba la cambra
    i_telefon IN  CAMBRA.TELEFON%TYPE, -- Telfon de contacte de la cambra
    i_llocWeb IN  CAMBRA.LLOC_WEB%TYPE,-- Adrea URL del lloc web de la cambra (opcional '' o null) 
    i_estat   IN  ESTAT.IBD%TYPE,      -- Identificador del estat membre (0 si s de pais o regi)
    i_pais    IN  PAIS_REGIO.IBD%TYPE, -- Identificador del pais o regi (0 si s d'estat membre)
    o_rsp     OUT VARCHAR2             -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** P01B.BAIXA/ALTA DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/*  Fa una baixa o alta lgica d'una cambra, informant una data, en la taula CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que existeixi la cambra parlmentria
   - Regla de negoci 5: Baixa/alta lgica d'una cambra parlamentria en la taula CAMBRA mitjanant
     una data de baixa que deixa la cambra inactiva/activa. Si es passa una data s'actualitza la 
     data de baixa i la cambra queda inactiva. Si no es passa cap data, es treu la data de baixa i
     la cambra torna a estar activa.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE CAMBRA_B (
    i_ibd     IN  CAMBRA.IBD%TYPE, -- Codi identificador bbdd de la cambra parlamentria
    i_data    IN  VARCHAR2,        -- Data de la baixa, cadena en format 'DD/MM/YYYY' 
                                   -- '' o null per tornar a donar d'alta.
    o_rsp     OUT VARCHAR2         -- Resultat de l'execuci del procediment
);

---------------------------------------------------------------------------------------------------
-- *** P01M.MODIFICACI DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Modifica les dades d'una cambra palamentria en la taula CAMBRA.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd de la cambra i_ibd i al menys un parmetre a modificar. 
   - Si tant <i_estat> com <i_pais>, sn tots dos a la vegada > 0, es tracta com un error, 
     ja que una cambra ho s, o b d'un estat membre, o b d'un pais o regi 
   - Si <i_estat> > 0  <i_pais> > 0, crida als procediments que fan alta, baixa i modificaci
     en les taules CAMBRA_ESTAT i CAMBRA_PAIS_REGIO.
   - Comprova que un estat no tingui ms de dues cambres i que un pas o regi noms en tingui una
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE CAMBRA_M (
    i_ibd     IN  CAMBRA.IBD%TYPE,      -- Codi identificador bbdd de la cambra parlamentria
    i_nom     IN  CAMBRA.CAMBRA%TYPE,   -- Nom  ('' o null per no modificar)
    i_adreca  IN  CAMBRA.ADRECA%TYPE,   -- Adrea postal  ('' o null per no modificar)
    i_ciutat  IN  CAMBRA.CIUTAT%TYPE,   -- Ciutat on es troba  ('' o null per no modificar)
    i_telefon IN  CAMBRA.TELEFON%TYPE,  -- Telfon de contacte  ('' o null er no modificar)
    i_llocWeb IN  CAMBRA.LLOC_WEB%TYPE, -- Adrea URL del lloc web  ('' o null per no modificar)
    i_estat   IN  ESTAT.IBD%TYPE,       -- Identificador del estat membre (0 per no modificar)
    i_pais    IN  PAIS_REGIO.IBD%TYPE,  -- Identificador del pais o regi (0 per no modificar)
    o_rsp     OUT VARCHAR2              -- Resultat de l'execuci del procediment 
);


---------------------------------------------------------------------------------------------------
-- *** 02. C A M B R E S    P A R L A M E N T  R I E S   E S T A T S    M E M B R E S
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P02A.ALTA DE CAMBRA PARLAMENTRIA D'ESTAT MEMBRE
---------------------------------------------------------------------------------------------------
-- Insereix una nova cambra en la taula CAMBRA_ESTAT 
-- Cridat des del procediment d'alta de cambra parlamentria CAMBRA_A
-- Regla de negoci 14: L'execuci del procediment es registra en la taula log_Procediment.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_ESTAT_A (
    i_cambra  IN  CAMBRA_ESTAT.CAMBRA%TYPE,  -- Codi identificador bbdd de la cambra
    i_estat   IN  CAMBRA_ESTAT.ESTAT%TYPE,   -- Codi identificador bbdd del estat membre
    o_rsp     OUT VARCHAR2                   -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** P02B.BAIXA DE CAMBRA PARLAMENTRIA D'ESTAT MEMBRE
---------------------------------------------------------------------------------------------------
-- Elimina una cambra en la taula CAMBRA_ESTAT 
-- Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_ESTAT_B (
    i_cambra  IN  CAMBRA_ESTAT.CAMBRA%TYPE,  -- Codi identificador bbdd de la cambra
    o_rsp     OUT VARCHAR2                   -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** P02M.MODIFICACI DE CAMBRA PARLAMENTRIA D'ESTAT MEMBRE
---------------------------------------------------------------------------------------------------
-- Modifica l'estat membre d'una cambra en la taula CAMBRA_ESTAT 
-- Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_ESTAT_M (
    i_cambra  IN  CAMBRA_ESTAT.CAMBRA%TYPE,  -- Codi identificador bbdd de la cambra
    i_estat   IN  CAMBRA_ESTAT.ESTAT%TYPE,   -- Codi identificador bbdd del estat membre
    o_rsp     OUT VARCHAR2                   -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** 03. C A M B R E S    P A R L A M E N T  R I E S   P A  S O S   O   R E G I O N S
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P03A.ALTA DE CAMBRA PARLAMENTRIA DE PAIS O REGI
---------------------------------------------------------------------------------------------------
-- Insereix una nova cambra en la taula CAMBRA_PAIS_REGIO
-- Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_PAIS_REGIO_A (
    i_cambra  IN  CAMBRA_PAIS_REGIO.CAMBRA%TYPE,     -- Codi identificador bbdd de la cambra
    i_pais    IN  CAMBRA_PAIS_REGIO.PAIS_REGIO%TYPE, -- Codi identificador bbdd del pais o regi
    o_rsp     OUT VARCHAR2                           -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** P03B.BAIXA DE CAMBRA PARLAMENTRIA DE PAIS O REGI
---------------------------------------------------------------------------------------------------
-- Elimina una cambra en la taula CAMBRA_PAIS_REGIO
-- Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_PAIS_REGIO_B (
    i_cambra  IN  CAMBRA_PAIS_REGIO.CAMBRA%TYPE,     -- Codi identificador bbdd de la cambra
    o_rsp     OUT VARCHAR2                           -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** P03M.MODIFICACI DE CAMBRA PARLAMENTRIA DE PAIS O REGI
---------------------------------------------------------------------------------------------------
-- Modifica el pais o regi d'una cambra en la taula CAMBRA_PAIS_REGIO
-- Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_PAIS_REGIO_M (
    i_cambra  IN  CAMBRA_PAIS_REGIO.CAMBRA%TYPE,     -- Codi identificador bbdd de la cambra
    i_pais    IN  CAMBRA_PAIS_REGIO.PAIS_REGIO%TYPE, -- Codi identificador bbdd del pais o regi
    o_rsp     OUT VARCHAR2                           -- Resultat de l'execuci del procediment 
);


---------------------------------------------------------------------------------------------------
-- *** 04. M E M B R E  S    D E    C A M B R E S    P A R L A M E N T  R I E S 
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P04A.ALTA DE MEMBRE DE CAMBRA PARLAMENTRIA
/* Insereix un nou membre d'una cambra parlamentria en la taula MEMBRE_CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra i obtenci del nmero actual de membres
   - Regla de negoci 6: La no existncia d'el nmero identificador de membre dins la cambra indicada
   - L'existncia del partit poltic
   - Regla de negoci 7: Incrementa el comptador de nmero de membres de la cambra parlamentria 
     de la que formar part.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE MEMBRE_A (
    i_codi    IN  MEMBRE_CAMBRA.CODI%TYPE,     -- Codi identificador (NIF, etc.)
    i_nom     IN  MEMBRE_CAMBRA.NOM%TYPE,      -- Nom del membre de la cambra
    i_cognoms IN  MEMBRE_CAMBRA.COGNOMS%TYPE,  -- Cognoms del membre de la cambra
    i_telefon IN  MEMBRE_CAMBRA.TELEFON%TYPE,  -- Telfon del membre de la cambra
    i_email   IN  MEMBRE_CAMBRA.EMAIL%TYPE,    -- Adrea elctrnica del membre de la cambra
    i_partit  IN  MEMBRE_CAMBRA.PARTIT%TYPE,   -- Identificador bbdd del partit al que pertany
    i_cambra  IN  MEMBRE_CAMBRA.CAMBRA%TYPE,   -- Identificador bbdd de la cambra a la que pertany
    i_numero  IN  MEMBRE_CAMBRA.NUMERO%TYPE,   -- Nmero identificador dins de la cambra
    o_rsp     OUT VARCHAR2                     -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** P04B.BAIXA/ALTA DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Fa una baixa o alta lgica d'un membre, informant una data, en la taula MEMBRE_CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que existeixi el membre de la cambra.
   - Regla de negoci 5: Baixa/alta lgica d'un membre d'una cambra parlamentria en la taula 
     MEMBRE_CAMBRA mitjanant una data de baixa que deixa el membre de la cambra inactiu/actiu.
   - Regla de negoci 5 i 7: Si es passa una data s'actualitza la data de baixa, el membre de la 
     cambra queda inactiu, i es decrementa el comptador de nmero de membres de la cambra 
     parlamentria a la que pertany. Si no es passa cap data, si el membre est de baixa, 
     es treu la data de baixa, el membre torna a estar actiu, i s'incrementa el comptador de 
     nmero de membres de la cambra parlamentria a la que pertany.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE MEMBRE_B (
    i_ibd   IN  MEMBRE_CAMBRA.IBD%TYPE, -- Codi identificador bbdd del membre de la cambra
    i_data  IN  VARCHAR2,               -- Data de la baixa, cadena en format 'DD/MM/YYYY' 
                                        -- '' o null per tornar a donar d'alta.
    o_rsp   OUT VARCHAR2                -- Resultat de l'execuci del procediment
);

---------------------------------------------------------------------------------------------------
-- *** P04M.MODIFICACI DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Modifica les dades d'un membre d'una cambra palamentria en la taula MEMBRE_CAMBRA.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd del membre de la cambra i al menys un parmetre a modificar. 
   - Regle de negoci 7: Si es canvia la cambra parlamentria actualitza els comptadors de nmero 
     de membres de l'antiga cambra i de la nova.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE MEMBRE_M (
    i_ibd     IN  MEMBRE_CAMBRA.IBD%TYPE,      -- Codi identificador bbdd del membre de la cambra
    i_codi    IN  MEMBRE_CAMBRA.CODI%TYPE,     -- Codi identificador (NIF, etc.)
    i_nom     IN  MEMBRE_CAMBRA.NOM%TYPE,      -- Nom del membre de la cambra
    i_cognoms IN  MEMBRE_CAMBRA.COGNOMS%TYPE,  -- Cognoms del membre de la cambra
    i_telefon IN  MEMBRE_CAMBRA.TELEFON%TYPE,  -- Telfon del membre de la cambra
    i_email   IN  MEMBRE_CAMBRA.EMAIL%TYPE,    -- Adrea elctrnica del membre de la cambra
    i_partit  IN  MEMBRE_CAMBRA.PARTIT%TYPE,   -- Codi identificador bbdd del partit al que pertany
    i_cambra  IN  MEMBRE_CAMBRA.CAMBRA%TYPE,   -- Codi identif. bbdd de la cambra a la que pertany
    i_numero  IN  MEMBRE_CAMBRA.NUMERO%TYPE,   -- Nmero identificador dins de la cambra
    o_rsp     OUT VARCHAR2                     -- Resultat de l'execuci del procediment 
);


---------------------------------------------------------------------------------------------------
-- *** 05. D E S P E S E S  
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P05A.ALTA DE DESPESA GENERAL DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Insereix una nova despesa general d'una cambra parlamentria en les taules DESPESA i
   DESPESA_CAMBRA 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria
   - L'existncia de la categoria general
   - L'existncia de la forma de pagament
   - L'existncia de la situaci de despesa
   - Regla de negoci 2: Que el provedor estigui informat si la despesa > 30 euros.
   - Regla de negoci 8: El clcul del import del IVA i del import net es fa a nivell de valors 
     automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0
   - Regla de negoci 9: No es pot entrar despeses en cambres donades de baixa si la data de 
     despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_CAMBRA_A (
    i_data         IN  DESPESA.DATA_DESPESA%TYPE,     -- Data de la despesa
    i_cambra       IN  DESPESA_CAMBRA.CAMBRA%TYPE,    -- Codi identificador bbdd de la cambra
    i_categoria    IN  DESPESA_CAMBRA.CATEGORIA%TYPE, -- Categoria de la despesa
    i_import       IN  DESPESA.IMPORT_BRUT%TYPE,      -- Import brut de la despesa
    i_tipusIva     IN  DESPESA.TIPUS_IVA%TYPE,        -- Tipus d'IVA de la  despesa
    i_descripcio   IN  DESPESA.DESCRIPCIO%TYPE,       -- Descripci de la despesa
    i_justificacio IN  DESPESA.JUSTIFICACIO%TYPE,     -- Justificaci de la despesa
    i_pagament     IN  DESPESA.PAGAMENT%TYPE,         -- Forma de pagament de la despesa
    i_situacio     IN  DESPESA.SITUACIO%TYPE,         -- Situaci de la despesa
    i_proveidor    IN  DESPESA.PROVEIDOR%TYPE,        -- Provedor 
    i_escaneig     IN  DESPESA.URL_ESCANEIG%TYPE,     -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2                       -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** P05M.MODIFICACI DE DESPESA GENERAL DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Modifica les dades d'una despesa general d'una cambra parlamentria en les taules DESPESA i
   DESPESA_CAMBRA 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd de la despesa i al menys un parmetre a modificar. 
   - L'existncia de la cambra parlamentria si s'indica
   - L'existncia de la categoria general, si s'indica
   - L'existncia de la forma de pagament, si s'indica
   - L'existncia de la situaci de despesa, si s'indica
   - Si s'indiquen els parmetres corresponents:
      - Regla de negoci 2: El clcul del import del IVA i del import net es fa a nivell de valors 
        automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0   
      - Regla de negoci 8: Que el provedor estigui informat si la despesa > 30 euros.
   - Regla de negoci 9: No es pot entrar despeses en cambres donades de baixa si la data de 
     despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_CAMBRA_M (
    i_ibd          IN  DESPESA.IBD%TYPE,              -- Codi identificador bbdd de la despesa
    i_data         IN  VARCHAR2,                      -- Data de la despesa en format dd/mm/yyyy
    i_cambra       IN  DESPESA_CAMBRA.CAMBRA%TYPE,    -- Codi identificador bbdd de la cambra
    i_categoria    IN  DESPESA_CAMBRA.CATEGORIA%TYPE, -- Categoria de la despesa
    i_import       IN  DESPESA.IMPORT_BRUT%TYPE,      -- Import brut de la despesa
    i_tipusIva     IN  DESPESA.TIPUS_IVA%TYPE,        -- Tipus d'IVA de la  despesa
    i_descripcio   IN  DESPESA.DESCRIPCIO%TYPE,       -- Descripci de la despesa
    i_justificacio IN  DESPESA.JUSTIFICACIO%TYPE,     -- Justificaci de la despesa
    i_pagament     IN  DESPESA.PAGAMENT%TYPE,         -- Forma de pagament de la despesa
    i_situacio     IN  DESPESA.SITUACIO%TYPE,         -- Situaci de la despesa
    i_proveidor    IN  DESPESA.PROVEIDOR%TYPE,        -- Provedor 
    i_escaneig     IN  DESPESA.URL_ESCANEIG%TYPE,     -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2                       -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** 06. D E S P E S E S    P E R S O N A L S    D E     M E M B R E S    D E    C A M B R E S
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P06A.ALTA DE DESPESA PERSONAL DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Insereix una nova despesa personal d'un membre d'una cambra en les taules DESPESA i 
   DESPESA_MEMBRE 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia del membre de la cambra parlamentria
   - L'obtenci del partit poltic del membre en el moment d'emmagatzemar la despesa
   - L'existncia de la categoria personal
   - L'existncia de la forma de pagament
   - L'existncia de la situaci de despesa
   - Regla de negoci 2: El clcul del import del IVA i del import net es fa a nivell de valors 
     automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0
   - Regla de negoci 8; Que el provedor estigui informat si la despesa > 30 euros.
   - Regla de negoci 10: No es pot entrar despeses en membres donats de baixa si la data de 
     despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_MEMBRE_A (
    i_data         IN  DESPESA.DATA_DESPESA%TYPE,     -- Data de la despesa
    i_membre       IN  DESPESA_MEMBRE.MEMBRE%TYPE,    -- Codi dentificador bbdd del membre
    i_categoria    IN  DESPESA_MEMBRE.CATEGORIA%TYPE, -- Categoria de la despesa
    i_import       IN  DESPESA.IMPORT_BRUT%TYPE,      -- Import brut de la despesa
    i_tipusIva     IN  DESPESA.TIPUS_IVA%TYPE,        -- Tipus d'IVA de la  despesa
    i_descripcio   IN  DESPESA.DESCRIPCIO%TYPE,       -- Descripci de la despesa
    i_justificacio IN  DESPESA.JUSTIFICACIO%TYPE,     -- Justificaci de la despesa
    i_pagament     IN  DESPESA.PAGAMENT%TYPE,         -- Forma de pagament de la despesa
    i_situacio     IN  DESPESA.SITUACIO%TYPE,         -- Situaci de la despesa
    i_proveidor    IN  DESPESA.PROVEIDOR%TYPE,        -- Provedor 
    i_escaneig     IN  DESPESA.URL_ESCANEIG%TYPE,     -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2                       -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** P056M.MODIFICACI DE DESPESA PERSONAL DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Modifica les dades d'una despesa personal d'un membre en les taules DESPESA i
   DESPESA_MEMBRE 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd de la despesa i al menys un parmetre a modificar. 
   - L'existncia del membre de cambra parlamentria si s'indica
   - L'obtenci del partit poltic del membre en el moment d'emmagatzemar la despesa
   - Que la data de la despesa no sigui superior a la data de baixa del membre, si n'hi ha
   - L'existncia de la categoria personal, si s'indica
   - L'existncia de la forma de pagament, si s'indica
   - L'existncia de la situaci de despesa, si s'indica
   - Si s'indiquen els parmetres corresponents:
      - Regla de negoci 8; Que el provedor estigui informat si la despesa > 30 euros.
      - Regla de negoci 2: El clcul del import del IVA i del import net es fa a nivell de valors 
        automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0
      - Regla de negoci 10: No es pot entrar despeses en membres donats de baixa si la data de 
        despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_MEMBRE_M (
    i_ibd          IN  DESPESA.IBD%TYPE,              -- Codi dentificador bbdd de la despesa
    i_data         IN  VARCHAR2,                      -- Data de la despesa en format 'DD/MM/YYYY' 
    i_membre       IN  DESPESA_MEMBRE.MEMBRE%TYPE,    -- Codi identificador bbdd del membre
    i_categoria    IN  DESPESA_MEMBRE.CATEGORIA%TYPE, -- Categoria de la despesa
    i_import       IN  DESPESA.IMPORT_BRUT%TYPE,      -- Import brut de la despesa
    i_tipusIva     IN  DESPESA.TIPUS_IVA%TYPE,        -- Tipus d'IVA de la  despesa
    i_descripcio   IN  DESPESA.DESCRIPCIO%TYPE,       -- Descripci de la despesa
    i_justificacio IN  DESPESA.JUSTIFICACIO%TYPE,     -- Justificaci de la despesa
    i_pagament     IN  DESPESA.PAGAMENT%TYPE,         -- Forma de pagament de la despesa
    i_situacio     IN  DESPESA.SITUACIO%TYPE,         -- Situaci de la despesa
    i_proveidor    IN  DESPESA.PROVEIDOR%TYPE,        -- Codi provedor 
    i_escaneig     IN  DESPESA.URL_ESCANEIG%TYPE,     -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2                       -- Resultat de l'execuci del procediment 
);

---------------------------------------------------------------------------------------------------
-- *** P05/06B.BAIXA DE DESPESA 
---------------------------------------------------------------------------------------------------
/* Elimina una despesa en les taules DESPESA i DESPESA_CAMBRA o DESPESA_MEMBRE 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_B (
    i_despesa  IN  DESPESA.IBD%TYPE,   -- Codi identificador bbdd de la despesa
    i_tipus    IN  CHAR,               -- Indicador del tipus de despesa: C=Cambra, M=Membre
    o_rsp      OUT VARCHAR2            -- Resultat de l'execuci del procediment 
);


END;
/

---------------------------------------------------------------------------------------------------
-- ***************************     C O S    D E L    P A Q U E T      ************************** --
---------------------------------------------------------------------------------------------------

CREATE OR REPLACE PACKAGE BODY DCE.ABM IS

---------------------------------------------------------------------------------------------------
-- *** 01. C A M B R E S    P A R L A M E N T  R I E S
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** F01.CAMBRES PARLAMENTRIES PER ESTAT
---------------------------------------------------------------------------------------------------
-- Funci que comprova si existeix un estat membre i si ja hi ha dues cambres d'alta per aquest.
-- Retorna: -1 si no existeix l'estat o, si existeix, el nmero de cambres que hi ha.
-- Procediments que la criden : CAMBRA_A, CAMBRA_B i CAMBRA_M
---------------------------------------------------------------------------------------------------
FUNCTION F_CAMBRA_ESTAT (
    i_estat IN CAMBRA_ESTAT.ESTAT%TYPE   -- Codi identificador bbdd del estat a comprovar
) RETURN NUMBER AS
    
    v_count NUMBER(3);    -- Comptador

BEGIN
    v_count   := 0;
    
    SELECT COUNT(*) INTO v_count FROM dce.estat WHERE ibd = i_estat;         
    IF (v_count = 0) THEN return -1; END IF;
    
    SELECT COUNT(*) INTO v_count FROM dce.cambra c, dce.cambra_estat ce
        WHERE c.ibd = ce.cambra AND ce.estat = i_estat AND c.data_baixa IS NULL; 
    RETURN v_count;
        	    
END F_CAMBRA_ESTAT;

---------------------------------------------------------------------------------------------------
-- *** F02.CAMBRES PARLAMENTRIES PER PAIS O REGI
---------------------------------------------------------------------------------------------------
-- Funci que comprova si existeix un pas o regi i si ja hi ha una cambra d'alta per aquest.
-- Retorna: -1 si no existeix el pais o, si existeix, el nmero de cambres que hi ha. 
-- Procediments que la criden : CAMBRA_A, CAMBRA_B i CAMBRA_M
---------------------------------------------------------------------------------------------------
FUNCTION F_CAMBRA_PAIS_REGIO (
    i_pais IN CAMBRA_PAIS_REGIO.PAIS_REGIO%TYPE   -- Codi identificador bbdd del estat a comprovar
)RETURN NUMBER AS

    v_count NUMBER(3);    -- Comptador

BEGIN
    v_count   := 0;
    
    SELECT COUNT(*) INTO v_count FROM dce.pais_regio WHERE ibd = i_pais;         
    IF (v_count = 0) THEN return -1; END IF;
    
    SELECT COUNT(*) INTO v_count FROM dce.cambra c, dce.cambra_pais_regio cp
        WHERE c.ibd = cp.cambra AND cp.pais_regio = i_pais AND c.data_baixa IS NULL; 
    return v_count;
    
END F_CAMBRA_PAIS_REGIO;

---------------------------------------------------------------------------------------------------
-- *** P01A.ALTA DE CAMBRA PARLAMENTRIA 
---------------------------------------------------------------------------------------------------
/* Insereix una nova cambra parlamentria en la taula CAMBRA.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Si tant <i_estat> com <i_pais>, sn tots dos a la vegada = 0  b > 0, es tracta com un error,
     ja que una cambra o b ho s d'un estat membre o b d'un pais o regi. 
   - L'existncia d'una cambra amb el mateix nom.
   - Si <i_estat> > 0 crida al procediment CAMBRA_ESTAT_A que fa l'alta en la taula CAMBRA_ESTAT
   - Si <i_pais> > 0 crida al procediment CAMBRA_PAIS_REGIO_A i fa l'alta en la taula 
     CAMBRA_PAIS_REGIO
   - Regles de negoci 3 i 4: L'existncia del estat membre o del pais/regi i del nmero de 
     cambres d'alta que tenen, que com a mxim s 1 per pais o regi i 2 per estat membre. 
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE CAMBRA_A (
    i_nom     IN  CAMBRA.CAMBRA%TYPE,  -- Nom de la cambra
    i_adreca  IN  CAMBRA.ADRECA%TYPE,  -- Adrea postal de la cambra
    i_ciutat  IN  CAMBRA.CIUTAT%TYPE,  -- Ciutat on es troba la cambra
    i_telefon IN  CAMBRA.TELEFON%TYPE, -- Telfon de contacte de la cambra
    i_llocWeb IN  CAMBRA.LLOC_WEB%TYPE,-- URL del lloc web de la cambra (opcional '' o null) 
    i_estat   IN  ESTAT.IBD%TYPE,      -- Identificador del estat membre (0 si s de pais o regi)
    i_pais    IN  PAIS_REGIO.IBD%TYPE, -- Identificador del pais o regi (0 si s d'estat membre)
    o_rsp     OUT VARCHAR2             -- Resultat de l'execuci del procediment 
) IS    
    -- Variables pel tractament d'excepcions
    exc_noNom               EXCEPTION; -- No s'ha indicat el parmetre i_nom
    exc_noAdreca            EXCEPTION; -- No s'ha indicat el parmetre i_adreca
    exc_noCiutat            EXCEPTION; -- No s'ha indicat el parmetre i_ciutat
    exc_noTelefon           EXCEPTION; -- No s'ha indicat el parmetre i_telefon
    exc_noEstatNoPais       EXCEPTION; -- No s'ha indicat ni i_estat ni i_pais
    exc_siEstatSiPais       EXCEPTION; -- S'ha indicat tant i_estat com i_pais
    exc_noTrobaEstat        EXCEPTION; -- No s'ha trobat l'estat membre i_estat
    exc_noTrobaPais         EXCEPTION; -- No s'ha trobat el pais o regi i_pais
    exc_jaHiHaCambra        EXCEPTION; -- Ja existeix la cambra indicada a i_nom  
    exc_jaHiHaCambraPais    EXCEPTION; -- Ja existeix una altra cambra amb i_pais
    exc_jaHiHa2CambraEstat  EXCEPTION; -- Ja existeixen dues cambres amb i_estat
      	
    -- Variables per a la inserci de dades
    v_nom          VARCHAR2(50); -- Nom de la cambra a inserir
    v_adreca       VARCHAR2(50); -- Adrea postal de la cambra a inserir
    v_ciutat       VARCHAR2(30); -- Ciutat de la cambra a inserir
    v_telefon      VARCHAR2(20); -- Telfon de la cambra a inserir
    v_llocWeb      VARCHAR2(50); -- Adrea URL del lloc web de la cambra a inserir
    v_idEstat      NUMBER(3);    -- Identificador bbdd del estat membre a inserir
    v_idPais       NUMBER(3);    -- Identificador bbdd del pais o regi a inserir
    v_idCambra     NUMBER(3);    -- Identificador bbdd de la cambra
    v_declaracio   NUMBER(2);    -- Nmero declaraci sentncia SQL
  	v_count        NUMBER(3);    -- Comptador

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.CAMBRA_A';
    log_entrada  := '<' || i_nom || '> <' || i_adreca || '> <' || i_ciutat
        || '> <' || i_telefon || '> <' || i_estat || '> <' || i_pais || '>';
    log_resultat := 'OK. ALTA DE NOVA CAMBRA EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_nom IS NULL OR i_nom = '') THEN RAISE exc_noNom; END IF;
    IF (i_adreca IS NULL OR i_adreca = '') THEN RAISE exc_noAdreca; END IF;
    IF (i_ciutat IS NULL OR i_ciutat = '') THEN RAISE exc_noCiutat; END IF;
    IF (i_telefon IS NULL OR i_telefon = '') THEN RAISE exc_noTelefon; END IF;
    IF ((i_estat IS NULL OR i_estat = 0) AND (i_pais IS NULL OR i_pais = 0))  
        THEN RAISE exc_noEstatNoPais; 
    END IF;
    IF ((i_estat > 0) AND (i_pais > 0)) THEN RAISE exc_siEstatSiPais; END IF;
    
    -- Inicialitzaci de les variables de la inserci de dades
    v_nom        := UPPER(i_nom);
    v_adreca     := UPPER(i_adreca);
    v_ciutat     := UPPER(i_ciutat);
    v_telefon    := UPPER(i_telefon);
    v_llocWeb    := LOWER(i_llocWeb);
    v_idEstat    := 0;
    v_idPais     := 0;
    v_declaracio := 0;
    v_count      := 0;
    
    -- Comprovaci de l'existencia del estat membre o del pais/regi i 
    -- del nmero de cambres d'alta
    IF (i_estat > 0) THEN
		v_count := F_CAMBRA_ESTAT(i_estat);
		IF v_count = -1 THEN RAISE exc_noTrobaEstat; END IF;
        IF v_count > 1 THEN RAISE exc_jaHiHa2CambraEstat; END IF;
    ELSE
		v_count := F_CAMBRA_PAIS_REGIO(i_pais);
		IF v_count = -1 THEN RAISE exc_noTrobaPais; END IF;
        IF v_count > 0 THEN RAISE exc_jaHiHaCambraPais; END IF;
    END IF;

    -- Comprovaci de l'existncia de la cambra
    SELECT COUNT(*) INTO v_count FROM dce.cambra WHERE UPPER(cambra) = v_nom;
    IF (v_count > 0) THEN RAISE exc_jaHiHaCambra; END IF;
 
    -- Execuci: Alta de la cambra en la relaci pare
    v_declaracio := 1;
    INSERT INTO DCE.CAMBRA (CAMBRA, ADRECA, CIUTAT, TELEFON, LLOC_WEB)
        VALUES (v_nom, v_adreca, v_ciutat, v_telefon, v_llocWeb);

    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
    
    -- Execuci: Alta de la cambra en la relaci filla d'acord a si es tracte
    -- d'una cambra d'estat membre o de pais/regi, i amb el valor actual
    -- de la seqncia de cambres CAM_SEQ_IBD.currval
    v_idCambra := CAM_SEQ_IBD.currval;
    IF (i_Estat > 0) THEN
        v_declaracio := 4;
        -- Crida al procediment que fa l'alta de la cambra d'estat membre
        DCE.ABM.CAMBRA_ESTAT_A(v_idCambra, i_Estat, o_rsp);
    ELSE
        v_declaracio := 5;
        -- Crida al procediment que fa l'alta de la cambra de paiso regi
        DCE.ABM.CAMBRA_PAIS_REGIO_A(v_idCambra, i_Pais, o_rsp);

    END IF;
            
    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noNom THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL NOM DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noAdreca THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT L''ADREA POSTAL DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noCiutat THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA CIUTAT ON ES TROBA LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTelefon THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL TELFON DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noEstatNoPais THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT NI L'' ESTAT MEMBRE NI EL PAS O REGI' ||
            ' DE LA CAMBRA PARLAMENTRIA. CAL INDICAR-NE UN DELS DOS';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_siEstatSiPais THEN 
        log_resultat := 
            'ERROR: S''HA INDICAT A LA VEGADA L'' ESTAT MEMBRE I EL PAS O REGI' ||
            ' DE LA CAMBRA PARLAMENTRIA. CAL INDICAR-NE UN DE SOL.';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaEstat THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT L''ESTAT MEMBRE. (IBD ' || i_estat || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
                   
    WHEN exc_noTrobaPais THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT EL PAIS O REGI. (IBD ' || i_pais || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHaCambra THEN
        log_resultat := 
            'ERROR: JA EXISTEIX UNA CAMBRA AMB AQUEST NOM: ' || i_nom;
        o_rsp := log_resultat;
         PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHa2CambraEstat THEN
        log_resultat := 
            'ERROR: JA EXISTEIXEN DUES CAMBRES D''ALTA PER AQUEST ESTAT (IBD ' || i_estat || ')';
        o_rsp := log_resultat;
         PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHaCambraPais THEN
        log_resultat := 
            'ERROR: JA EXISTEIX UNA CAMBRA D''ALTA PER AQUEST PAIS O REGI. (IBD '|| i_pais ||')';
        o_rsp := log_resultat;
         PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);


--    WHEN DUP_VAL_ON_INDEX THEN
--        CASE v_declaracio
--  La seqncia s'incrementa al fer el insert. Via exc_jaHiHaCambra no s'incrementa
--            WHEN 1 THEN log_resultat := 
--               'ERROR: VALOR DUPLICAT EN NDEX A CAMBRA. NOM: ' || i_nom;
--        END CASE;
--        ROLLBACK;
--        o_rsp := log_resultat;
--        PRO_LOG_PROCEDIMENT
--        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRA_A;

---------------------------------------------------------------------------------------------------
-- *** P01B.BAIXA/ALTA DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/*  Fa una baixa o alta lgica d'una cambra, informant una data, en la taula CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que existeixi la cambra parlmentria
   - Regla de negoci 5: Baixa/alta lgica d'una cambra parlamentria en la taula CAMBRA mitjanant
     una data de baixa que deixa la cambra inactiva/activa. Si es passa una data s'actualitza la 
     data de baixa i la cambra queda inactiva. Si no es passa cap data, es treu la data de baixa i
     la cambra torna a estar activa.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE CAMBRA_B (
    i_ibd     IN  CAMBRA.IBD%TYPE, -- Codi identificador bbdd de la cambra parlamentria
    i_data    IN  VARCHAR2,        -- Data de la baixa, cadena en format 'DD/MM/YYYY' 
                                   -- '' o null per tornar a donar d'alta.
    o_rsp     OUT VARCHAR2         -- Resultat de l'execuci del procediment
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra            EXCEPTION;  -- No s'ha indicat el parmetre i_cambra
    exc_noTrobaEstat        EXCEPTION;  -- No s'ha trobat l'estat membre 
    exc_noTrobaPais         EXCEPTION;  -- No s'ha trobat el pais o regi
    exc_jaHiHaCambraPais    EXCEPTION;  -- Ja existeix una altra cambra per pais
    exc_jaHiHa2CambraEstat  EXCEPTION;  -- Ja existeixen dues cambres per estat
    exc_noTrobaTipusCambra  EXCEPTION;  -- No s'ha trobat la cambra ni per estat ni per pais
    exc_jaEsAlta            EXCEPTION;  -- La cambra ja es troba en situaci d'alta
   	
    -- Variables per a la inserci i la comprovaci de la baixa
    v_dataIns      DATE;          -- Data de la baixa en format de data
    v_dataActual   VARCHAR2(10);  -- Valor de la data de baixa actual de la cambra
    v_idEstatPais  NUMBER(3);     -- Identifcador del estat o delpais o regi
    v_count        NUMBER(3);     -- Comptador
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces    := 'ABM.CAMBRA_B'; 
    log_entrada   := '<' || i_ibd || '> <' || i_data || '>';
    log_abans     := '';
    log_despres   := '';
    log_error     := 'S';
    log_usuari    := USER;
    log_data      := SYSDATE;
    o_rsp         := 'OK';

    -- Inicialitzaci de les variables per a la inserci i la comprovaci de la baixa
    v_dataIns     := null;
    v_dataActual  := '';
    v_idEstatPais := 0;
    v_count       := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_ibd IS NULL OR i_ibd = '') THEN RAISE exc_noCambra; END IF;
    
    -- Comprovaci de l'existncia de la cambra parlamentria
    SELECT TO_CHAR(data_baixa,'dd/mm/yyyy') INTO v_dataActual 
        FROM dce.cambra WHERE ibd = i_ibd;
    
    IF ((i_data IS NULL OR i_data = '') AND (v_dataActual IS NULL OR v_dataActual = '')) THEN 
        -- Ja es troba en situaci d'alta
        RAISE exc_jaEsAlta;       
    END IF;

    -- Si es vol tornar a deixar d'alta es comprova el nmero de cambres d'alta 
    -- per estat o per pais o regi
    IF (i_data IS NULL OR i_data = '') THEN  
        IF (v_dataActual IS NULL OR v_dataActual = '') THEN 
            -- Ja es troba en situaci d'alta
            RAISE exc_jaEsAlta;              
        ELSE
            -- Comprovaci de si la cambra pertany a un estat mebre
            SELECT COUNT(*) INTO v_count FROM dce.cambra_estat WHERE cambra = i_ibd;

            -- No pertany a un estat membre
            IF (v_count = 0) THEN  
                SELECT COUNT(*) INTO v_count FROM dce.cambra_pais_regio WHERE cambra = i_ibd;

                -- Tampoc pertany a un pais o regi
                IF (v_count = 0) THEN RAISE exc_noTrobaTipusCambra; END IF;

                -- Pertany a un pais o regi
                SELECT pais_regio INTO v_idEstatPais FROM dce.cambra_pais_regio 
                    WHERE cambra = i_ibd;

	            -- Comprovaci del nmero de cambres d'alta
                v_count := F_CAMBRA_PAIS_REGIO(v_idEstatPais);
                IF v_count = -1 THEN RAISE exc_noTrobaPais; END IF;
                IF v_count > 0 THEN RAISE exc_jaHiHaCambraPais; END IF;
            ELSE
                -- Pertnay a un estat membre
                SELECT estat INTO v_idEstatPais FROM dce.cambra_estat WHERE cambra = i_ibd; 

                -- Comprovaci del nmero de cambres d'alta
                v_count := F_CAMBRA_ESTAT(v_idEstatPais);
                IF v_count = -1 THEN RAISE exc_noTrobaEstat; END IF;
                IF v_count > 1 THEN RAISE exc_jaHiHa2CambraEstat; END IF;
            END IF;
        
            v_dataIns := null;
            log_resultat := 'OK. CAMBRA EN SITUACI D''ALTA';
         
        END IF;
    
    -- Si es vol deixar de baixa
    ELSE   
        log_resultat := 'OK. CAMBRA EN SITUACI DE BAIXA';    
        IF (v_dataActual IS NOT NULL) THEN 

            -- Ja es troba en situaci de baixa  
            log_resultat := 'OK. PER LA CAMBRA JA EST EN SITUACI DE BAIXA'; 
            log_abans    := 'Data de baixa anterior: ' || v_dataActual;
            log_despres  := 'Nova data de baixa: ' || i_data;
        END IF;    
        v_dataIns := TO_DATE(i_data);    
    END IF;
             
    -- Execuci: Baixa o Alta de la cambra
    UPDATE DCE.CAMBRA SET data_baixa = v_dataIns WHERE ibd = i_ibd;
    
    -- Execuci correcte del procediment
    log_error := 'N';
    o_rsp := log_resultat;
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
        
    -- Control d'excepcions
    EXCEPTION
    WHEN exc_noCambra THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA. (IBD ' || i_ibd || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaEsAlta THEN
        log_resultat := 
            'ERROR: JA ES TROBA D''ALTA LA CAMBRA. (IBD ' || i_ibd || ')';
        o_rsp := log_resultat;
         PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaEstat THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT L''ESTAT MEMBRE. (IBD ' || v_idEstatPais || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
                   
    WHEN exc_noTrobaPais THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT EL PAIS O REGI. (IBD ' || v_idEstatPais || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaTipusCambra THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT NI L''ESTAT MEMBRE NI EL PAIS O ' ||
            'REGI D''AQUESTA CAMBRA (IBD ' || i_ibd || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHa2CambraEstat THEN
        log_resultat := 
            'ERROR: JA EXISTEIXEN DUES CAMBRES D''ALTA PER AQUEST ESTAT. (IBD ' || 
             v_idEstatPais || ')';
        o_rsp := log_resultat; 
         PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHaCambraPais THEN
        log_resultat := 
            'ERROR: JA EXISTEIX UNA CAMBRA D''ALTA PER AQUEST PAIS O REGI. (IBD ' || 
             v_idEstatPais || ')';
        o_rsp := log_resultat;
         PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRA_B;

---------------------------------------------------------------------------------------------------
-- *** P01M.MODIFICACI DE CAMBRA PARLAMENTRIA 
---------------------------------------------------------------------------------------------------
/* Modifica les dades d'una cambra palamentria en la taula CAMBRA.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd de la cambra i_ibd i al menys un parmetre a modificar. 
   - Si tant <i_estat> com <i_pais>, sn tots dos a la vegada > 0, es tracta com un error, 
     ja que una cambra ho s, o b d'un estat membre, o b d'un pais o regi 
   - Si <i_estat> > 0  <i_pais> > 0, crida als procediments que fan alta, baixa i modificaci
     en les taules CAMBRA_ESTAT i CAMBRA_PAIS_REGIO.
   - Comprova que un estat no tingui ms de dues cambres i que un pas o regi noms en tingui una
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE CAMBRA_M (
    i_ibd     IN  CAMBRA.IBD%TYPE,      -- Codi identificador bbdd de la cambra parlamentria
    i_nom     IN  CAMBRA.CAMBRA%TYPE,   -- Nom de la cambra 
    i_adreca  IN  CAMBRA.ADRECA%TYPE,   -- Adrea postal de la cambra 
    i_ciutat  IN  CAMBRA.CIUTAT%TYPE,   -- Ciutat on es troba la cambra 
    i_telefon IN  CAMBRA.TELEFON%TYPE,  -- Telfon de contacte
    i_llocWeb IN  CAMBRA.LLOC_WEB%TYPE, -- Adrea URL del lloc web 
    i_estat   IN  ESTAT.IBD%TYPE,       -- Codi identificador bbdd del estat membre 
    i_pais    IN  PAIS_REGIO.IBD%TYPE,  -- Codi identificador bbdd del pais o regi 
    o_rsp     OUT VARCHAR2              -- Resultat de l'execuci del procediment 
) IS    
    -- Variables pel tractament d'excepcions
    exc_noIbd                EXCEPTION; -- No s'ha indicat el parmetre i_ibd
    exc_capModificacio       EXCEPTION; -- No s'ha indicat cap parmetre a modificar
    exc_siEstatSiPais        EXCEPTION; -- S'ha indicat tant i_estat com i_pais
    exc_noTrobaEstat         EXCEPTION; -- No s'ha trobat l'estat nou indicat ai_estat
    exc_noTrobaPais          EXCEPTION; -- No s'ha trobat el pais nou indicat a i_pais
    exc_jaHiHaCambra         EXCEPTION; -- Ja existeix la cambra indicada a i_nom  
    exc_jaHiHaCambraPais     EXCEPTION; -- Ja existeix una altra cambra amb i_pais
    exc_jaHiHa2CambraEstat   EXCEPTION; -- Ja existeixen dos cambres amb i_estat
	
     -- Variables per a les dades actuals de la cambra a modificar
    ant_nom      VARCHAR2(50); -- Nom de la cambra 
    ant_adreca   VARCHAR2(40); -- Adrea postal de la cambra
    ant_ciutat   VARCHAR2(30); -- Ciutat de la cambra
    ant_telefon  VARCHAR2(14); -- Telfon de la cambra
    ant_llocWeb  VARCHAR2(50); -- Adrea URL del lloc web de la cambra
    ant_idEstat  NUMBER(3);    -- Identificador bbdd del estat membre
    ant_idPais   NUMBER(3);    -- Identificador bb del pais o regi
    
    -- Variables per a la modificaci de dades
    nou_nom      VARCHAR2(50); -- Canvi de nom de la cambra 
    nou_adreca   VARCHAR2(40); -- Canvi d'adrea postal de la cambra
    nou_ciutat   VARCHAR2(30); -- Canvi de ciutat de la cambra
    nou_telefon  VARCHAR2(14); -- Canvi de telfon de la cambra
    nou_llocWeb  VARCHAR2(50); -- Canvi d'adrea URL del lloc web de la cambra
    nou_idEstat  NUMBER(3);    -- Canvi de identificador bbdd del estat membre
    nou_idPais   NUMBER(3);    -- Canvi de identificador bbdd del pais o regi

    v_declaracio NUMBER(2);    -- Nmero declaraci sentncia SQL
    v_ibd        NUMBER(3);    -- Identificador bbdd
    v_count      NUMBER(3);    -- Comptador
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.CAMBRA_M';
    log_entrada  := '<' || i_ibd || '> <' || i_nom || '> <' || 
        i_adreca || '> <' || i_ciutat || '> <' || i_telefon || 
        '> <' || i_estat || '> <' || i_pais || '>';
    log_resultat := 'OK. MODIFICACI DE CAMBRA EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_ibd IS NULL OR i_ibd = '') THEN RAISE exc_noIbd; END IF;
    IF (i_nom IS NULL OR i_nom = '') AND 
       (i_adreca IS NULL OR i_adreca = '') AND 
       (i_ciutat IS NULL OR i_ciutat = '') AND
       (i_telefon IS NULL OR i_telefon = '') AND
       (i_llocWeb IS NULL OR i_llocWeb = '') AND
       (i_estat IS NULL OR i_estat = '' OR i_estat = 0) AND
       (i_pais IS NULL OR i_pais = '' OR i_pais = 0) 
    THEN RAISE exc_capModificacio; END IF;
    
    IF ((i_estat > 0) AND (i_pais > 0)) THEN RAISE exc_siEstatSiPais; END IF;
    
    -- Inicialitzaci de les variables de la modificaci de dades
    nou_nom      := ''; 
    nou_adreca   := '';
    nou_ciutat   := '';
    nou_telefon  := '';
    nou_llocWeb  := '';
    nou_idEstat  := 0;
    nou_idPais   := 0;
    v_declaracio := 0;
    v_count      := 0;
    
    nou_nom      := UPPER(i_nom);
    nou_adreca   := UPPER(i_adreca);
    nou_ciutat   := UPPER(i_ciutat);
    nou_telefon  := UPPER(i_telefon);
    nou_llocWeb  := LOWER(i_llocWeb);
    nou_idEstat  := i_estat;
    nou_idPais   := i_pais;
        
    -- Inicialitzaci de les variables de les dades actuals de la cambra a modificar
    ant_nom     := '';
    ant_adreca  := '';
    ant_ciutat  := '';
    ant_telefon := '';
    ant_llocWeb := '';
    ant_idEstat := 0;
    ant_idPais  := 0;
    v_ibd       := 0;
    
    -- Comprovaci de l'existncia de la cambra i obtenci dels valors actuals
    v_declaracio := 1;
    SELECT UPPER(cambra), UPPER(adreca), UPPER(ciutat), 
        UPPER(telefon), LOWER(lloc_web)
        INTO ant_nom, ant_adreca, ant_ciutat, ant_telefon, ant_llocWeb
        FROM dce.cambra WHERE ibd = i_ibd;
        
    -- Comprovaci de si pertany a un estat i obtenci del valor actual
    v_declaracio := 2;
    SELECT COUNT(*) INTO v_count FROM dce.cambra_estat WHERE cambra = i_ibd;
    IF (v_count = 0) THEN
        -- Comprovaci de si pertany a un pais o regi i obtenci del valor actual
        v_declaracio := 3;
        SELECT pais_regio INTO ant_idPais
            FROM dce.cambra_pais_regio WHERE cambra = i_ibd;
    ELSE
        SELECT estat INTO ant_idEstat FROM dce.cambra_estat WHERE cambra = i_ibd;
    END IF;

    log_abans := 'Cambra:<' || i_ibd || '> Nom:<' || ant_nom || '> Adrea:<' || 
        ant_adreca || '> Ciutat:<' || ant_ciutat || '> Telfon:<' || ant_telefon || 
        '> Estat membre:<' || ant_idEstat || '> Pais/Regi:<' || ant_idPais || '>';

     -- Si no s'han indicat valors nous es mantenen els actuals
    IF (nou_nom = '' OR nou_nom IS NULL) THEN nou_nom := ant_nom; END IF;
    IF (nou_adreca = '' OR nou_adreca IS NULL) THEN nou_adreca := ant_adreca; END IF;
    IF (nou_ciutat = '' OR nou_ciutat IS NULL) THEN nou_ciutat := ant_ciutat; END IF;
    IF (nou_telefon = '' OR nou_telefon IS NULL) THEN nou_telefon := ant_telefon; END IF;
    IF (nou_llocWeb = '' OR nou_llocWeb IS NULL) THEN nou_llocWeb := ant_llocWeb; END IF;
    IF (nou_idEstat = 0 OR nou_idEstat = '' OR nou_idEstat IS NULL) 
        THEN nou_idEstat := ant_idEstat; END IF;
    IF (nou_idPais = 0 OR nou_idPais = '' OR nou_idPais IS NULL) 
        THEN nou_idPais := ant_idPais; END IF;
    
    log_despres := 'Cambra:<' || i_ibd || '> Nom:<' || nou_nom || '> Adrea:<' ||
        nou_adreca || '> Ciutat:<' || nou_ciutat || '> Telfon:<' || nou_telefon || 
        '> Estat membre:<' || nou_idEstat || '> Pais/Regi:<' || nou_idPais || '>' ;       
    
    -- Comprovaci de la no existncia d'una cambra amb nom = i_nom
    IF (nou_nom <> ant_nom) THEN
      v_declaracio := 4;
      SELECT COUNT(*) INTO v_count FROM CAMBRA WHERE UPPER(CAMBRA) = nou_nom; 
      IF (v_count > 0) THEN RAISE exc_jaHiHaCambra; END IF;    
    END IF;
    
    -- Comprovaci de l'existencia del nou estat membre i de la no existncia de
    -- dues cambres per l'estat membre
    IF (nou_idEstat > 0 AND nou_idEstat <> ant_idEstat) THEN
        v_count := F_CAMBRA_ESTAT(nou_idEstat);
        IF v_count = -1 THEN RAISE exc_noTrobaEstat; END IF;
        IF v_count > 1 THEN RAISE exc_jaHiHa2CambraEstat; END IF;
    END IF;
    
    -- Comprovaci de l'existncia del nou pais o regi
    IF (nou_idPais > 0 AND nou_idPais <> ant_idPais) THEN
        v_declaracio := 7;
        SELECT ibd INTO v_ibd FROM dce.pais_regio 
            WHERE ibd = nou_idPais;
            
        -- Comprovaci de la no existncia d'una cambra amb pais <> i_pais
        v_declaracio := 8;
        SELECT COUNT(*) INTO v_count FROM dce.cambra_pais_regio
            WHERE pais_regio = nou_idPais AND cambra <> i_ibd;
        IF (v_count > 0) THEN RAISE exc_jaHiHaCambraPais; END IF;
            v_count := F_CAMBRA_PAIS_REGIO(i_pais);
	    IF v_count = -1 THEN RAISE exc_noTrobaPais; END IF;
        IF v_count > 0 THEN RAISE exc_jaHiHaCambraPais; END IF;

    END IF;

    -- Comprovem si la cambra pertany a un estat membre i si s'ha indicat
    -- un canvi d'estat membre, o b un canvi d'estat membre a pais o regi
    IF (ant_idEstat > 0) THEN
        IF (nou_idEstat > 0 AND nou_idEstat <> ant_idEstat) THEN  -- canvi d'estat membre

           -- Crida al procediment que fa la modificaci de la cambra d'estat membre
           DCE.ABM.CAMBRA_ESTAT_M(i_ibd, nou_idEstat, o_rsp);
        ELSE
           IF (nou_idPais > 0) THEN  -- canvi d'estat membre a pais o regi

               -- Crida al procediment que fa l'eliminaci de la cambra d'estat membre
               DCE.ABM.CAMBRA_ESTAT_B(i_ibd, o_rsp);

               -- Crida al procediment que fa l'alta de la cambra de pais o regi
               DCE.ABM.CAMBRA_PAIS_REGIO_A(i_ibd, nou_idPais, o_rsp);
           END IF;
        END IF; 
    ELSE
        -- Comprovem si la cambra pertany a un pais o regi i si s'ha indicat
        -- un canvi de pais o regi, o b un canvi de pais o regi a estat membre
        IF (nou_idPais > 0 AND nou_idPais <> ant_idPais) THEN  -- canvi de pais o regi

           --Crida al procediment que fa la modificaci de la cambra de pais o regi
           DCE.ABM.CAMBRA_PAIS_REGIO_M(i_ibd, nou_idPais, o_rsp);
        ELSE
           IF (nou_idEstat > 0) THEN  -- canvi de pais o regi a estat membre

               -- Crida al procediment que fa l'eliminaci de la cambra de pais o regi
               DCE.ABM.CAMBRA_PAIS_REGIO_B(i_ibd, o_rsp);               

               -- Crida al procediment que fa l'alta de la cambra d'estat membre
               DCE.ABM.CAMBRA_ESTAT_A(i_ibd, nou_idEstat, o_rsp);               
           END IF;
        END IF;
    END IF;
    
    -- Execuci: Modificaci de la cambra en la relaci pare   
    v_declaracio := 9;
    UPDATE DCE.CAMBRA  SET 
        CAMBRA = nou_nom, 
        ADRECA = nou_adreca,
        CIUTAT = nou_ciutat,
        TELEFON = nou_telefon,
        LLOC_WEB = nou_llocWeb
        WHERE IBD = i_ibd;
       
    -- Execuci correcte del procediment
    log_proces   := 'ABM.CAMBRA_M';
    log_entrada  := '<' || i_ibd || '> <' || i_nom || '> <' || 
        i_adreca || '> <' || i_ciutat || '> <' || i_telefon || 
        '> <' || i_estat || '> <' || i_pais || '>';
    log_resultat := 'OK. MODIFICACI DE CAMBRA EFECTUADA';
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;

    -- Control d'excepcions
    EXCEPTION 
    WHEN exc_noIbd THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_capModificacio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT CAP CAMP A MODIFICAR DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_siEstatSiPais THEN
        log_resultat := 
            'ERROR: S''HA INDICAT A LA VEGADA L'' ESTAT MEMBRE I EL PAS O REGI' ||
            ' DE LA CAMBRA PARLAMENTRIA. CAL INDICAR-NE UN DE SOL.';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaEstat THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT L''ESTAT MEMBRE. (IBD ' || nou_idEstat || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
                   
    WHEN exc_noTrobaPais THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT EL PAIS O REGI. (IBD ' || nou_idPais || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        CASE v_declaracio
            WHEN 1 THEN log_resultat := 
                   'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA. (IBD ' || i_ibd || ')';
            WHEN 3 THEN log_resultat := 
                   'ERROR: NO S''HA TROBAT NI l''ESTAT MEMBRE NI EL PAIS O REGI' ||
                   ' DE LA CAMBRA AMB IBD = ' || i_ibd;        
        END CASE;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHaCambra THEN
        log_resultat := 
            'ERROR: JA EXISTEIX UNA CAMBRA AMB AQUEST NOM: ' || i_nom;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHa2CambraEstat THEN
        log_resultat := 
            'ERROR: JA EXISTEIXEN DUES CAMBRES PER AQUEST ESTAT MEMBRE. (IBD ' || 
            nou_idEstat || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHaCambraPais THEN
        log_resultat := 
            'ERROR: JA EXISTEIX UNA CAMBRA PER AQUEST PAIS O REGI. (IBD ' || nou_idPais || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRA_M;

---------------------------------------------------------------------------------------------------
-- *** 02. C A M B R E S    P A R L A M E N T  R I E S   E S T A T S    M E MB R E S
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P02A.ALTA DE CAMBRA PARLAMENTRIA D'ESTAT MEMBRE
---------------------------------------------------------------------------------------------------
-- Insereix una nova cambra en la taula CAMBRA_ESTAT 
-- Cridat des dels procediments d'alta i modificaci de cambra parlamentria CAMBRA_A i CAMBRA_M
-- Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_ESTAT_A (
    i_cambra  IN  CAMBRA_ESTAT.CAMBRA%TYPE,  -- Codi identificador bbdd de la cambra
    i_estat   IN  CAMBRA_ESTAT.ESTAT%TYPE,   -- Codi dentificador bbdd del estat membre
    o_rsp     OUT VARCHAR2                   -- Resultat de l'execuci del procediment 
) IS    
    -- Variables pel tractament d'excepcions
    exc_noCambra      EXCEPTION;  -- No s'ha indicat el parmetre i_cambra
    exc_noEstat       EXCEPTION;  -- No s'ha indicat el parmetre i_estat
    exc_jaHiHaCambra  EXCEPTION;  -- Ja existeix la cambra indicada a i_cambra  
    
    v_declaracio  NUMBER(2);  -- Nmero declaraci sentncia SQL
    v_count       NUMBER(3);  -- Comptador
    v_idEstat     NUMBER(3);  -- Identificador bbdd del estat membre a inserir
     
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.CAMBRA_ESTAT_A';
    log_entrada  := '<' || i_cambra || '> <' || i_estat || '>';
    log_resultat := 'OK. ALTA DE CAMBRA D''ESTAT EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    v_declaracio := 0;
    v_count      := 0;
    v_idEstat    := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) THEN RAISE exc_noCambra; END IF;
    IF (i_estat IS NULL OR i_estat = '' OR i_estat = 0) THEN RAISE exc_noEstat; END IF;    
    
    -- Comprovaci de l'existencia del estat membre
    v_declaracio := 1;
    SELECT ibd INTO v_idEstat FROM dce.estat WHERE ibd = i_estat;            

    -- Comprovaci de l'existncia de la cambra parlamentria
    v_declaracio := 2;
    SELECT COUNT(*) INTO v_count FROM dce.cambra WHERE ibd = i_cambra;
    
    -- Comprovaci de l'existncia de la cambra d'estat
    SELECT COUNT(*) INTO v_count FROM dce.cambra_estat WHERE cambra = i_cambra;
    IF (v_count > 0) THEN RAISE exc_jaHiHaCambra; END IF;
   
    -- Execuci: Alta de la cambra d'estat
    INSERT INTO DCE.CAMBRA_ESTAT (CAMBRA, ESTAT) VALUES (i_cambra, i_estat);
       
    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;

    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noCambra THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noEstat THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DEL ESTAT MEMBRE';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_declaracio
    WHEN 1 THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT L''ESTAT MEMBRE. (IBD ' || i_estat || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    WHEN 2 THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT LA CAMBRA. (IBD ' || i_cambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);    
    END CASE;
    
    WHEN exc_jaHiHaCambra THEN
        log_resultat := 
            'ERROR: JA EXISTEIX AQUESTA CAMBRA D''ESTAT. (IBD ' || i_cambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN DUP_VAL_ON_INDEX THEN
        log_resultat := 
            'ERROR: VALOR DUPLICAT EN NDEX A CAMBRA_ESTAT. (CAMBRA-ESTAT ' ||
             i_cambra || '-' || i_estat || ')';        
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRA_ESTAT_A;

---------------------------------------------------------------------------------------------------
-- *** P02B.BAIXA DE CAMBRA PARLAMENTRIA D'ESTAT MEMBRE
---------------------------------------------------------------------------------------------------
-- Elimina una cambra en la taula CAMBRA_ESTAT 
-- Cridat des del procediment de modificaci de cambra parlamentria CAMBRA_M
-- Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_ESTAT_B (
    i_cambra  IN  CAMBRA_ESTAT.CAMBRA%TYPE,  -- Codi identificador bbdd de la cambra
    o_rsp     OUT VARCHAR2                   -- Resultat de l'execuci del procediment 
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra      EXCEPTION;  -- No s'ha indicat el parmetre i_cambra
    v_idCambra        NUMBER(3);  -- Codi identificador bbdd de la cambra a eliminar
  	    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.CAMBRA_ESTAT_B';
    log_entrada  := '<' || i_cambra || '>';
    log_resultat := 'OK. ELIMINACI DE CAMBRA D''ESTAT EFECTUADA';
    log_abans    := '';
    log_despres  := '';    
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := 'OK';
    v_idCambra   := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) THEN RAISE exc_noCambra; END IF;
    
    -- Comprovaci de l'existncia de la cambra 
    SELECT cambra INTO v_idCambra 
        FROM dce.cambra_estat WHERE cambra = i_cambra;
             
    -- Execuci: Baixa de la cambra
    DELETE FROM dce.cambra_estat WHERE cambra =i_cambra;
    
    -- Execuci correcte del procediment
    log_error := 'N';
    o_rsp := log_resultat;
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
        
    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noCambra THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA. (IBD ' || i_cambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRA_ESTAT_B;

---------------------------------------------------------------------------------------------------
-- *** P02M.MODIFICACI DE CAMBRA PARLAMENTRIA D'ESTAT MEMBRE
---------------------------------------------------------------------------------------------------
-- Modifica l'estat membre d'una cambra en la taula CAMBRA_ESTAT 
-- Cridat des del procediment de modificaci de cambra parlamentria CAMBRA_M
-- Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_ESTAT_M (
    i_cambra  IN  CAMBRA_ESTAT.CAMBRA%TYPE,  -- Codi identificador bbdd de la cambra
    i_estat   IN  CAMBRA_ESTAT.ESTAT%TYPE,   -- Codi identificador bbdd del estat membre
    o_rsp     OUT VARCHAR2                   -- Resultat de l'execuci del procediment 
) IS    
    -- Variables pel tractament d'excepcions
    exc_noCambra      EXCEPTION;  -- No s'ha indicat el parmetre i_cambra
    exc_noEstat       EXCEPTION;  -- No s'ha indicat el parmetre i_estat
    exc_jaHiHa2Cambra EXCEPTION;  -- Ja hi ha dues cambres per l'estat indicad a i_estat  

    v_count        NUMBER(3);  -- Comptador
    v_declaracio   NUMBER(2);  -- Nmero declaraci sentncia SQL  	
    v_idEstat      NUMBER(3);  -- Identificador bbdd del estat membre a modificar
    ant_Cambra     NUMBER(3);  -- Identificador bbdd de la cambra parlamentria
    ant_Estat      NUMBER(3);  -- Identificador bbdd del estat membre
     
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.CAMBRA_ESTAT_M';
    log_entrada  := '<' || i_cambra || '> <' || i_estat || '>';
    log_resultat := 'OK. MODIFICACI DE CAMBRA D''ESTAT EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    v_count      := 0;
    v_declaracio := 0;
    v_idEstat    := 0;
    ant_Cambra   := 0;
    ant_estat    := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) THEN RAISE exc_noCambra; END IF;
    IF (i_estat IS NULL OR i_estat = '' OR i_estat = 0) THEN RAISE exc_noEstat; END IF;    
    
    -- Comprovaci de l'existencia del estat membre
    v_declaracio := 1;
    SELECT ibd INTO v_idEstat FROM dce.estat WHERE ibd = i_estat;            

    -- Comprovaci de l'existncia de la cambra parlamentria d'estat
    v_declaracio := 2;
    SELECT cambra, estat INTO ant_cambra, ant_estat 
       FROM dce.cambra_estat WHERE cambra = i_cambra;

    log_abans   := 'Cambra:<' || ant_cambra || '> Estat membre:<' || ant_estat || '>';
    log_despres := 'Cambra:<' || i_cambra   || '> Estat membre:<' || i_estat   || '>';
    
    -- Comprovaci de l'existncia de dues cambres parlamentries per a l'estat indicat
    SELECT COUNT(*) INTO v_count FROM dce.cambra_estat WHERE estat = i_estat;
    IF (v_count > 1) THEN RAISE exc_jaHiHa2Cambra; END IF;
   
    -- Execuci: Modificaci de la cambra d'estat
    UPDATE DCE.CAMBRA_ESTAT SET ESTAT = i_estat WHERE CAMBRA = i_cambra;
       
    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;

    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noCambra THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noEstat THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DEL ESTAT MEMBRE';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        CASE v_declaracio
            WHEN 1 THEN log_resultat := 
                'ERROR: NO S''HA TROBAT L''ESTAT MEMBRE. (IBD ' || i_estat || ')';
            WHEN 2 THEN log_resultat := 
                'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA. (IBD ' || i_cambra || ')';
        END CASE;         
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHa2Cambra THEN
        log_resultat := 
            'ERROR: JA EXISTEIXEN DUES CAMBRES PER AQUEST ESTAT. (IBD ' || i_estat || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN DUP_VAL_ON_INDEX THEN
        log_resultat := 
            'ERROR: VALOR DUPLICAT EN NDEX A CAMBRA_ESTAT. (CAMBRA-ESTAT ' ||
             i_cambra || '-' || i_estat || ')';        
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRA_ESTAT_M;

---------------------------------------------------------------------------------------------------
-- *** 03. C A M B R E S    P A R L A M E N T  R I E S   P A  S O S   O   R E G I O N S
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P03A.ALTA DE CAMBRA PARLAMENTRIA DE PAIS O REGI
---------------------------------------------------------------------------------------------------
-- Insereix una nova cambra en la taula CAMBRA_PAIS_REGIO
-- Cridat des dels procediments d'alta i modificaci de cambra parlamentria CAMBRA_A i CAMBRA_M
-- Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_PAIS_REGIO_A (
    i_cambra  IN  CAMBRA_PAIS_REGIO.CAMBRA%TYPE,     -- Codi dentificador bbdd de la cambra
    i_pais    IN  CAMBRA_PAIS_REGIO.PAIS_REGIO%TYPE, -- Codi identificador bbdd del pais o regi
    o_rsp     OUT VARCHAR2                           -- Resultat de l'execuci del procediment 
) IS    
    -- Variables pel tractament d'excepcions
    exc_noCambra      EXCEPTION; -- No s'ha indicat el parmetre i_cambra
    exc_noPais        EXCEPTION; -- No s'ha indicat el parmetre i_pais
    exc_jaHiHaCambra  EXCEPTION; -- Ja existeix la cambra indicada a i_cambra  
    
    v_count    NUMBER(3);  -- Comptador
    v_idPais   NUMBER(3);  -- Identificador bbdd del pais o regi a inserir
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.CAMBRA_PAIS_REGIO_A';
    log_entrada  := '<' || i_cambra || '> <' || i_pais || '>';
    log_resultat := 'OK. ALTA DE CAMBRA DE PAIS O REGI EFECTUADA';
    log_abans    := '';
    log_despres  := '';    
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    v_count      := 0;
    v_idPais     := 0;
      
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) THEN RAISE exc_noCambra; END IF;
    IF (i_pais IS NULL OR i_pais = '' OR i_pais = 0) THEN RAISE exc_noPais; END IF;    
     
    -- Comprovaci de l'existencia del pais o regi
    SELECT IBD INTO v_idPais FROM PAIS_REGIO WHERE IBD = i_pais;            

    -- Comprovaci de l'existncia de la cambra de pais o regi
    SELECT COUNT(*) INTO v_count FROM dce.cambra_pais_regio WHERE cambra = i_cambra;
    IF (v_count > 0) THEN RAISE exc_jaHiHaCambra; END IF;
   
    -- Execuci: Alta de la cambra de pais o regi
    INSERT INTO DCE.CAMBRA_PAIS_REGIO (CAMBRA, PAIS_REGIO) VALUES (i_cambra, i_pais);
       
    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;

    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noCambra THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noPais THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DEL PAIS O REGI';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT EL PAIS O REGI. (IBD ' || i_pais || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHaCambra THEN
        log_resultat := 
            'ERROR: JA EXISTEIX UNA CAMBRA DE PAIS O REGI AMB AQUEST IBD (' || i_cambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN DUP_VAL_ON_INDEX THEN
        log_resultat := 
            'ERROR: VALOR DUPLICAT EN NDEX A CAMBRA_PAIS_REGIO. (CAMBRA-PAIS ' ||
             i_cambra || '-' || i_pais || ')';        
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRA_PAIS_REGIO_A;

---------------------------------------------------------------------------------------------------
-- *** P03B.BAIXA DE CAMBRA PARLAMENTRIA DE PAIS O REGI
---------------------------------------------------------------------------------------------------
-- Elimina una cambra en la taula CAMBRA_PAIS_REGIO
-- Cridat des del procediment de modificaci de cambra parlamentria CAMBRA_M
-- Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_PAIS_REGIO_B (
    i_cambra  IN  CAMBRA_PAIS_REGIO.CAMBRA%TYPE,     -- Codi identificador bbdd de la cambra
    o_rsp     OUT VARCHAR2                           -- Resultat de l'execuci del procediment 
) IS
    -- Variables pel tractament d'excepcions
    exc_noCambra      EXCEPTION;  -- No s'ha indicat el parmetre i_cambra
    exc_noTrobaCambra EXCEPTION;  -- No s'ha trobat la cambra indicada a i_cambra

    v_idCambra  NUMBER(3);  -- Identificador bbdd de la cambra a eliminar
  	    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.CAMBRA_PAIS_REGIO_B';
    log_entrada  := '<' || i_cambra || '>';
    log_resultat := 'OK. ELIMINACI DE CAMBRA DE PAIS O REGI EFECTUADA';    
    log_abans    := '';
    log_despres  := '';    
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := 'OK';
    v_idCambra   := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) THEN RAISE exc_noCambra; END IF;
    
    -- Comprovaci de l'existncia de la cambra 
    SELECT cambra INTO v_idCambra 
        FROM dce.cambra_pais_regio WHERE cambra = i_cambra;
             
    -- Execuci: Baixa de la cambra
    DELETE FROM dce.cambra_pais_regio WHERE cambra =i_cambra;
    
    -- Execuci correcte del procediment
    log_error := 'N';
    o_rsp := log_resultat;
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
        
    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noCambra THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA. (IBD ' || i_cambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRA_PAIS_REGIO_B;

---------------------------------------------------------------------------------------------------
-- *** P03M.MODIFICACI DE CAMBRA PARLAMENTRIA DE PAIS O REGI
---------------------------------------------------------------------------------------------------
-- Modifica el pais o regi d'una cambra en la taula CAMBRA_PAIS_REGIO
-- Cridat des del procediment de modificaci de cambra parlamentria CAMBRA_M
-- Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
---------------------------------------------------------------------------------------------------
PROCEDURE CAMBRA_PAIS_REGIO_M (
    i_cambra  IN  CAMBRA_PAIS_REGIO.CAMBRA%TYPE,     -- Codi identificador bbdd de la cambra
    i_pais    IN  CAMBRA_PAIS_REGIO.PAIS_REGIO%TYPE, -- Codi identificador bbdd del pais o regi
    o_rsp     OUT VARCHAR2                           -- Resultat de l'execuci del procediment 
) IS    
    -- Variables pel tractament d'excepcions
    exc_noCambra      EXCEPTION;  -- No s'ha indicat el parmetre i_cambra
    exc_noPais        EXCEPTION;  -- No s'ha indicat el parmetre i_pais
    exc_jaHiHa1Cambra EXCEPTION;  -- Ja hi ha una cambra pel pais o regi indicat a i_pais  

    v_count        NUMBER(3);  -- Comptador
    v_declaracio   NUMBER(2);  -- Nmero declaraci sentncia SQL  	
    v_idPais       NUMBER(3);  -- Identificador bbdd del pais o regi a modificar
    ant_cambra     NUMBER(3);  -- Identificador bbdd de la cambra parlamentria
    ant_pais       NUMBER(3);  -- Identificador bbdd del pais o regi
     
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.CAMBRA_PAIS_REGIO_M';
    log_entrada  := '<' || i_cambra || '> <' || i_pais || '>';
    log_resultat := 'OK. MODIFICACI DE CAMBRA DE PAIS O REGI EFECTUADA';
    log_abans    := '';
    log_despres  := '';    
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    v_count      := 0;
    v_declaracio := 0;
    v_idPais     := 0;
    ant_cambra   := 0;
    ant_pais     := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) THEN RAISE exc_noCambra; END IF;
    IF (i_pais IS NULL OR i_pais = '' OR i_pais = 0) THEN RAISE exc_noPais; END IF;    
    
    -- Comprovaci de l'existencia del pais o regi
    v_declaracio := 1;
    SELECT ibd INTO v_idPais FROM dce.pais_regio WHERE ibd = i_pais;            

    -- Comprovaci de l'existncia de la cambra parlamentria de pais o regi
    v_declaracio := 2;
    SELECT cambra, pais_regio INTO ant_cambra, ant_pais 
        FROM dce.cambra_pais_regio WHERE cambra = i_cambra;

    log_abans   := 'Cambra:<' || ant_cambra || '> Pais/Regi:<' || ant_pais || '>';
    log_despres := 'Cambra:<' || i_cambra   || '> Pais/Regi:<' || i_pais   || '>';

    -- Comprovaci de l'existncia de cambra parlamentria pel pais o regi indicat
    SELECT COUNT(*) INTO v_count FROM dce.cambra_pais_regio WHERE pais_regio = i_pais;
    IF (v_count > 0) THEN RAISE exc_jaHiHa1Cambra; END IF;
   
    -- Execuci: Modificaci de la cambra de pais o regi
    UPDATE DCE.CAMBRA_PAIS_REGIO SET PAIS_REGIO = i_pais WHERE CAMBRA = i_cambra;
       
    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;

    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noCambra THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noPais THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DEL PAIS O REGI';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        CASE v_declaracio
            WHEN 1 THEN log_resultat := 
                'ERROR: NO S''HA TROBAT EL PAIS O REGI. (IBD ' || i_pais || ')';
            WHEN 2 THEN log_resultat := 
                'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA. (IBD ' || i_cambra || ')';
        END CASE;         
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHa1Cambra THEN
        log_resultat := 
            'ERROR: JA EXISTEIXEN UNA CAMBRA PER AL PAIS O REGI. (IBD ' || i_pais || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN DUP_VAL_ON_INDEX THEN
        log_resultat := 
            'ERROR: VALOR DUPLICAT EN NDEX A CAMBRA_PAIS_REGIO. (CAMBRA-PAIS ' ||
             i_cambra || '-' || i_pais || ')'; 
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END CAMBRA_PAIS_REGIO_M;


---------------------------------------------------------------------------------------------------
-- *** 04. M E M B R E  S    D E    C A M B R E S    P A R L A M E N T  R I E S 
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P04A.ALTA DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Insereix un nou membre d'una cambra parlamentria en la taula MEMBRE_CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra i obtenci del nmero actual de membres
   - Regla de negoci 6: La no existncia d'el nmero identificador de membre dins la cambra indicada
   - L'existncia del partit poltic
   - Regla de negoci 7: Incrementa el comptador de nmero de membres de la cambra parlamentria 
     de la que formar part.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE MEMBRE_A (
    i_codi    IN  MEMBRE_CAMBRA.CODI%TYPE,     -- Codi identificador (NIF, etc.)
    i_nom     IN  MEMBRE_CAMBRA.NOM%TYPE,      -- Nom del membre de la cambra
    i_cognoms IN  MEMBRE_CAMBRA.COGNOMS%TYPE,  -- Cognoms del membre de la cambra
    i_telefon IN  MEMBRE_CAMBRA.TELEFON%TYPE,  -- Telfon del membre de la cambra
    i_email   IN  MEMBRE_CAMBRA.EMAIL%TYPE,    -- Adrea elctrnica del membre de la cambra
    i_partit  IN  MEMBRE_CAMBRA.PARTIT%TYPE,   -- Codi identificador bbdd del partit al que pertany
    i_cambra  IN  MEMBRE_CAMBRA.CAMBRA%TYPE,   -- Codi identif. bbdd de la cambra a la que pertany
    i_numero  IN  MEMBRE_CAMBRA.NUMERO%TYPE,   -- Nmero identificador dins de la cambra
    o_rsp     OUT VARCHAR2                     -- Resultat de l'execuci del procediment 
) IS
    -- Variables pel tractament d'excepcions
    exc_noCodi              EXCEPTION; -- No s'ha indicat el parmetre i_codi
    exc_noNom               EXCEPTION; -- No s'ha indicat el parmetre i_nom
    exc_noCognoms           EXCEPTION; -- No s'ha indicat el parmetre i_cogmnos
    exc_noTelefon           EXCEPTION; -- No s'ha indicat el parmetre i_telefon
    exc_noEmail             EXCEPTION; -- No s'ha indicat el parmetre i_email
    exc_noPartit            EXCEPTION; -- No s'ha indicat ni i_estat ni i_partit
    exc_noCambra            EXCEPTION; -- S'ha indicat tant i_estat com i_cambra
    exc_noNumero            EXCEPTION; -- No s'ha trobat l'estat membre i_numero
    exc_noTrobaPartit       EXCEPTION; -- No s'ha trobat el partit poltic  
    exc_jaHiHaMembreCambra  EXCEPTION; -- Ja existeix un membre per aquests cambra i nmero
     
    -- Variables per a la inserci de dades
    v_codi       VARCHAR2(12); -- Codi identficatiu del membre de la cambra a inserir
    v_nom        VARCHAR2(20); -- Nom del membre de la cambra a inserir
    v_cognoms    VARCHAR2(50); -- Cognoms del membre de la cambra a inserir
    v_telefon    VARCHAR2(20); -- Telfon del membre de la cambra a inserir
    v_email      VARCHAR2(50); -- Adrea electrnica del membre de la cambra a inserir
    v_numero     NUMBER(3);    -- Nmero de membre dins la cambra
  	
    v_count      NUMBER(3);    -- Comptador
    v_cambra     VARCHAR2(50); -- Nom d la cambra parlamentria

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.MEMBRE_A';
    log_entrada  := '<' || i_codi || '> <' || i_nom || '> <' || i_cognoms
        || '> <' || i_telefon || '> <' || i_email || '> <' || i_partit 
        || '> <' || i_cambra || '> <' || i_numero ||'>';
    log_resultat := 'OK. ALTA DE NOU MEMBRE DE CAMBRA EFECTUADA';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
       
    -- Comprovaci dels parmetres d'entrada
    IF (i_codi IS NULL OR i_codi = '') THEN RAISE exc_noCodi; END IF;
    IF (i_nom IS NULL OR i_nom = '') THEN RAISE exc_noNom; END IF;
    IF (i_cognoms IS NULL OR i_cognoms = '') THEN RAISE exc_noCognoms; END IF;
    IF (i_telefon IS NULL OR i_telefon = '') THEN RAISE exc_noTelefon; END IF;
    IF (i_email IS NULL OR i_email = '') THEN RAISE exc_noEmail; END IF;
    IF (i_partit IS NULL OR i_partit = '' OR i_partit = 0) THEN RAISE exc_noPartit; END IF;
    IF (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) THEN RAISE exc_noCambra; END IF;
    IF (i_numero IS NULL OR i_numero ='' OR i_numero = 0) THEN RAISE exc_noNumero; END IF;
    
    -- Inicialitzaci de les variables de la inserci de dades
    v_codi       := UPPER(i_codi);
    v_nom        := UPPER(i_nom);
    v_cognoms    := UPPER(i_cognoms);
    v_telefon    := UPPER(i_telefon);
    v_email      := LOWER(i_email);
    v_numero     := 0;

    v_count      := 0;
    v_cambra     := '';
    
    -- Comprovaci de l'existncia de la cambra i obtenci del nmero actual de membres
    SELECT cambra, num_membres 
        INTO v_cambra, v_numero FROM dce.cambra WHERE ibd = i_cambra;

    -- Comprovaci de la no existncia d'el nmero identificador de membre dins la cambra 
    SELECT COUNT(*) INTO v_count FROM dce.membre_cambra 
        WHERE cambra = i_cambra AND numero = i_numero;
    IF (v_count > 0) THEN RAISE exc_jaHiHaMembreCambra; END IF;

    -- Comprovaci de l'existncia del partit poltic
    SELECT COUNT(*) INTO v_count FROM dce.partit_politic WHERE ibd = i_partit;
    IF (v_count = 0) THEN RAISE exc_noTrobaPartit; END IF;
 
    -- Execuci: Alta del membre de la cambra
    INSERT INTO DCE.MEMBRE_CAMBRA (CODI, NOM, COGNOMS, TELEFON, EMAIL, PARTIT, CAMBRA, NUMERO)
        VALUES (v_codi, v_nom, v_cognoms, v_telefon, v_email, i_partit, i_cambra, i_numero);

    -- Incrementa en 1 el comptador de membres de la cambra i_cambra
    UPDATE DCE.CAMBRA SET NUM_MEMBRES = (v_numero + 1) WHERE IBD = i_cambra;
    
    log_abans   := 'El nmero de membres de la cambra ' || i_cambra || ' ' || v_cambra || 
        ' era ' || v_numero;
    log_despres := 'El nmero de membres de la cambra ' || i_cambra || ' ' || v_cambra || 
        ' s ' || (v_numero + 1);
        
    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
                
    -- Control d'excepcions
    EXCEPTION   
    WHEN exc_noCodi THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL CODI IDENTIFICADOR DEL/DE LA MEMBRE DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noNom THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL NOM DEL/DE LA MEMBRE DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noCognoms THEN
        log_resultat := 
            'ERROR: NO S''HAN INDICAT ELS COGNOMS DEL/DE LA MEMBRE DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTelefon THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL TELFON DEL/DE LA MEMBRE DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noEmail THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT L''ADREA ELECTRNICA DEL/DE LA MEMBRE DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noNumero THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL NMERO DE MEMBRE DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noPartit THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DEL PARTIT POLTIC';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noCambra THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaPartit THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT EL PARTIT POLTIC. (IBD ' || i_partit || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
                   
    WHEN NO_DATA_FOUND THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA. (IBD ' || i_cambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHaMembreCambra THEN
        log_resultat := 
            'ERROR: JA EXISTEIX UN/A MEMBRE EN LA CAMBRA PARLAMENTRIA ' || i_cambra || 
            ' AMB EL NMERO ' || i_numero;
        o_rsp := log_resultat;
         PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END MEMBRE_A;

---------------------------------------------------------------------------------------------------
-- *** P04B.BAIXA/ALTA DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Fa una baixa o alta lgica d'un membre, informant una data, en la taula MEMBRE_CAMBRA
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que existeixi el membre de la cambra.
   - Regla de negoci 5: Baixa/alta lgica d'un membre d'una cambra parlamentria en la taula 
     MEMBRE_CAMBRA mitjanant una data de baixa que deixa el membre de la cambra inactiu/actiu.
   - Regla de negoci 5 i 7: Si es passa una data s'actualitza la data de baixa, el membre de la 
     cambra queda inactiu, i es decrementa el comptador de nmero de membres de la cambra 
     parlamentria a la que pertany. Si no es passa cap data, si el membre est de baixa, es treu
     la data de baixa, el membre torna a estar actiu, i s'incrementa el comptador de nmero de 
     membres de la cambra parlamentria a la que pertany.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE MEMBRE_B (
    i_ibd   IN  MEMBRE_CAMBRA.IBD%TYPE, -- Codi identificador bbdd del membre de la cambra
    i_data  IN  VARCHAR2,               -- Data de la baixa, cadena en format 'DD/MM/YYYY' 
                                        -- '' o null per tornar a donar d'alta.
    o_rsp   OUT VARCHAR2                -- Resultat de l'execuci del procediment
) IS

    -- Variables pel tractament d'excepcions
    exc_noMembre       EXCEPTION;  -- No s'ha indicat el parmetre i_membre
    exc_noTrobaCambra  EXCEPTION;  -- No s'ha trobat la cambra parlamentria
    exc_jaEsAlta       EXCEPTION;  -- El membe de la cambra ja est d'alta
    
    -- Variables per a la inserci i la comprovaci de la baixa
    v_dataIns       DATE;          -- Data de la baixa en format de data
    v_dataActual    VARCHAR2(10);  -- Valor de la data de baixa actual del membre de la cambra

    v_count         NUMBER(3);     -- Comptador
    v_idCambra      NUMBER(3);     -- Identificador bbdd de cambra parlamentria

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces    := 'ABM.MEMBRE_B'; 
    log_entrada   := '<' || i_ibd || '> <' || i_data || '>';
    log_abans     := '';
    log_despres   := '';
    log_error     := 'S';
    log_usuari    := USER;
    log_data      := SYSDATE;
    o_rsp         := 'OK';

    -- Inicialitzaci de les variables per a la inserci i la comprovaci de la baixa
    v_dataIns     := null;
    v_dataActual  := '';
    v_count       := 0;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_ibd IS NULL OR i_ibd = '') THEN RAISE exc_noMembre; END IF;
    
    -- Comprovaci de l'existncia del membre de la cambra parlamentria
    SELECT TO_CHAR(data_baixa,'dd/mm/yyyy'), cambra INTO v_dataActual, v_idCambra
        FROM dce.membre_cambra WHERE ibd = i_ibd;
    
	-- Comprovaci de la cambra parlamentria
	SELECT COUNT(*) INTO v_count FROM dce.cambra WHERE ibd = v_idCambra;
	IF(v_count = 0) THEN RAISE exc_noTrobaCambra; END IF;
	
    -- Si es vol tornar a deixar d'alta
    IF (i_data IS NULL OR i_data = '') THEN   
        IF (v_dataActual IS NULL OR v_dataActual = '') THEN
            -- Ja es troba en situaci d'alta
            RAISE exc_jaEsAlta;
        ELSE
            -- Incrementa en 1 el nmero de membres de la cambra a la que pertany
            UPDATE DCE.CAMBRA SET NUM_MEMBRES = (NUM_MEMBRES + 1) WHERE IBD = v_idCambra;
            log_resultat := 'OK. MEMBRE DE CAMBRA EN SITUACI D''ALTA';
        END IF;
        v_dataIns := null;      
         
    -- Si es vol deixar de baixa
    ELSE 
        IF (v_dataActual IS NOT NULL) THEN 

            -- Ja es troba en situaci de baixa
            log_resultat := 'OK. PER EL/LA MEMBRE DE CAMBRA JA EST EN SITUACI DE BAIXA';
            log_abans    := 'Data de baixa anterior: ' || v_dataActual;
            log_despres  := 'Nova data de baixa: ' || i_data;
        ELSE

            -- Decrementa en 1 el nmero de membres de la cambra a la que pertany
            UPDATE DCE.CAMBRA SET NUM_MEMBRES = (NUM_MEMBRES - 1) WHERE IBD = v_idCambra;
            log_resultat := 'OK. MEMBRE DE CAMBRA EN SITUACI DE BAIXA';
        END IF;   
        v_dataIns := TO_DATE(i_data);
    END IF;
             
    -- Execuci: Baixa o Alta del membre de la cambra
    UPDATE DCE.MEMBRE_CAMBRA SET DATA_BAIXA = v_dataIns WHERE IBD = i_ibd;
    
    -- Execuci correcte del procediment
    log_error := 'N';
    o_rsp := log_resultat;
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;
        
    -- Control d'excepcions
    EXCEPTION
    WHEN exc_noMembre THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR DEL/DE LA MEMBRE DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT EL/LA MEMBRE DE CAMBRA. (IBD  ' || i_ibd || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaEsAlta THEN 
        log_resultat := 'ERROR: JA ES TROBA D''ALTA EL/LA MEMBRE. (IBD ' || i_ibd || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaCambra THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA.(IBD ' 
            || v_idCambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
                   
    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
        
END MEMBRE_B;

---------------------------------------------------------------------------------------------------
-- *** P04M.MODIFICACI DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Modifica les dades d'un membre d'una cambra palamentria en la taula MEMBRE_CAMBRA.
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd del membre de la cambra i al menys un parmetre a modificar. 
   - Regle de negoci 7: Si es canvia la cambra parlamentria actualitza els comptadors de nmero 
     de membres de l'antiga cambra i de la nova.
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE MEMBRE_M (
    i_ibd     IN  MEMBRE_CAMBRA.IBD%TYPE,      -- Codi identificador bbdd del membre de la cambra
    i_codi    IN  MEMBRE_CAMBRA.CODI%TYPE,     -- Codi identificador (NIF, etc.)
    i_nom     IN  MEMBRE_CAMBRA.NOM%TYPE,      -- Nom del membre de la cambra
    i_cognoms IN  MEMBRE_CAMBRA.COGNOMS%TYPE,  -- Cognoms del membre de la cambra
    i_telefon IN  MEMBRE_CAMBRA.TELEFON%TYPE,  -- Telfon del membre de la cambra
    i_email   IN  MEMBRE_CAMBRA.EMAIL%TYPE,    -- Adrea elctrnica del membre de la cambra
    i_partit  IN  MEMBRE_CAMBRA.PARTIT%TYPE,   -- Codi identif. bbdd del partit al que pertany
    i_cambra  IN  MEMBRE_CAMBRA.CAMBRA%TYPE,   -- Codi identif. bbdd de la cambra a la que pertany
    i_numero  IN  MEMBRE_CAMBRA.NUMERO%TYPE,   -- Nmero identificador dins de la cambra
    o_rsp     OUT VARCHAR2                     -- Resultat de l'execuci del procediment 
) IS
    -- Variables pel tractament d'excepcions
    exc_noIbd              EXCEPTION; -- No s'ha indicat el parmetre i_ibd
    exc_capModificacio     EXCEPTION; -- No s'ha indicat cap parmetre a modificar
    exc_noTrobaPartit      EXCEPTION; -- No s'ha trobat el partit poltic indicat a i_partit
    exc_noTrobaCambra      EXCEPTION; -- No s'ha trobat la cambra indicaada a i_cambra
    exc_jaHiHaCambraNum    EXCEPTION; -- Ja existeix un/a membre amb aquest nmero a i_cambra
 	
     -- Variables per a les dades actuals del/de la membre de cambra a modificar
    ant_codi      VARCHAR2(12); -- Codi identificador (NIF, etc) del membre de la cambra
    ant_cognoms   VARCHAR2(50); -- Cognoms del membre de la cambra
    ant_nom       VARCHAR2(20); -- Nom del membre dela cabra
    ant_telefon   VARCHAR2(14); -- Telfon de la cambra
    ant_email     VARCHAR2(50); -- Adrea e-mail del membre de la cambra
    ant_idPartit  NUMBER(3);    -- Identificador bbdd del partit poltic
    ant_idCambra  NUMBER(3);    -- Identificador bbdd de la cambra parlamentria
    ant_numero    NUMBER(3);    -- Nmero de membre dins la cambra
    
    -- Variables per a la modificaci de dades
    nou_codi      VARCHAR2(12); -- Canvi de codi identificador del/de la membre de la cambra 
    nou_cognoms   VARCHAR2(50); -- Canvi de cognoms del/de la membre de la  cambra
    nou_nom       VARCHAR2(20); -- Canvi de nom del membre de la cambra
    nou_telefon   VARCHAR2(14); -- Canvi de telfon del/de la membre de la cambra
    nou_email     VARCHAR2(50); -- Canvi d'adrea electrnica del/de la membre de la cambra
    nou_idPartit  NUMBER(3);    -- Canvi de identificador bbdd del partit politic
    nou_idCambra  NUMBER(3);    -- Canvi de identificador bbdd de la cambra
    nou_numero    NUMBER(3);    -- Canvi de nmero de membre dins la cambra
    
    v_ibd         NUMBER(3);    -- Identificador bbdd
    v_count       NUMBER(3);    -- Comptador
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.MEMBRE_M';
    log_entrada  := '<' || i_ibd || '> <' || i_cognoms || '> <' || i_nom || '> <' || 
        i_telefon || '> <' || i_email || '> <' || i_partit || '> <' || 
        i_cambra || '> <' || i_numero || '>';
    log_resultat := 'OK. MODIFICACI DE MEBRE DE CAMBRA EFECTUAT';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_ibd IS NULL OR i_ibd = '') THEN RAISE exc_noIbd; END IF;
    IF (i_codi IS NULL OR i_codi = '') AND
       (i_cognoms IS NULL OR i_cognoms = '') AND 
       (i_nom IS NULL OR i_nom = '') AND 
       (i_telefon IS NULL OR i_telefon = '') AND
       (i_email IS NULL OR i_email = '') AND
       (i_partit IS NULL OR i_partit = '' OR i_partit = 0) AND
       (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) AND
       (i_numero IS NULL OR i_numero = '' OR i_numero = 0)
    THEN RAISE exc_capModificacio; END IF;
      
    -- Inicialitzaci de les variables de la modificaci de dades
    nou_codi     := ''; 
    nou_cognoms  := '';
    nou_nom      := '';
    nou_telefon  := '';
    nou_email    := '';
    nou_idPartit := '';
    nou_idCambra := 0;
    nou_numero   := 0;
    v_count      := 0;
    
    nou_codi     := UPPER(i_codi);        
    nou_cognoms  := UPPER(i_cognoms);
    nou_nom      := UPPER(i_nom);
    nou_telefon  := UPPER(i_telefon);
    nou_email    := LOWER(i_email);
    nou_idPartit := i_partit;
    nou_idCambra := i_cambra;
    nou_numero   := i_numero;
    
    -- Inicialitzaci de les variables de les dades actuals del membre a modificar
    ant_codi     := '';
    ant_cognoms  := '';
    ant_nom      := '';
    ant_telefon  := '';
    ant_email    := '';
    ant_idPartit := 0;
    ant_idCambra := 0;
    ant_numero   := 0;
    v_ibd        := 0;
    
    -- Comprovaci de l'existncia del membre i obtenci dels valors actuals;
    SELECT UPPER(codi), UPPER(cognoms), UPPER(nom), UPPER(telefon), LOWER(email),
        partit, cambra, numero
        INTO ant_codi, ant_cognoms, ant_nom, ant_telefon, ant_email, ant_idPartit,
             ant_idCambra, ant_numero
        FROM dce.membre_cambra WHERE ibd = i_ibd;
        
    log_abans := 'Membre:<' || i_ibd || '> Cognoms:<' || ant_cognoms || '> Nom:<' || 
        ant_nom || '> Telfon:<' || ant_telefon || '> e-Mail: <' || ant_email || 
        '> Partit poltic:<' || ant_idPartit || '> Cambra:<' || ant_idCambra || 
        '> Nmero membre:<' || ant_numero || '>';

     -- Si no s'han indicat valors nous es mantenen els actuals
    IF (nou_codi = '' OR nou_codi IS NULL) THEN nou_codi := ant_codi; END IF;
    IF (nou_cognoms = '' OR nou_cognoms IS NULL) THEN nou_cognoms := ant_cognoms; END IF;
    IF (nou_nom = '' OR nou_nom IS NULL) THEN nou_nom := ant_nom; END IF;
    IF (nou_telefon = '' OR nou_telefon IS NULL) THEN nou_telefon := ant_telefon; END IF;
    IF (nou_email = '' OR nou_email IS NULL) THEN nou_email := ant_email; END IF;
    IF (nou_idPartit = 0 OR nou_idPartit = '' OR nou_idPartit IS NULL) 
        THEN nou_idPartit := ant_idPartit; END IF;
    IF (nou_idCambra = 0 OR nou_idCambra = '' OR nou_idCambra IS NULL) 
        THEN nou_idCambra := ant_idCambra; END IF;
    IF (nou_numero = 0 OR nou_numero = '' OR nou_numero IS NULL) 
        THEN nou_numero := ant_numero; END IF;
    
    log_despres := 'Membre:<' || i_ibd || '> Cognoms:<' || nou_cognoms || '> Nom:<' || 
        nou_nom || '> Telfon:<' || nou_telefon || '> e-Mail: <' || nou_email || 
        '> Partit poltic:<' || nou_idPartit || '> Cambra:<' || nou_idCambra || 
        '> Nmero membre:<' || nou_numero || '>';
    
    -- Comprovaci de l'existncia del nou partit poltic
    IF (nou_idPartit <> ant_idPartit) THEN
        SELECT COUNT(*) INTO v_count FROM dce.partit_politic WHERE ibd = nou_idPartit;
        IF (v_count = 0) THEN RAISE exc_noTrobaPartit; END IF;
    END IF;
    
	-- Comprovaci de l'existncia de la nova cambra parlamentria
    IF (nou_idCambra <> ant_idCambra) THEN
	    SELECT COUNT(*) INTO v_count FROM dce.cambra WHERE ibd = nou_idCambra;
	    IF(v_count = 0) THEN RAISE exc_noTrobaCambra; END IF;
    END IF;
    
    -- Comprovaci de la no existncia d'un membre amb cambra i/o nmero nous indicats
    IF ((nou_idCambra <> ant_idCambra) OR (nou_numero <> ant_numero))THEN
      SELECT COUNT(*) INTO v_count FROM dce.membre_cambra 
          WHERE cambra = nou_idCambra AND numero = nou_numero AND ibd <> i_ibd; 
      IF (v_count > 0) THEN RAISE exc_jaHiHaCambraNum; END IF;    
    END IF;
    
    -- Execuci: Modificaci del membre de la cambra
    UPDATE DCE.MEMBRE_CAMBRA  SET 
        COGNOMS   = nou_cognoms,
        NOM       = nou_nom,
        TELEFON   = nou_telefon,
        EMAIL     = nou_email,
        PARTIT    = nou_idPartit,
        CAMBRA    = nou_idCambra,
        NUMERO    = nou_numero
        WHERE IBD = i_ibd;
       
    IF (nou_idCambra <> ant_idCambra) THEN
        -- Incrementa en 1 el nmero de membres de la nova cambra a la que pertany
        UPDATE DCE.CAMBRA SET NUM_MEMBRES = (NUM_MEMBRES + 1) WHERE IBD = nou_idCambra;
            
        -- Decrementa en 1 el nmero de membres de l'antiga cambra a la que pertanyia
        UPDATE DCE.CAMBRA SET NUM_MEMBRES = (NUM_MEMBRES - 1) WHERE IBD = ant_idCambra;
    END IF;
    
    -- Execuci correcte del procediment
    log_proces   := 'ABM.MEMBRE_M';
    log_entrada  := '<' || i_ibd || '> <' || i_cognoms || '> <' || i_nom || 
        i_telefon || '> <' || i_email || '> <' || i_partit || 
        '> <' || i_cambra || '> <' || i_numero || '>';
    log_resultat := 'OK. MODIFICACI DE MEMBRE DE CAMBRA EFECTUADA';
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;

    -- Control d'excepcions
    EXCEPTION 
    WHEN exc_noIbd THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR BBDD DEL/DE LA MEMBRE DE LA CAMBRA PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_capModificacio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT CAP CAMP A MODIFICAR DEL/DE LA MEMBRE DE LA CAMBRA ' ||
            'PARLAMENTRIA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaPartit THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT EL PARTIT POLTIC. (IBD = ' || i_partit || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
                   
    WHEN exc_noTrobaCambra THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA. (IBD ' ||i_cambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        log_resultat := 'ERROR: NO S''HA TROBAT EL/LA MEMBRE DE CAMBRA. (IBD ' || i_ibd || ')';        
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_jaHiHaCambraNum THEN
        log_resultat := 
            'ERROR: JA EXISTEIX QUEST NMERO DE MEMBRE PER A AQUESTA CAMBRA PARLAMENTRIA. ' || 
            '(IBD CAMBRA-NMERO ' || nou_idCambra || '-' || nou_numero || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END MEMBRE_M;

---------------------------------------------------------------------------------------------------
-- *** 05. D E S P E S E S    G E N E R A L S    D E    L E S    C A M B R E S 
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P05A.ALTA DE DESPESA GENERAL DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Insereix una nova despesa general d'una cambra parlamentria en les taules DESPESA i
   DESPESA_CAMBRA 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia de la cambra parlamentria
   - L'existncia de la categoria general
   - L'existncia de la forma de pagament
   - L'existncia de la situaci de despesa
   - Regla de negoci 2: Que el provedor estigui informat si la despesa > 30 euros.
   - Regla de negoci 8: El clcul del import del IVA i del import net es fa a nivell de valors 
     automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0
   - Regla de negoci 9: No es pot entrar despeses en cambres donades de baixa si la data de 
     despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula LOG_PROCEDIMENT.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_CAMBRA_A (
    i_data         IN  DESPESA.DATA_DESPESA%TYPE,     -- Data de la despesa
    i_cambra       IN  DESPESA_CAMBRA.CAMBRA%TYPE,    -- Codi identificador bbdd de la cambra
    i_categoria    IN  DESPESA_CAMBRA.CATEGORIA%TYPE, -- Categoria de la despesa
    i_import       IN  DESPESA.IMPORT_BRUT%TYPE,      -- Import brut de la despesa
    i_tipusIva     IN  DESPESA.TIPUS_IVA%TYPE,        -- Tipus d'IVA de la  despesa
    i_descripcio   IN  DESPESA.DESCRIPCIO%TYPE,       -- Descripci de la despesa
    i_justificacio IN  DESPESA.JUSTIFICACIO%TYPE,     -- Justificaci de la despesa
    i_pagament     IN  DESPESA.PAGAMENT%TYPE,         -- Forma de pagament de la despesa
    i_situacio     IN  DESPESA.SITUACIO%TYPE,         -- Situaci de la despesa
    i_proveidor    IN  DESPESA.PROVEIDOR%TYPE,        -- Provedor 
    i_escaneig     IN  DESPESA.URL_ESCANEIG%TYPE,     -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2                       -- Resultat de l'execuci del procediment 
) IS
    -- Variables pel tractament d'excepcions
    exc_noData              EXCEPTION; -- No s'ha indicat el parmetre i_data
    exc_noCambra            EXCEPTION; -- No s'ha indicat el parmetre i_cambra
    exc_dataSupBaixa        EXCEPTION; -- La data de baixa de la cambra s < que la de la despesa
    exc_noCategoria         EXCEPTION; -- No s'ha indicat el parmetre i_categoria
    exc_noImport            EXCEPTION; -- No s'ha indicat el parmetre i_import
    exc_noTipusIva          EXCEPTION; -- No s'ha indicat el parmetre i_tipusIva
    exc_noDescripcio        EXCEPTION; -- No s'ha indicat el parmetre i_descripcio
    exc_noJustificacio      EXCEPTION; -- No s'ha indicat ni i_estat ni i_justificacio
    exc_noPagament          EXCEPTION; -- No s'ha indicat el parmetre i_pagament
    exc_noSituacio          EXCEPTION; -- No s'ha indicat el parmetre i_situacio
    exc_noTrobaCategoria    EXCEPTION; -- No s'ha trobat la categoria
    exc_noTrobaPagament     EXCEPTION; -- No s'ha trobat la forma de pagament
    exc_noTrobaSituacio     EXCEPTION; -- No s'ha trobat la situaci
    exc_importMenorZero     EXCEPTION; -- El import no pot ser negatiu
    exc_tipusIvaMenorZero   EXCEPTION; -- El tipus d'IVA no pot ser negatiu
    exc_noProveidor         EXCEPTION; -- Cal indicar el proveidor si la despesa > 30
 
    -- Variables per a la inserci de dades
    v_data         DATE;          -- Data de la despesa en format dd/mm/yyyy
    v_descripcio   VARCHAR2(50);  -- Descripci de la despesa
    v_justificacio VARCHAR2(50);  -- Justificaci de la despesa
    v_escaneig     VARCHAR2(50);  -- URL amb la informaci escanejada de la despesa
    v_count        NUMBER(3);     -- Comptador
    v_dataBaixa    DATE;          -- Data de baixa de la cambra parlamentria
    v_idDespesa    NUMBER(9);     -- Identificador bbdd de la despesa
    v_any          NUMBER(4);     -- Any de la despesa pel registre en les estadstiques
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.DESPESA_CAMBRA_A';
    log_entrada  := '<' || i_data || '> <' || i_import || '> <' || i_tipusIva || '> <' || 
        i_descripcio || '> <' || i_justificacio || '> <' || i_pagament || '> <' || 
        i_proveidor || '> <' || i_escaneig ||'>';
    log_resultat := 'OK. ALTA DE DESPESA GENERAL DE CAMBRA EFECTUAT';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    v_dataBaixa  := null;

    -- Comprovaci dels parmetres d'entrada
    IF (i_data IS NULL OR i_data = '') THEN RAISE exc_noData; END IF;
    IF (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) THEN RAISE exc_noCambra; END IF;
    IF (i_categoria IS NULL OR i_categoria = '') THEN RAISE exc_noCategoria; END IF;
    IF (i_import IS NULL OR i_import = '' OR i_import = 0) THEN RAISE exc_noImport; END IF;
    IF (i_tipusIva IS NULL OR i_tipusIva = '' OR i_tipusIva = 0) THEN RAISE exc_noTipusIva; END IF;
    IF (i_descripcio IS NULL OR i_descripcio = '') THEN RAISE exc_noDescripcio; END IF;
    IF (i_justificacio IS NULL OR i_justificacio = '') THEN RAISE exc_noJustificacio; END IF;
    IF (i_pagament IS NULL OR i_pagament = '' OR i_pagament = 0) THEN RAISE exc_noPagament; END IF;
    IF (i_situacio IS NULL OR i_situacio = '' OR i_situacio = 0) THEN RAISE exc_noSituacio; END IF;
    
    -- Inicialitzaci de les variables de la inserci de dades
    v_data         := TO_DATE(i_data);
    v_descripcio   := UPPER(i_descripcio);
    v_justificacio := UPPER(i_justificacio);
    v_escaneig     := LOWER(i_escaneig);
    v_count        := 0;
    v_any          := TO_NUMBER(TO_CHAR(v_data,'YYYY'));

    -- Comprovacio de la cambra parlamentaria
    SELECT data_baixa INTO v_dataBaixa FROM dce.cambra WHERE ibd = i_cambra;
    IF(v_dataBaixa IS NOT NULL AND v_data > v_dataBaixa) THEN RAISE exc_dataSupBaixa; END IF;

    -- Comprovacio de la categoria
    SELECT COUNT(*) INTO v_count FROM dce.categoria_general WHERE ibd = i_categoria;
    IF(v_count = 0) THEN RAISE exc_noTrobaCategoria; END IF;

    -- Comprovacio de la forma de pagament
    SELECT COUNT(*) INTO v_count FROM dce.forma_pagament WHERE ibd = i_pagament;
    IF(v_count = 0) THEN RAISE exc_noTrobaPagament; END IF;

    -- Comprovacio de la situaci
    SELECT COUNT(*) INTO v_count FROM dce.situacio_despesa WHERE ibd = i_situacio;
    IF(v_count = 0) THEN RAISE exc_noTrobaSituacio; END IF;

    -- Comprovaci del import, del tipus d'IVA i del provedor si la despesa > 30
    IF (i_import < 0) THEN RAISE exc_importMenorZero; END IF;
    IF (i_tipusIva < 0) THEN RAISE exc_tipusIvaMenorZero; END IF;
    IF (i_import > 30 AND (i_proveidor IS NULL OR i_proveidor = '')) THEN 
        RAISE exc_noProveidor; END IF;

    -- Execuci: Alta de la despesa
    INSERT INTO DCE.DESPESA (DATA_DESPESA, IMPORT_BRUT, TIPUS_IVA, 
        DESCRIPCIO, JUSTIFICACIO, PAGAMENT, SITUACIO, PROVEIDOR, URL_ESCANEIG)
        VALUES (v_data, i_import, i_tipusIva, v_descripcio, 
                v_justificacio, i_pagament, i_situacio, i_proveidor,i_escaneig);

    -- Execuci: Alta de la despesa general amb el valor actual de la seqncia
    -- de despeses DES_SEQ_IBD.currval
    v_idDespesa := DES_SEQ_IBD.currval;

    INSERT INTO DCE.DESPESA_CAMBRA (DESPESA, CAMBRA, CATEGORIA)
        VALUES (v_idDespesa, i_cambra, i_categoria);  
         
    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;        
        
    -- Actualitzaci de les taules estadstiques E1, E3, i E8   
    DCE.EST.ESTADISTICA_01_INSERIR(i_cambra, o_rsp);
    DCE.EST.ESTADISTICA_03_INSERIR(v_any, o_rsp);
    DCE.EST.ESTADISTICA_08_INSERIR(v_any, o_rsp);
    
    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noCambra THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA CAMBRA PARLAMENTRIA DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noData THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA DATA DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noCategoria THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA CATEGORIA DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noImport THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IMPORT DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTipusIva THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL TIPUS D''IVA DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noDescripcio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA DESCRIPCI DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noJustificacio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA JUSTIFICACI DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noPagament THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA FORMA DE PAGAMENT DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noSituacio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA SITUACI DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT LA CAMBRA. (IBD ' || i_cambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_dataSupBaixa THEN
        log_resultat := 
            'ERROR: LA DATA DE LA DESPESA S SUPERIOR A LA DATA DE BAIXA DE LA CAMBRA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaCategoria THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA CATEGORIA GENERAL. (IBD ' || i_categoria || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);                  

    WHEN exc_noTrobaPagament THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA FORMA DE PAGAMENT. (IBD ' || i_pagament || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaSituacio THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA SITUACI. (IBD ' || i_situacio || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_importMenorZero THEN 
        log_resultat := 'ERROR: EL IMPORT NO POT SER NEGATIU. ' || i_import || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_tipusIvaMenorZero THEN 
        log_resultat := 'ERROR: EL TIPUS D''IVA NO POT SER NEGATIU. ' || i_tipusIva || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noProveidor THEN 
        log_resultat := 'ERROR: CAL INDICAR EL PROVEDOR SI LA DESPESA > 30 EUROS. ';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESA_CAMBRA_A;

---------------------------------------------------------------------------------------------------
-- *** P05M.MODIFICACI DE DESPESA GENERAL DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Modifica les dades d'una despesa general d'una cambra parlamentria en les taules DESPESA i
   DESPESA_CAMBRA 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd de la despesa i al menys un parmetre a modificar. 
   - L'existncia de la cambra parlamentria si s'indica
   - L'existncia de la categoria general, si s'indica
   - L'existncia de la forma de pagament, si s'indica
   - L'existncia de la situaci de despesa, si s'indica
   - Si s'indiquen els parmetres corresponents:
      - Regla de negoci 2: El clcul del import del IVA i del import net es fa a nivell de valors 
        automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0   
      - Regla de negoci 8: Que el provedor estigui informat si la despesa > 30 euros.
   - Regla de negoci 9: No es pot entrar despeses en cambres donades de baixa si la data de 
     despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_CAMBRA_M (
    i_ibd          IN  DESPESA.IBD%TYPE,              -- Codi identificador bbdd de la despesa
    i_data         IN  VARCHAR2,                      -- Data de la despesa en format dd/mm/yyyy
    i_cambra       IN  DESPESA_CAMBRA.CAMBRA%TYPE,    -- Codi identificador bbdd de la cambra
    i_categoria    IN  DESPESA_CAMBRA.CATEGORIA%TYPE, -- Categoria de la despesa
    i_import       IN  DESPESA.IMPORT_BRUT%TYPE,      -- Import brut de la despesa
    i_tipusIva     IN  DESPESA.TIPUS_IVA%TYPE,        -- Tipus d'IVA de la  despesa
    i_descripcio   IN  DESPESA.DESCRIPCIO%TYPE,       -- Descripci de la despesa
    i_justificacio IN  DESPESA.JUSTIFICACIO%TYPE,     -- Justificaci de la despesa
    i_pagament     IN  DESPESA.PAGAMENT%TYPE,         -- Forma de pagament de la despesa
    i_situacio     IN  DESPESA.SITUACIO%TYPE,         -- Situaci de la despesa
    i_proveidor    IN  DESPESA.PROVEIDOR%TYPE,        -- Provedor 
    i_escaneig     IN  DESPESA.URL_ESCANEIG%TYPE,     -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2                       -- Resultat de l'execuci del procediment 
) IS
   -- Variables pel tractament d'excepcions
    exc_noDespesa           EXCEPTION; -- No s'ha indicat el parmetre i_ibd
    exc_capModificacio      EXCEPTION; -- No s'ha indicat cap canvi
    exc_dataSupBaixa        EXCEPTION; -- La data de baixa de la cambra s < que la de la despesa
    exc_noTrobaCategoria    EXCEPTION; -- No s'ha trobat la categoria
    exc_noTrobaPagament     EXCEPTION; -- No s'ha trobat la forma de pagament
    exc_noTrobaSituacio     EXCEPTION; -- No s'ha trobat la situaci
    exc_importMenorZero     EXCEPTION; -- El import no pot ser negatiu
    exc_tipusIvaMenorZero   EXCEPTION; -- El tipus d'IVA no pot ser negatiu
    exc_noProveidor         EXCEPTION; -- Cal indicar el proveidor si la despesa > 30
 
    v_data       DATE;         -- Data de la despesa en format date
    v_dataBaixa  DATE;         -- Data de baixa de la cambra parlamentria
    v_any        NUMBER(4);    -- Any de la despesa
    
    -- Variables per a les dades actuals de la despesa a modificar
    ant_data         DATE;         -- Data de baixa de la despesa
    ant_any          NUMBER(4);    -- Any de la despesa
    ant_idCambra     NUMBER(3);    -- Identificador de la cambra parlamentria
    ant_idCategoria  NUMBER(3);    -- Identificador de la categoria
    ant_import       NUMBER(12,2); -- Import brut de la despesa
    ant_tipusIva     NUMBER(5,2);  -- Tipus d'IVA de la  despesa
    ant_descripcio   VARCHAR2(50); -- Descripci de la despesa
    ant_justificacio VARCHAR2(50); -- Justificaci de la despesa
    ant_idPagament   NUMBER(3);    -- Identificador bbdd de la forma de pagament
    ant_idSituacio   NUMBER(3);    -- Identificador bbdd de la situaci
    ant_proveidor    VARCHAR2(12); -- Codi proveidor
    ant_escaneig     VARCHAR2(50); -- URL amb la informaci escanejada      
    
    -- Variables per a la modificaci de dades
    nou_data         DATE;         -- Canvi data de baixa de la despesa
    nou_any          NUMBER(4);    -- Canvi d'any de la despesa
    nou_idCambra     NUMBER(3);    -- Canvi identificador de la cambra parlamentria
    nou_idCategoria  NUMBER(3);    -- Canvi identificador de la categoria
    nou_import       NUMBER(12,2); -- Canvi import brut de la despesa
    nou_tipusIva     NUMBER(5,2);  -- Canvi tipus d'IVA de la  despesa
    nou_descripcio   VARCHAR2(50); -- Canvi descripci de la despesa
    nou_justificacio VARCHAR2(50); -- Canvi justificaci de la despesa
    nou_idPagament   NUMBER(3);    -- Canvi identificador bbdd de la forma de pagament
    nou_idSituacio   NUMBER(3);    -- Canvi identificador bbdd de la situaci
    nou_proveidor    VARCHAR2(12); -- Canvi codi proveidor
    nou_escaneig     VARCHAR2(50); -- Canvi URL amb la informaci escanejada      
    
    v_count          NUMBER(3);    -- Comptador
    v_declaracio     NUMBER(2);    -- Nmero de sentencia SQL

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.DESPESA_CAMBRA_M';
    log_entrada  := '<' || i_ibd || '> <' || i_data || '> <' || i_cambra || '> <' || 
        i_categoria || '> <' || i_import || '> <' || i_tipusIva || '> <' || i_descripcio || '> <' || 
        i_justificacio || '> <' || i_pagament || '> <' || i_proveidor || '> <' || i_escaneig ||'>';
    log_resultat := 'OK. MODIFICACI DE DESPESA GENERAL DE CAMBRA EFECTUAT';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    
    -- Comprovaci dels parmetres d'entrada
    IF (i_ibd IS NULL OR i_ibd = '') THEN RAISE exc_noDespesa; END IF;
    IF (i_data IS NULL OR i_data = '') AND
       (i_cambra IS NULL OR i_cambra = '' OR i_cambra = 0) AND
       (i_categoria IS NULL OR i_categoria = '' OR i_categoria = 0) AND
       (i_import IS NULL OR i_import = '' OR i_import = 0) AND
       (i_tipusIva IS NULL OR i_tipusIva = '' OR i_tipusIva = 0) AND
       (i_descripcio IS NULL OR i_descripcio = '') AND 
       (i_justificacio IS NULL OR i_justificacio = '') AND 
       (i_pagament IS NULL OR i_pagament = '') AND
       (i_situacio IS NULL OR i_situacio = '' OR i_situacio = 0) AND
       (i_proveidor IS NULL OR i_proveidor = '') AND
       (i_escaneig IS NULL OR i_escaneig = '')
    THEN RAISE exc_capModificacio; END IF;
      
    -- Inicialitzaci de les variables de la modificaci de dades

    nou_data         := null; 
    nou_any          := 0;
    nou_idCambra     := 0;
    nou_idCategoria  := 0;
    nou_import       := 0;
    nou_tipusIva     := 0;
    nou_descripcio   := '';
    nou_justificacio := '';
    nou_idPagament   := 0;
    nou_idSituacio   := 0;
    nou_proveidor    := '';
    nou_escaneig     := '';

    v_dataBaixa := null;
    v_count     := 0;
      
    IF(i_data IS NULL OR i_data = '') THEN 
        v_data := null;
        v_any  := 0;
    ELSE 
        v_data := TO_DATE(i_data); 
        v_any  := TO_NUMBER(TO_CHAR(v_data,'YYYY'));
    END IF;

    nou_data         := v_data;  
    nou_any          := v_any;
    nou_idCambra     := i_cambra;
    nou_idCategoria  := i_categoria;
    nou_import       := i_import;
    nou_tipusIva     := i_tipusIva;
    nou_descripcio   := UPPER(i_descripcio);        
    nou_justificacio := UPPER(i_justificacio);
    nou_idPagament   := i_pagament;
    nou_idSituacio   := i_situacio;
    nou_proveidor    := UPPER(i_proveidor);
    nou_escaneig     := LOWER(i_escaneig);
            
    -- Inicialitzaci de les variables de les dades actuals del membre a modificar
    ant_data         := null; 
    ant_any          := 0;
    ant_idCambra     := 0;
    ant_idCategoria  := 0;
    ant_import       := 0;
    ant_tipusIva     := 0;
    ant_descripcio   := '';
    ant_justificacio := '';
    ant_idPagament   := 0;
    ant_idSituacio   := 0;
    ant_proveidor    := '';
    ant_escaneig     := '';

    -- Comprovaci de l'existncia de la despesa i obtenci dels valors actuals
    v_declaracio := 1; 
    SELECT d.data_despesa, dc.cambra, dc.categoria, d.import_brut, d.tipus_iva, 
        UPPER(d.descripcio), UPPER(d.justificacio), d.pagament, d.situacio, 
        d.proveidor, LOWER(d.url_escaneig),
        TO_NUMBER(TO_CHAR(d.data_despesa,'YYYY'))
        INTO ant_data, ant_idCambra, ant_idCategoria, ant_import, ant_tipusIva, 
            ant_descripcio, ant_justificacio, ant_idPagament, ant_idSituacio,
            ant_proveidor, ant_escaneig, ant_any
        FROM dce.despesa d, dce.despesa_cambra dc 
        WHERE dc.despesa = d.ibd AND d.ibd = i_ibd;
        
    log_abans := 'Despesa:<' || i_ibd || '> Cambra:<' || ant_idCambra || 
        '> Categoria:<' || ant_idCategoria || '> Import:<' || ant_import || 
        '> tipus IVA: <' || ant_tipusIva || '> Descripci:<' || ant_descripcio || 
        '> Justificaci:<' || ant_justificacio || '> Pagament:<' || 
        ant_idPagament || '> Situaci:<' || ant_idSituacio || '> Provedor:<' ||
        ant_proveidor || '> Escaneig:<' || ant_escaneig || '>';

     -- Si no s'han indicat valors nous es mantenen els actuals
    IF (nou_data = '' OR nou_data IS NULL) THEN nou_data := ant_data; END IF;  
    IF (nou_any = 0) THEN nou_any := ant_any; END IF;    
    IF (nou_idCambra = '' OR nou_idCambra IS NULL OR nou_idCambra = 0) THEN 
        nou_idCambra := ant_idCambra; END IF;
    IF (nou_idCategoria = '' OR nou_idCategoria IS NULL OR nou_idCategoria = 0) THEN 
        nou_idCategoria := ant_idCategoria; END IF;
    IF (nou_import = '' OR nou_import IS NULL OR nou_import = 0) THEN 
        nou_import := ant_import; END IF;
    IF (nou_tipusIva = '' OR nou_tipusIva IS NULL OR nou_tipusIva = 0) THEN 
        nou_tipusIva := ant_tipusIva; END IF;
    IF (nou_descripcio = '' OR nou_descripcio IS NULL) THEN 
        nou_descripcio := ant_descripcio; END IF;
    IF (nou_justificacio = '' OR nou_justificacio IS NULL) THEN 
        nou_justificacio := ant_justificacio; END IF;
    IF (nou_idPagament = 0 OR nou_idPagament = '' OR nou_idPagament IS NULL) 
        THEN nou_idPagament := ant_idPagament; END IF;
    IF (nou_idSituacio = 0 OR nou_idSituacio = '' OR nou_idSituacio IS NULL) 
        THEN nou_idSituacio := ant_idSituacio; END IF;
    IF (nou_proveidor = '' OR nou_proveidor IS NULL) THEN 
        nou_proveidor := ant_proveidor; END IF;

    log_despres := 'Despesa:<' || i_ibd || '> Cambra:<' || nou_idCambra || 
        '> Categoria:<' || nou_idCategoria || '> Import:<' || nou_import || 
        '> tipus IVA: <' || nou_tipusIva || '> Descripci:<' || nou_descripcio || 
        '> Justificaci:<' || nou_justificacio || '> Pagament:<' || 
        nou_idPagament || '> Situaci:<' || nou_idSituacio || '> Provedor:<' ||
        nou_proveidor || '> Escaneig:<' || nou_escaneig || '>';

   -- Comprovacio de la cambra parlamentaria
    v_declaracio := 2; 
    IF (nou_idCambra <> ant_idCambra) THEN    
        SELECT data_baixa INTO v_dataBaixa FROM dce.cambra WHERE ibd = i_cambra;
        IF(v_dataBaixa IS NOT NULL AND v_data > v_dataBaixa) THEN 
            RAISE exc_dataSupBaixa; END IF;
        END IF;

    -- Comprovacio de la categoria
    IF (nou_idCategoria <> ant_idCategoria) THEN 
        SELECT COUNT(*) INTO v_count FROM dce.categoria_personal WHERE ibd = nou_idCategoria;
        IF(v_count = 0) THEN RAISE exc_noTrobaCategoria; END IF;
    END IF;
    
    -- Comprovacio de la forma de pagament
    IF (nou_idPagament <> ant_idPagament) THEN    
        SELECT COUNT(*) INTO v_count FROM dce.forma_pagament WHERE ibd = nou_idPagament;
        IF(v_count = 0) THEN RAISE exc_noTrobaPagament; END IF;
    END IF;
    
    -- Comprovacio de la situaci
    IF (nou_idSituacio <> ant_idSituacio) THEN    
        SELECT COUNT(*) INTO v_count FROM dce.situacio_despesa WHERE ibd = nou_idSituacio;
        IF(v_count = 0) THEN RAISE exc_noTrobaSituacio; END IF;
    END IF;

    -- Comprovaci del import, del tipus d'IVA i del provedor si la despesa > 30
    IF (nou_import < 0) THEN RAISE exc_importMenorZero; END IF;
    IF (nou_tipusIva < 0) THEN RAISE exc_tipusIvaMenorZero; END IF;
    IF (nou_proveidor IS NULL AND nou_import > 30) THEN RAISE exc_noProveidor; END IF;

    -- Execuci: Modificaci de la despesa
    UPDATE DCE.DESPESA  SET 
        DATA_DESPESA = nou_data,
        IMPORT_BRUT  = nou_import,
        TIPUS_IVA    = nou_tipusIva,
        DESCRIPCIO   = nou_descripcio,
        JUSTIFICACIO = nou_justificacio,
        PAGAMENT     = nou_idPagament,
        SITUACIO     = nou_idSituacio,
        PROVEIDOR    = nou_proveidor,
        URL_ESCANEIG = nou_escaneig
    WHERE IBD = i_ibd;
    
    -- Noms si canvia la cambra o la categoria cal actualitzar la taula DESPESA_CAMBRA
    IF (nou_idCambra <> ant_idCambra OR nou_idCategoria <> ant_idCategoria) THEN
     
        -- Execuci: Modificaci de la despesa general
        UPDATE DCE.DESPESA_CAMBRA  SET 
           CAMBRA       = nou_idCambra,
           CATEGORIA    = nou_idCategoria
        WHERE DESPESA = i_ibd;
                 
    END IF;

    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;     

    -- Actualitzaci de les taules estadstiques E1, E3, i E8    
    IF (ant_idCambra <> nou_idCambra) THEN 
        -- Si canvia la cambra cal actualitzar les estadstiques de l'antiga i de la nova
        DCE.EST.ESTADISTICA_01_INSERIR(ant_idCambra, o_rsp);
        DCE.EST.ESTADISTICA_01_INSERIR(nou_idCambra, o_rsp);
        DCE.EST.ESTADISTICA_03_INSERIR(ant_any, o_rsp);
        DCE.EST.ESTADISTICA_03_INSERIR(nou_any, o_rsp);
        DCE.EST.ESTADISTICA_08_INSERIR(ant_any, o_rsp);
        DCE.EST.ESTADISTICA_08_INSERIR(nou_any, o_rsp);
    
    ELSE  -- Si no canvia la cambra per canvia l'any o l'import
    
        -- Si canvia l'any cal canviar les estadstiques del antic i del nou
        IF (ant_any <> nou_any) THEN 
            DCE.EST.ESTADISTICA_01_INSERIR(nou_idCambra, o_rsp);
            DCE.EST.ESTADISTICA_03_INSERIR(ant_any, o_rsp);
            DCE.EST.ESTADISTICA_03_INSERIR(nou_any, o_rsp);
            DCE.EST.ESTADISTICA_08_INSERIR(ant_any, o_rsp);
            DCE.EST.ESTADISTICA_08_INSERIR(nou_any, o_rsp);
        ELSE  -- Si no canvia l'any per canvia l'import
            
            -- Si canvia l'import noms cal canviar l'estadstica actual
            IF (nou_import <> ant_import) THEN 
                DCE.EST.ESTADISTICA_01_INSERIR(nou_idCambra, o_rsp);
                DCE.EST.ESTADISTICA_03_INSERIR(ant_any, o_rsp);
                DCE.EST.ESTADISTICA_08_INSERIR(ant_any, o_rsp);
            END IF;
            -- Altrament no cal actualitzar cap estadstica
        END IF;
    END IF;    
         
    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;        
    
    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noDespesa THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT L''IDENTIFICADOR DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_capModificacio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT CAP CAMP A MODIFICAR DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_declaracio
      WHEN 1 THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT LA DESPESA DE LA CAMBRA PARLAMENTRIA. (IBD ' || i_ibd || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
      WHEN 2 THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT LA CAMBRA PARLAMENTRIA. (IBD ' || i_cambra || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    END CASE;
    
    WHEN exc_dataSupBaixa THEN
        log_resultat := 
            'ERROR: LA DATA DE LA DESPESA S SUPERIOR A LA DATA DE BAIXA DE LA CAMBRA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaCategoria THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA CATEGORIA GENERAL. (IBD ' || i_categoria || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);                  

    WHEN exc_noTrobaPagament THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA FORMA DE PAGAMENT. (IBD ' || i_pagament || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaSituacio THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA SITUACI. (IBD ' || i_situacio || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_importMenorZero THEN 
        log_resultat := 'ERROR: EL IMPORT NO POT SER NEGATIU. ' || i_import || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_tipusIvaMenorZero THEN 
        log_resultat := 'ERROR: EL TIPUS D''IVA NO POT SER NEGATIU. ' || i_tipusIva || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noProveidor THEN 
        log_resultat := 'ERROR: CAL INDICAR EL PROVEDOR SI LA DESPESA > 30 EUROS. ';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESA_CAMBRA_M;

---------------------------------------------------------------------------------------------------
-- *** 06. D E S P E S E S    P E R S O N A L S    D E     M E M B R E S    D E    C A M B R E S
---------------------------------------------------------------------------------------------------

---------------------------------------------------------------------------------------------------
-- *** P06A.ALTA DE DESPESA PERSONAL DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Insereix una nova despesa personal d'un membre d'una cambra en les taules DESPESA i 
   DESPESA_MEMBRE 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - L'existncia del membre de la cambra parlamentria
   - L'obtenci del partit poltic del membre en el moment d'emmagatzemar la despesa
   - L'existncia de la categoria personal
   - L'existncia de la forma de pagament
   - L'existncia de la situaci de despesa
   - Regla de negoci 2: El clcul del import del IVA i del import net es fa a nivell de valors 
     automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0
   - Regla de negoci 8; Que el provedor estigui informat si la despesa > 30 euros.
   - Regla de negoci 10: No es pot entrar despeses en membres donats de baixa si la data de 
     despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_MEMBRE_A (
    i_data         IN  DESPESA.DATA_DESPESA%TYPE,     -- Data de la despesa
    i_membre       IN  DESPESA_MEMBRE.MEMBRE%TYPE,    -- Codi identificador bbdd del/de la membre
    i_categoria    IN  DESPESA_MEMBRE.CATEGORIA%TYPE, -- Categoria de la despesa
    i_import       IN  DESPESA.IMPORT_BRUT%TYPE,      -- Import brut de la despesa
    i_tipusIva     IN  DESPESA.TIPUS_IVA%TYPE,        -- Tipus d'IVA de la  despesa
    i_descripcio   IN  DESPESA.DESCRIPCIO%TYPE,       -- Descripci de la despesa
    i_justificacio IN  DESPESA.JUSTIFICACIO%TYPE,     -- Justificaci de la despesa
    i_pagament     IN  DESPESA.PAGAMENT%TYPE,         -- Forma de pagament de la despesa
    i_situacio     IN  DESPESA.SITUACIO%TYPE,         -- Situaci de la despesa
    i_proveidor    IN  DESPESA.PROVEIDOR%TYPE,        -- Provedor 
    i_escaneig     IN  DESPESA.URL_ESCANEIG%TYPE,     -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2                       -- Resultat de l'execuci del procediment 
) IS

    -- Variables pel tractament d'excepcions
    exc_noData              EXCEPTION; -- No s'ha indicat el parmetre i_data
    exc_noMembre            EXCEPTION; -- No s'ha indicat el parmetre i_membre
    exc_dataSupBaixa        EXCEPTION; -- La data de baixa del membre s < que la de la despesa
    exc_noCategoria         EXCEPTION; -- No s'ha indicat el parmetre i_categoria
    exc_noImport            EXCEPTION; -- No s'ha indicat el parmetre i_import
    exc_noTipusIva          EXCEPTION; -- No s'ha indicat el parmetre i_tipusIva
    exc_noDescripcio        EXCEPTION; -- No s'ha indicat el parmetre i_descripcio
    exc_noJustificacio      EXCEPTION; -- No s'ha indicat ni i_estat ni i_justificacio
    exc_noPagament          EXCEPTION; -- No s'ha indicat el parmetre i_pagament
    exc_noSituacio          EXCEPTION; -- No s'haindicat el parmetre i_situacio
    exc_noTrobaCategoria    EXCEPTION; -- No s'ha trobat la categoria
    exc_noTrobaPagament     EXCEPTION; -- No s'ha trobat la forma de pagament
    exc_noTrobaSituacio     EXCEPTION; -- No s'ha trobat la situaci
    exc_importMenorZero     EXCEPTION; -- El import no pot ser negatiu
    exc_tipusIvaMenorZero   EXCEPTION; -- El tipus d'IVA no pot ser negatiu
    exc_noProveidor         EXCEPTION; -- Cal indicar el proveidor si la despesa > 30
 
    v_dataBaixa    DATE;          -- Data de baixa del membre de cambra parlamentria
    v_idDespesa    NUMBER(9);     -- Identificador bbdd de la despesa
    
    -- Variables per a la inserci de dades
    v_data         DATE;          -- Data de la despesa en format dd/mm/yyyy
    v_descripcio   VARCHAR2(50);  -- Descripci de la despesa
    v_justificacio VARCHAR2(50);  -- Justificaci de la despesa
    v_escaneig     VARCHAR2(50);  -- URL amb la informaci escanejada de la despesa
    v_count        NUMBER(3);     -- Comptador
    v_idPartit     NUMBER(3);     -- Identificador bbdd del partit poltic del/de la membre en el 
                                  -- moment d'emmagatzemar la despesa.
    v_idCambra     NUMBER(3);     -- Identificador bbdd de la cambra parlamentria a la que pertany 
                                  -- el membre. Pel registre en les estadstiques 
    v_any          NUMBER(4);     -- Any de la despesa pel registre en les estadstiques 

BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.DESPESA_MEMBRE_A';
    log_entrada  := '<' || i_data || '> <' || i_membre || '> <' || i_categoria || '> <' || 
        i_import || '> <' || i_tipusIva || '> <' || i_descripcio || '> <' || 
        i_justificacio || '> <' || i_pagament || '> <' || i_proveidor || '> <' || i_escaneig ||'>';
    log_resultat := 'OK. ALTA DE DESPESA PERSONAL DE MEMBRE DE CAMBRA EFECTUAT';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
    v_dataBaixa  := null;

    -- Comprovaci dels parmetres d'entrada
    IF (i_data IS NULL OR i_data = '') THEN RAISE exc_noData; END IF;
    IF (i_membre IS NULL OR i_membre = '' OR i_membre = 0) THEN RAISE exc_noMembre; END IF;
    IF (i_categoria IS NULL OR i_categoria = '') THEN RAISE exc_noCategoria; END IF;
    IF (i_import IS NULL OR i_import = '' OR i_import = 0) THEN RAISE exc_noImport; END IF;
    IF (i_tipusIva IS NULL OR i_tipusIva = '' OR i_tipusIva = 0) THEN RAISE exc_noTipusIva; END IF;
    IF (i_descripcio IS NULL OR i_descripcio = '') THEN RAISE exc_noDescripcio; END IF;
    IF (i_justificacio IS NULL OR i_justificacio = '') THEN RAISE exc_noJustificacio; END IF;
    IF (i_pagament IS NULL OR i_pagament = '' OR i_pagament = 0) THEN RAISE exc_noPagament; END IF;
    IF (i_situacio IS NULL OR i_situacio = '' OR i_situacio = 0) THEN RAISE exc_noSituacio; END IF;
    
    -- Inicialitzaci de les variables de la inserci de dades
    v_data         := TO_DATE(i_data);
    v_descripcio   := UPPER(i_descripcio);
    v_justificacio := UPPER(i_justificacio);
    v_escaneig     := LOWER(i_escaneig);
    v_count        := 0;
    v_idPartit := 0;
    v_idCambra     := 0;
    v_any          := TO_NUMBER(TO_CHAR(v_data,'YYYY'));
    
    -- Comprovacio del membre de cambra parlamentaria
    SELECT data_baixa, partit, cambra INTO v_dataBaixa, v_idPartit, v_idCambra 
        FROM dce.membre_cambra WHERE ibd = i_membre;
    IF(v_dataBaixa IS NOT NULL AND v_data > v_dataBaixa) THEN RAISE exc_dataSupBaixa; END IF;

    -- Comprovacio de la categoria
    SELECT COUNT(*) INTO v_count FROM dce.categoria_personal WHERE ibd = i_categoria;
    IF(v_count = 0) THEN RAISE exc_noTrobaCategoria; END IF;

    -- Comprovacio de la forma de pagament
    SELECT COUNT(*) INTO v_count FROM dce.forma_pagament WHERE ibd = i_pagament; 
    IF(v_count = 0) THEN RAISE exc_noTrobaPagament; END IF;

    -- Comprovacio de la situaci
    SELECT COUNT(*) INTO v_count FROM dce.situacio_despesa WHERE ibd = i_situacio;
    IF(v_count = 0) THEN RAISE exc_noTrobaSituacio; END IF;

    -- Comprovaci del import, del tipus d'IVA i del provedor si la despesa > 30
    IF (i_import < 0) THEN RAISE exc_importMenorZero; END IF;
    IF (i_tipusIva < 0) THEN RAISE exc_tipusIvaMenorZero; END IF;
    IF (i_import > 30 AND (i_proveidor IS NULL OR i_proveidor = '')) THEN 
        RAISE exc_noProveidor; END IF;

    -- Execuci: Alta de la despesa 
    INSERT INTO DCE.DESPESA (DATA_DESPESA, IMPORT_BRUT, TIPUS_IVA, DESCRIPCIO, 
        JUSTIFICACIO, PAGAMENT, SITUACIO, PROVEIDOR, URL_ESCANEIG)
        VALUES (v_data, i_import, i_tipusIva, v_descripcio, 
                v_justificacio, i_pagament, i_situacio, i_proveidor,i_escaneig);
 
    -- Execuci: Alta de la despesa personal amb el valor actual de la seqncia
    -- de despeses DES_SEQ_IBD.currval
    v_idDespesa := DES_SEQ_IBD.currval;    
    INSERT INTO DCE.DESPESA_MEMBRE (DESPESA, MEMBRE, PARTIT, CATEGORIA)
        VALUES (v_idDespesa, i_membre, v_idPartit, i_categoria);
 
    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;        

    -- Actualitzaci de les taules estadstiques E1, E2, E3, E4, E5, E6, E7 i E8
    DCE.EST.ESTADISTICA_01_INSERIR(v_idCambra, o_rsp);
    DCE.EST.ESTADISTICA_02_INSERIR(v_idCambra, v_any, o_rsp);
    DCE.EST.ESTADISTICA_03_INSERIR(v_any, o_rsp);
    DCE.EST.ESTADISTICA_04_INSERIR(i_membre, o_rsp);
    DCE.EST.ESTADISTICA_05_INSERIR(v_any, o_rsp);
    DCE.EST.ESTADISTICA_06_INSERIR(v_any, o_rsp);
    DCE.EST.ESTADISTICA_07_INSERIR(v_any, o_rsp);
    DCE.EST.ESTADISTICA_08_INSERIR(v_any, o_rsp);
            
    -- Control d'excepcions
    EXCEPTION   
    WHEN exc_noMembre THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL MEMBRE DE CAMBRA PARLAMENTRIA DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noData THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA DATA DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noCategoria THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA CATEGORIA DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noImport THEN
        log_resultat := 
            'ERROR: NO S''HAN INDICAT EL IMPORT DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTipusIva THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL TIPUS D''IVA DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noDescripcio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA DESCRIPCI DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noJustificacio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA JUSTIFICACI DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noPagament THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA FORMA DE PAGAMENT DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noSituacio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT LA SITUACI DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT EL MEMBRE DE CAMBRA. (IBD ' || i_membre || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_dataSupBaixa THEN
        log_resultat := 
            'ERROR: LA DATA DE LA DESPESA S SUPERIOR A LA DATA DE BAIXA DEL MEMBRE DE CAMBRA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaCategoria THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA CATEGORIA PERSONAL. (IBD ' || i_categoria || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);                  

    WHEN exc_noTrobaPagament THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA FORMA DE PAGAMENT. (IBD ' || i_pagament || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaSituacio THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA SITUACI. (IBD ' || i_situacio || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_importMenorZero THEN 
        log_resultat := 'ERROR: EL IMPORT NO POT SER NEGATIU. ' || i_import || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_tipusIvaMenorZero THEN 
        log_resultat := 'ERROR: EL TIPUS D''IVA NO POT SER NEGATIU. ' || i_tipusIva || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noProveidor THEN 
        log_resultat := 'ERROR: CAL INDICAR EL PROVEDOR SI LA DESPESA > 30 EUROS. ';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESA_MEMBRE_A;

---------------------------------------------------------------------------------------------------
-- *** P056M.MODIFICACI DE DESPESA PERSONAL DE MEMBRE DE CAMBRA PARLAMENTRIA
---------------------------------------------------------------------------------------------------
/* Modifica les dades d'una despesa personal d'un membre en les taules DESPESA i
   DESPESA_MEMBRE 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats que no sn opcionals estiguin informats
   - Cal indicar el identificador bbdd de la despesa i al menys un parmetre a modificar. 
   - L'existncia del membre de cambra parlamentria si s'indica
   - L'obtenci del partit poltic del membre en el moment d'emmagatzemar la despesa
   - Que la data de la despesa no sigui superior a la data de baixa del membre, si n'hi ha
   - L'existncia de la categoria personal, si s'indica
   - L'existncia de la forma de pagament, si s'indica
   - L'existncia de la situaci de despesa, si s'indica
   - Si s'indiquen els parmetres corresponents:
      - Regla de negoci 8; Que el provedor estigui informat si la despesa > 30 euros.
      - Regla de negoci 2: El clcul del import del IVA i del import net es fa a nivell de valors 
        automtics de les columnes de la base de dades. El import i el tipus d'IVA han de ser > 0
      - Regla de negoci 10: No es pot entrar despeses en membres donats de baixa si la data de 
        despesa s superior a la de baixa.
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_MEMBRE_M (
    i_ibd          IN  DESPESA.IBD%TYPE,              -- Codi identificador bbdd de la despesa
    i_data         IN  VARCHAR2,                      -- Data de la despesa en format 'DD/MM/YYYY' 
    i_membre       IN  DESPESA_MEMBRE.MEMBRE%TYPE,    -- Codi identificador bbdd del membre
    i_categoria    IN  DESPESA_MEMBRE.CATEGORIA%TYPE, -- Categoria de la despesa
    i_import       IN  DESPESA.IMPORT_BRUT%TYPE,      -- Import brut de la despesa
    i_tipusIva     IN  DESPESA.TIPUS_IVA%TYPE,        -- Tipus d'IVA de la  despesa
    i_descripcio   IN  DESPESA.DESCRIPCIO%TYPE,       -- Descripci de la despesa
    i_justificacio IN  DESPESA.JUSTIFICACIO%TYPE,     -- Justificaci de la despesa
    i_pagament     IN  DESPESA.PAGAMENT%TYPE,         -- Forma de pagament de la despesa
    i_situacio     IN  DESPESA.SITUACIO%TYPE,         -- Situaci de la despesa
    i_proveidor    IN  DESPESA.PROVEIDOR%TYPE,        -- Codi provedor 
    i_escaneig     IN  DESPESA.URL_ESCANEIG%TYPE,     -- URL amb la informaci escanejada
    o_rsp          OUT VARCHAR2                       -- Resultat de l'execuci del procediment 
) IS
   -- Variables pel tractament d'excepcions
    exc_noDespesa           EXCEPTION; -- No s'ha indicat el parmetre i_ibd
    exc_capModificacio      EXCEPTION; -- No s'ha indicat cap canvi
    exc_dataSupBaixa        EXCEPTION; -- La data de baixa del membre s < que la de la despesa
    exc_noTrobaCategoria    EXCEPTION; -- No s'ha trobat la categoria
    exc_noTrobaPagament     EXCEPTION; -- No s'ha trobat la forma de pagament
    exc_noTrobaSituacio     EXCEPTION; -- No s'ha trobat la situaci
    exc_importMenorZero     EXCEPTION; -- El import no pot ser negatiu
    exc_tipusIvaMenorZero   EXCEPTION; -- El tipus d'IVA no pot ser negatiu
    exc_noProveidor         EXCEPTION; -- Cal indicar el proveidor si la despesa > 30
 
    v_data           DATE;         -- Data de la despesa en format date
    v_dataBaixa      DATE;         -- Data de baixa del membre de cambra parlamentria
    v_idPartit       NUMBER(3);    -- Identificador bbdd del partit poltic del nou membre
                                   -- en el moment d'emmagatzemar la despesa.
    
    -- Variables per a les dades actuals de la despesa a modificar
    ant_data         DATE;         -- Data de baixa de la despesa
    ant_any          NUMBER(4);    -- Any de la despesa
    ant_idMembre     NUMBER(3);    -- Identificador del membre de cambra parlamentria
    ant_idPartit     NUMBER(3);    -- Identificador del partit poltic
    ant_idCambra     NUMBER(3);    -- Identificador de la cambra parlamentria
    ant_idCategoria  NUMBER(3);    -- Identificador de la categoria
    ant_import       NUMBER(12,2); -- Import brut de la despesa
    ant_tipusIva     NUMBER(5,2);  -- Tipus d'IVA de la  despesa
    ant_descripcio   VARCHAR2(50); -- Descripci de la despesa
    ant_justificacio VARCHAR2(50); -- Justificaci de la despesa
    ant_idPagament   NUMBER(3);    -- Identificador bbdd de la forma de pagament
    ant_idSituacio   NUMBER(3);    -- Identificador bbdd de la situaci
    ant_proveidor    VARCHAR2(12); -- Codi proveidor
    ant_escaneig     VARCHAR2(50); -- URL amb la informaci escanejada      
    
    -- Variables per a la modificaci de dades
    nou_data         DATE;         -- Canvi data de baixa de la despesa
    nou_any          NUMBER(4);    -- Canvi d'any de la despesa
    nou_idMembre     NUMBER(3);    -- Canvi identificador del membre de cambra
    nou_idPartit     NUMBER(3);    -- Canvi identificador del partit poltic
    nou_idCambra     NUMBER(3);    -- Canvi identificador de  la cambra parlamentria
    nou_idCategoria  NUMBER(3);    -- Canvi identificador de la categoria
    nou_import       NUMBER(12,2); -- Canvi import brut de la despesa
    nou_tipusIva     NUMBER(5,2);  -- Canvi tipus d'IVA de la  despesa
    nou_descripcio   VARCHAR2(50); -- Canvi descripci de la despesa
    nou_justificacio VARCHAR2(50); -- Canvi justificaci de la despesa
    nou_idPagament   NUMBER(3);    -- Canvi identificador bbdd de la forma de pagament
    nou_idSituacio   NUMBER(3);    -- Canvi identificador bbdd de la situaci
    nou_proveidor    VARCHAR2(12); -- Canvi codi proveidor
    nou_escaneig     VARCHAR2(50); -- Canvi URL amb la informaci escanejada      
    
    v_count          NUMBER(3);    -- Comptador
    v_idCambra       NUMBER(3);    -- Identif. bbdd de la cambra parlamentria a la que pertany 
                                   -- el membre. Pel registre en les estadstiques 
    v_any            NUMBER(4);    -- Any de la despesa pel registre en les estadstiques 
    v_declaracio     NUMBER(2);    -- Nmero declaraci sentncia SQL
    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces   := 'ABM.DESPESA_MEMBRE_M';
    log_entrada  := '<' || i_ibd || '> <' || i_data || '> <' || i_membre || '> <' || 
        i_categoria || '> <' || i_import || '> <' || 
        i_tipusIva || '> <' || i_descripcio || '> <' || i_justificacio || '> <' || 
        i_pagament || '> <' || i_proveidor || '> <' || i_escaneig ||'>';
    
    log_resultat := 'OK. MODIFICACI DE DESPESA PERSONAL DE MEMBRE DE CAMBRA EFECTUAT';
    log_abans    := '';
    log_despres  := '';
    log_error    := 'S';
    log_usuari   := USER;
    log_data     := SYSDATE;
    o_rsp        := log_resultat;
   
    -- Comprovaci dels parmetres d'entrada
    IF (i_ibd IS NULL OR i_ibd = '') THEN RAISE exc_noDespesa; END IF;
    IF (i_data IS NULL OR i_data = '') AND
       (i_membre IS NULL OR i_membre = '' OR i_membre = 0) AND
       (i_categoria IS NULL OR i_categoria = '' OR i_categoria = 0) AND
       (i_import IS NULL OR i_import = '' OR i_import = 0) AND
       (i_tipusIva IS NULL OR i_tipusIva = '' OR i_tipusIva = 0) AND
       (i_descripcio IS NULL OR i_descripcio = '') AND 
       (i_justificacio IS NULL OR i_justificacio = '') AND 
       (i_pagament IS NULL OR i_pagament = '') AND
       (i_situacio IS NULL OR i_situacio = '' OR i_situacio = 0) AND
       (i_proveidor IS NULL OR i_proveidor = '') AND
       (i_escaneig IS NULL OR i_escaneig = '')
    THEN RAISE exc_capModificacio; END IF;
      
    -- Inicialitzaci de les variables de la modificaci de dades
    nou_data         := null; 
    nou_any          := 0;
    nou_idMembre     := 0;
    nou_idPartit     := 0;
    nou_idCambra     := 0;
    nou_idCategoria  := 0;
    nou_import       := 0;
    nou_tipusIva     := 0;
    nou_descripcio   := '';
    nou_justificacio := '';
    nou_idPagament   := 0;
    nou_idSituacio   := 0;
    nou_proveidor    := '';
    nou_escaneig     := '';

    v_dataBaixa      := null;
    v_idPartit       := 0;
    v_count          := 0;
    v_declaracio     := 0;
    
    IF(i_data IS NULL OR i_data = '') THEN 
        v_data := null;
        v_any  := 0;
    ELSE 
        v_data := TO_DATE(i_data); 
        v_any  := TO_NUMBER(TO_CHAR(v_data,'YYYY'));
    END IF;

    nou_data         := v_data;
    nou_any          := v_any;
    nou_idMembre     := i_membre;
    nou_idCategoria  := i_categoria;
    nou_import       := i_import;
    nou_tipusIva     := i_tipusIva;
    nou_descripcio   := UPPER(i_descripcio);        
    nou_justificacio := UPPER(i_justificacio);
    nou_idPagament   := i_pagament;
    nou_idSituacio   := i_situacio;
    nou_proveidor    := UPPER(i_proveidor);
    nou_escaneig     := LOWER(i_escaneig);
            
    -- Inicialitzaci de les variables de les dades actuals del membre a modificar
    ant_data         := null; 
    ant_any          := 0;
    ant_idMembre     := 0;
    ant_idPartit     := 0;
    ant_idCambra     := 0;
    ant_idCategoria  := 0;
    ant_import       := 0;
    ant_tipusIva     := 0;
    ant_descripcio   := '';
    ant_justificacio := '';
    ant_idPagament   := 0;
    ant_idSituacio   := 0;
    ant_proveidor    := '';
    ant_escaneig     := '';
    
    -- Comprovaci de l'existncia de la despesa i obtenci dels valors actuals
    v_declaracio := 1;
    SELECT d.data_despesa, dm.membre, dm.partit, dm.categoria, d.import_brut, 
        d.tipus_iva, UPPER(d.descripcio), UPPER(d.justificacio), d.pagament, 
        d.situacio, d.proveidor, LOWER(d.url_escaneig),m.cambra,
        TO_NUMBER(TO_CHAR(d.data_despesa,'YYYY'))
    INTO ant_data, ant_idMembre, ant_idPartit, ant_idCategoria, ant_import, 
         ant_tipusIva, ant_descripcio, ant_justificacio, ant_idPagament, 
         ant_idSituacio, ant_proveidor, ant_escaneig, ant_idCambra, ant_any
    FROM dce.despesa d, dce.despesa_membre dm, dce.membre_cambra m
    WHERE dm.membre = m.ibd AND dm.despesa = d.ibd AND d.ibd = i_ibd;
        
    log_abans := 'Despesa:<' || i_ibd || '> Data:<' || ant_data || 
        '> Membre:<' || ant_idMembre ||
        '> Partit:<' || ant_idPartit || '> Cambra:<' || ant_idCambra || 
        '> Categoria:<' || ant_idCategoria || 
        '> Import:<' || ant_import || '> tipus IVA: <' || ant_tipusIva || 
        '> Descripci:<' || ant_descripcio || '> Justificaci:<' || 
        ant_justificacio || '> Pagament:<' || ant_idPagament || '> Situaci:<' || 
        ant_idSituacio || '> Provedor:<' || ant_proveidor || '> Escaneig:<' || 
        ant_escaneig || '>';

     -- Si no s'han indicat valors nous es mantenen els actuals
    IF (nou_data = '' OR nou_data IS NULL) THEN nou_data := ant_data; END IF;
    IF (nou_any = 0) THEN nou_any := ant_any; END IF;
    IF (nou_idMembre = '' OR nou_idMembre IS NULL OR nou_idMembre = 0) THEN 
        nou_idMembre := ant_idMembre; END IF;
    IF (nou_idPartit = '' OR nou_idPartit IS NULL OR nou_idPartit = 0) THEN 
        nou_idPartit := ant_idPartit; END IF;
    IF (nou_idCambra = '' OR nou_idCambra IS NULL OR nou_idCambra = 0) THEN 
        nou_idCambra := ant_idCambra; END IF;
    IF (nou_idCategoria = '' OR nou_idCategoria IS NULL OR nou_idCategoria = 0) THEN 
        nou_idCategoria := ant_idCategoria; END IF;
    IF (nou_import = '' OR nou_import IS NULL OR nou_import = 0) THEN 
        nou_import := ant_import; END IF;
    IF (nou_tipusIva = '' OR nou_tipusIva IS NULL OR nou_tipusIva = 0) THEN 
        nou_tipusIva := ant_tipusIva; END IF;
    IF (nou_descripcio = '' OR nou_descripcio IS NULL) THEN 
        nou_descripcio := ant_descripcio; END IF;
    IF (nou_justificacio = '' OR nou_justificacio IS NULL) THEN 
        nou_justificacio := ant_justificacio; END IF;
    IF (nou_idPagament = 0 OR nou_idPagament = '' OR nou_idPagament IS NULL) 
        THEN nou_idPagament := ant_idPagament; END IF;
    IF (nou_idSituacio = 0 OR nou_idSituacio = '' OR nou_idSituacio IS NULL) 
        THEN nou_idSituacio := ant_idSituacio; END IF;
    IF (nou_proveidor = '' OR nou_proveidor IS NULL) THEN 
        nou_proveidor := ant_proveidor; END IF;

    log_despres := 'Despesa:<' || i_ibd || '> Data:<' || nou_data || 
        '> Membre:<' || nou_idMembre ||
        '> Partit:<' || nou_idPartit || '> Cambra:<' || nou_idCambra ||
        '> Categoria:<' || nou_idCategoria || '> Import:<' || nou_import || 
        '> tipus IVA: <' || nou_tipusIva || '> Descripci:<' || nou_descripcio || 
        '> Justificaci:<' || nou_justificacio || '> Pagament:<' || 
        nou_idPagament || '> Situaci:<' || nou_idSituacio || '> Provedor:<' ||
        nou_proveidor || '> Escaneig:<' || nou_escaneig || '>';

   -- Comprovacio del membre de cambra parlamentaria
   -- Obt el partit poltic del nou membre en aquest moment
    v_declaracio := 2;
    IF (nou_idMembre <> ant_idMembre) THEN    
        SELECT data_baixa, partit, cambra INTO v_dataBaixa, nou_idPartit, nou_idCambra
           FROM MEMBRE_CAMBRA WHERE IBD = nou_idMembre;
        IF(v_dataBaixa IS NOT NULL AND v_data > v_dataBaixa) THEN 
            RAISE exc_dataSupBaixa; END IF;
	END IF;

    -- Comprovacio de la categoria
    IF (nou_idCategoria <> ant_idCategoria) THEN 
        SELECT COUNT(*) INTO v_count FROM dce.categoria_personal WHERE ibd = nou_idCategoria;
        IF(v_count = 0) THEN RAISE exc_noTrobaCategoria; END IF;
    END IF;
    
    -- Comprovacio de la forma de pagament
    IF (nou_idPagament <> ant_idPagament) THEN    
        SELECT COUNT(*) INTO v_count FROM dce.forma_pagament WHERE ibd = nou_idPagament;
        IF(v_count = 0) THEN RAISE exc_noTrobaPagament; END IF;
    END IF;
    
    -- Comprovacio de la situaci
    IF (nou_idSituacio <> ant_idSituacio) THEN    
        SELECT COUNT(*) INTO v_count FROM dce.situacio_despesa WHERE ibd = nou_idSituacio;
        IF(v_count = 0) THEN RAISE exc_noTrobaSituacio; END IF;
    END IF;

    -- Comprovaci del import, del tipus d'IVA i del provedor si la despesa > 30
    IF (nou_import < 0) THEN RAISE exc_importMenorZero; END IF;
    IF (nou_tipusIva < 0) THEN RAISE exc_tipusIvaMenorZero; END IF;
    IF (nou_proveidor IS NULL AND nou_import > 30) THEN RAISE exc_noProveidor; END IF;

    -- Execuci: Modificaci de la despesa
    UPDATE DCE.DESPESA  SET 
        DATA_DESPESA = nou_data,
        IMPORT_BRUT  = nou_import,
        TIPUS_IVA    = nou_tipusIva,
        DESCRIPCIO   = nou_descripcio,
        JUSTIFICACIO = nou_justificacio,
        PAGAMENT     = nou_idPagament,
        SITUACIO     = nou_idSituacio,
        PROVEIDOR    = nou_proveidor,
        URL_ESCANEIG = nou_escaneig
    WHERE IBD = i_ibd;

    -- Execuci: Modificaci de la despesa personal
    UPDATE DCE.DESPESA_MEMBRE  SET 
        MEMBRE       = nou_idMembre,
        PARTIT       = nou_idPartit,
        CATEGORIA    = nou_idCategoria
    WHERE DESPESA = i_ibd;

    -- Execuci correcte del procediment
    log_error := 'N';
    PRO_LOG_PROCEDIMENT
      (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
    COMMIT;        

    -- Actualitzaci de les taules estadstiques E1, E2, E3, E4, E5, E6, E7 i E8
    -- Aquesta actualitzaci tamb podria fer-se a nivell de disparador de base de dades
    IF (ant_idMembre <> nou_idMembre) THEN

    -- Si canvia el membre cal actualitzar les estadstiques del antic i del nou
        DCE.EST.ESTADISTICA_04_INSERIR(ant_idMembre, o_rsp);                
        DCE.EST.ESTADISTICA_04_INSERIR(nou_idMembre, o_rsp);                
    END IF;
    IF (ant_idCambra <> nou_idCambra) THEN -- Si en canviar el membre, canvia la cambra

        -- Si canvia la cambra cal actualitzar les estadstiques de l'antiga i de la nova
        DCE.EST.ESTADISTICA_01_INSERIR(ant_idCambra, o_rsp);
        DCE.EST.ESTADISTICA_01_INSERIR(nou_idCambra, o_rsp);
        DCE.EST.ESTADISTICA_02_INSERIR(ant_idCambra, ant_any, o_rsp);
        DCE.EST.ESTADISTICA_02_INSERIR(nou_idCambra, nou_any, o_rsp);
        DCE.EST.ESTADISTICA_03_INSERIR(ant_any, o_rsp);
        IF (ant_any <> nou_any) THEN DCE.EST.ESTADISTICA_03_INSERIR(nou_any, o_rsp); END IF;
        DCE.EST.ESTADISTICA_05_INSERIR(ant_any, o_rsp);
        IF (ant_any <> nou_any) THEN DCE.EST.ESTADISTICA_05_INSERIR(nou_any, o_rsp); END IF;
        DCE.EST.ESTADISTICA_06_INSERIR(ant_any, o_rsp);
        IF (ant_any <> nou_any) THEN DCE.EST.ESTADISTICA_06_INSERIR(nou_any, o_rsp); END IF;
        DCE.EST.ESTADISTICA_07_INSERIR(ant_any, o_rsp);
        IF (ant_any <> nou_any) THEN DCE.EST.ESTADISTICA_07_INSERIR(nou_any, o_rsp); END IF;
        DCE.EST.ESTADISTICA_08_INSERIR(ant_any, o_rsp);
        IF (ant_any <> nou_any) THEN DCE.EST.ESTADISTICA_08_INSERIR(nou_any, o_rsp); END IF;
        
    ELSE  -- Si no canvia la cambra per canvia l'any o l'import
    
        -- Si canvia l'any cal canviar les estadstiques del antic i del nou
        IF (ant_any <> nou_any) THEN 
            DCE.EST.ESTADISTICA_01_INSERIR(ant_idCambra, o_rsp);
            DCE.EST.ESTADISTICA_01_INSERIR(nou_idCambra, o_rsp);
            DCE.EST.ESTADISTICA_02_INSERIR(ant_idCambra, ant_any, o_rsp);
            DCE.EST.ESTADISTICA_02_INSERIR(ant_idCambra, nou_any, o_rsp);
            DCE.EST.ESTADISTICA_03_INSERIR(ant_any, o_rsp);
            DCE.EST.ESTADISTICA_03_INSERIR(nou_any, o_rsp);
            DCE.EST.ESTADISTICA_05_INSERIR(ant_any, o_rsp);
            DCE.EST.ESTADISTICA_05_INSERIR(nou_any, o_rsp);
            DCE.EST.ESTADISTICA_06_INSERIR(ant_any, o_rsp);
            DCE.EST.ESTADISTICA_06_INSERIR(nou_any, o_rsp);
            DCE.EST.ESTADISTICA_07_INSERIR(ant_any, o_rsp);
            DCE.EST.ESTADISTICA_07_INSERIR(nou_any, o_rsp);            
            DCE.EST.ESTADISTICA_08_INSERIR(ant_any, o_rsp);
            DCE.EST.ESTADISTICA_08_INSERIR(nou_any, o_rsp);            

        ELSE  -- Si no canvia l'any per canvia l'import
            
            -- Si canvia l'import noms cal canviar l'estadstica actual
            IF (nou_import <> ant_import) THEN 
                DCE.EST.ESTADISTICA_01_INSERIR(ant_idCambra, o_rsp);
                DCE.EST.ESTADISTICA_02_INSERIR(ant_idCambra, ant_any, o_rsp);
                DCE.EST.ESTADISTICA_03_INSERIR(ant_any, o_rsp);
                DCE.EST.ESTADISTICA_05_INSERIR(ant_any, o_rsp);
                DCE.EST.ESTADISTICA_06_INSERIR(ant_any, o_rsp);
                DCE.EST.ESTADISTICA_07_INSERIR(ant_any, o_rsp);                
                DCE.EST.ESTADISTICA_08_INSERIR(ant_any, o_rsp);                
            END IF;
            -- Altrament no cal actualitzar cap estadstica
        END IF;
    END IF;
    
    -- Control d'excepcions
    EXCEPTION    
    WHEN exc_noDespesa THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT L''IDENTIFICADOR DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_capModificacio THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT CAP CAMP A MODIFICAR DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
    CASE v_declaracio
      WHEN 1 THEN
         log_resultat := 
             'ERROR: NO S''HA TROBAT LA DESPESA DE MEMBRE DE CAMBRA. (IBD ' || i_ibd || ')';
         o_rsp := log_resultat;
         PRO_LOG_PROCEDIMENT
         (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
      WHEN 2 THEN
         log_resultat := 
             'ERROR: NO S''HA TROBAT LA DESPESA DEL/DE LA MEMBRE DE CAMBRA.' ||
             ' (IBD ' || nou_idMembre || ')';
         o_rsp := log_resultat;
         PRO_LOG_PROCEDIMENT
         (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);    
    END CASE;
    
    WHEN exc_dataSupBaixa THEN
        log_resultat := 
            'ERROR: LA DATA DE LA DESPESA S SUPERIOR A LA DATA DE BAIXA DEL MEMBRE DE CAMBRA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaCategoria THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA CATEGORIA PERSONAL. (IBD ' || i_categoria || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);                  

    WHEN exc_noTrobaPagament THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA FORMA DE PAGAMENT. (IBD ' || i_pagament || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTrobaSituacio THEN 
        log_resultat := 'ERROR: NO S''HA TROBAT LA SITUACI. (IBD ' || i_situacio || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_importMenorZero THEN 
        log_resultat := 'ERROR: EL IMPORT NO POT SER NEGATIU. ' || i_import || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_tipusIvaMenorZero THEN 
        log_resultat := 'ERROR: EL TIPUS D''IVA NO POT SER NEGATIU. ' || i_tipusIva || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noProveidor THEN 
        log_resultat := 'ERROR: CAL INDICAR EL PROVEDOR SI LA DESPESA > 30 EUROS. ';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESA_MEMBRE_M;

---------------------------------------------------------------------------------------------------
-- *** P05/06B.BAIXA DE DESPESA 
---------------------------------------------------------------------------------------------------
/* Elimina una despesa en les taules DESPESA i DESPESA_CAMBRA o DESPESA_MEMBRE 
---------------------------------------------------------------------------------------------------
Comprovacions i altres accions que fa:
   - Que els parmetres passats estiguin informats
   - Regla de negoci 11: Actualitza les dades en les taules estadstiques
   - Regla de negoci 12: L'execuci del procediment es registra en la taula log_Procediment.
-------------------------------------------------------------------------------------------------*/
PROCEDURE DESPESA_B (
    i_despesa  IN  DESPESA.IBD%TYPE,   -- Codi identificador bbdd de la despesa
    i_tipus    IN  CHAR,               -- Indicador del tipus de despesa: C=Cambra, M=Membre
    o_rsp      OUT VARCHAR2            -- Resultat de l'execuci del procediment 
) IS
   -- Variables pel tractament d'excepcions
    exc_noDespesa      EXCEPTION;  -- No s'ha indicat el parmetre i_despesa
    exc_noTipus        EXCEPTION;  -- No s'ha indicat el parmetre i_tipus
    exc_tipusErroni    EXCEPTION;  -- No s'ha indicat C o M a i_tipus

    v_idDespesa        NUMBER(9);  -- Identificador bbdd de la despesa a eliminar
    
    -- Variables per a les dades actuals de la despesa a eliminar
    ant_data         DATE;         -- Data de la despesa
    ant_any          NUMBER(4);    -- Any de la despesa 
    ant_ibd          NUMBER(9);    -- Identificador bbdd de la despesa
    ant_idCambra     NUMBER(3);    -- Identificador bbdd de la cambra parlamentria
    ant_idMembre     NUMBER(3);    -- Identificador bbdd del membre de cambra parlamentria
    ant_idPartit     NUMBER(3);    -- Identificador bbdd del partit poltic
    ant_idCategoria  NUMBER(3);    -- Identificador bbdd de la categoria de la despesa
    ant_idSituacio   NUMBER(3);    -- Identificador bbdd de la situaci de la despesa
    ant_idPagament   NUMBER(3);    -- Identificador bbdd de la forma de pagament de la despesa
    ant_importBrut   NUMBER(12,2); -- Import brut de la despesa
    ant_tipusIva     NUMBER(5,2);  -- Tipus d'IVA de la despesa
    ant_importIva    NUMBER(12,2); -- Import del IVA de la despesa
    ant_importNet    NUMBER(12,2); -- Import net de la despesa 
    ant_descripcio   VARCHAR2(50); -- Descripci de la despesa
    ant_justificacio VARCHAR2(50); -- Justificaci de la despesa
    ant_proveidor    VARCHAR2(12); -- Provedor de la despesa
    ant_escaneig     VARCHAR2(50); -- URL amb la informaci escanejada de documents de la despesa 
  	    
BEGIN
    -- Inicialitzaci de les variables del tractament d'excepcions
    log_proces       := 'ABM.DESPESA_B';
    log_entrada      := '<' || i_despesa || '> <' || i_tipus || '>';
    log_abans        := '';
    log_despres      := '';    
    log_error        := 'S';
    log_usuari       := USER;
    log_data         := SYSDATE;
    o_rsp            := 'OK';
    v_idDespesa      := 0;

    -- Inicialitzaci de variables per a les dades actuals de la despesa a eliminar
    ant_data         := null;
    ant_any          := 0;
    ant_ibd          := 0;
    ant_idCambra     := 0;
    ant_idMembre     := 0;
    ant_idPartit     := 0;
    ant_idCategoria  := 0;
    ant_idSituacio   := 0;
    ant_idPagament   := 0;
    ant_importBrut   := 0;
    ant_tipusIva     := 0;
    ant_importIva    := 0;
    ant_importNet    := 0;
    ant_descripcio   := '';
    ant_justificacio := '';
    ant_proveidor    := '';
    ant_escaneig     := '';

    -- Comprovaci dels parmetres d'entrada
    IF (i_despesa IS NULL OR i_despesa = '') THEN RAISE exc_noDespesa; END IF;
    IF (i_tipus IS NULL OR i_tipus = '') THEN RAISE exc_noTipus; END IF;
    If (upper(i_tipus) <> 'C' AND upper(i_tipus) <> 'M') THEN RAISE exc_tipusErroni; END IF;
    
    IF (upper(i_tipus) = 'C') THEN
        -- Comprovaci de l'existncia de la despesa general
        SELECT d.ibd, d.data_despesa, dc.cambra, dc.categoria, d.situacio, d.pagament, 
            d.import_brut, d.tipus_iva, d.import_iva, d.import_net, d.descripcio, 
            d.justificacio, d.proveidor, d.url_escaneig, 
            TO_NUMBER(TO_CHAR(d.data_despesa,'yyyy'))
        INTO ant_ibd, ant_data, ant_idCambra, ant_idCategoria, ant_idSituacio,ant_idPagament,
             ant_importBrut, ant_tipusIva, ant_importIva, ant_importNet, ant_descripcio,
             ant_justificacio, ant_proveidor,ant_escaneig, ant_any
        FROM dce.despesa d, dce.despesa_cambra dc 
        WHERE dc.despesa = d.ibd AND d.ibd = i_despesa;
             
        v_idDespesa := ant_ibd;
        log_abans := 'Data:<' || to_char(ant_data,'dd/mm/yyyy') || '> Ibd:<' || ant_ibd || 
            '> id.Cambra:<'   || ant_idCambra || '> id.Categoria:<' || ant_idCategoria ||
            '> id.Situacio:<' || ant_idSituacio || '> id.Pagament:<' || ant_idPagament ||
            '> Import Brut:<' || ant_importBrut ||'> tipus IVA:<' || ant_tipusIva ||
            '> Import IVA:<'  || ant_importIva || '> Import net:<' || ant_importNet ||
            '> Descripci:<'  || ant_descripcio || '> Justificaci:<' || ant_justificacio ||
            '> Justificaci;<' || ant_justificacio ||'> Provedor:<' || ant_proveidor ||
            '> URL Escaneig:<' || ant_escaneig || '>';
            
        -- Execuci: Baixa de la despesa general
        DELETE FROM DCE.DESPESA_CAMBRA WHERE DESPESA = i_despesa;  
        
        -- Execuci: Baixa de la despesa
        DELETE FROM DCE.DESPESA WHERE IBD = i_despesa;
        
        -- Execuci correcte del procediment
        log_error := 'N';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
        COMMIT;
  
        -- Actualitzaci de les taules estadstiques E1, E3 i E8       
        DCE.EST.ESTADISTICA_01_INSERIR(ant_idCambra, o_rsp);                
        DCE.EST.ESTADISTICA_03_INSERIR(ant_any, o_rsp);
        DCE.EST.ESTADISTICA_08_INSERIR(ant_any, o_rsp);                
        
        log_resultat := 'OK. ELIMINACI DE DESPESA EFECTUADA';
    ELSE
 
       -- Comprovaci de l'existncia de la despesa personal
        SELECT d.ibd, d.data_despesa, dm.membre, dm.partit, dm.categoria, d.situacio, d.pagament, 
            d.import_brut, d.tipus_iva, d.import_iva, d.import_net, d.descripcio, d.justificacio, 
            d.proveidor, d.url_escaneig, m.cambra, to_number(to_char(d.data_despesa,'yyyy'))
            INTO ant_ibd, ant_data, ant_idMembre, ant_idPartit, ant_idCategoria, ant_idSituacio,
            ant_idPagament, ant_importBrut, ant_tipusIva, ant_importIva, ant_importNet, 
            ant_descripcio, ant_justificacio, ant_proveidor,ant_escaneig, ant_idCambra, ant_any
        FROM dce.despesa d, dce.despesa_membre dm, dce.membre_cambra m
        WHERE dm.membre = m.ibd AND dm.despesa = d.ibd AND d.ibd = i_despesa;
             
        v_idDespesa := ant_ibd;
        log_abans := 'Data:<' || to_char(ant_data,'dd/mm/yyyy') || '> Ibd:<' || ant_ibd || 
            '> id.Membre:<' || ant_idMembre || '> id.Partit:<' || ant_idPartit || 
            '> id.Cambra:<' || ant_idCambra ||
            '> id.Categoria:<' || ant_idCategoria || '> id.Situacio:<' || ant_idSituacio || 
            '> id.Pagament:<' || ant_idPagament || '> Import Brut:<' || ant_importBrut || 
            '> tipus IVA:<' || ant_tipusIva || '> Import IVA:<' || ant_importIva || 
            '> Import net:<' || ant_importNet || '> Descripci:<' || ant_descripcio || 
            '> Justificaci:<' || ant_justificacio || '> Justificaci;<' || ant_justificacio ||
            '> Provedor:<' || ant_proveidor || '> URL Escaneig:<' || ant_escaneig || '>';
             
        -- Execuci: Baixa de la despesa personal
        DELETE FROM DCE.DESPESA_MEMBRE WHERE DESPESA = i_despesa;
        -- Actualitzaci de les taules estadstiques relacionades amb membres

        -- Execuci: Baixa de la despesa 
        DELETE FROM DCE.DESPESA WHERE IBD = i_despesa;
        
        -- Execuci correcte del procediment
        log_error := 'N';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);
        COMMIT;
    
        -- Actualitzaci de les taules estadstiques E1, E2, E3, E4, E5, E6, E7 i E8
        DCE.EST.ESTADISTICA_01_INSERIR(ant_idCambra, o_rsp);                        
        DCE.EST.ESTADISTICA_02_INSERIR(ant_idCambra, ant_any, o_rsp);
        DCE.EST.ESTADISTICA_03_INSERIR(ant_any, o_rsp);
        DCE.EST.ESTADISTICA_04_INSERIR(ant_idMembre, o_rsp);                
        DCE.EST.ESTADISTICA_05_INSERIR(ant_any, o_rsp);
        DCE.EST.ESTADISTICA_06_INSERIR(ant_any, o_rsp);
        DCE.EST.ESTADISTICA_07_INSERIR(ant_any, o_rsp);
        DCE.EST.ESTADISTICA_08_INSERIR(ant_any, o_rsp);                

        log_resultat := 'OK. ELIMINACI DE DESPESA PERSONAL DE MEMBRE DE CAMBRA EFECTUADA';   
    END IF;
                
    -- Control d'excepcions
    EXCEPTION   
    WHEN exc_noDespesa THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL IDENTIFICADOR DE LA DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_noTipus THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT EL TIPUS DE DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN exc_tipusErroni THEN
        log_resultat := 
            'ERROR: NO S''HA INDICAT C (DE CAMBRA) NI M (DE MEMBRE) PEL TIPUS DE DESPESA';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN NO_DATA_FOUND THEN
        log_resultat := 
            'ERROR: NO S''HA TROBAT LA DESPESA (IBD ' || i_despesa || ')';
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

    WHEN OTHERS THEN
        log_resultat := TRIM('ERROR ' || SUBSTR(SQLERRM,1,250));
        ROLLBACK;
        o_rsp := log_resultat;
        PRO_LOG_PROCEDIMENT
        (log_proces,log_entrada,log_resultat,log_abans,log_despres,log_error,log_usuari,log_data);

END DESPESA_B;

---------------------------------------------------------------------------------------------------
END;
/ 
